MODULE LAUNCH
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! MODULE: LAUNCH                                   !!
    !!                                                  !!
    !! Last: Feb 23 2018                                !!  
    !!                                                  !!
    !! Changes:                                         !!
    !! >> codes wrapped in module                       !!  
    !! >> 2018/02/23                                    !!
    !!    regime switching version                      !! 
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    
    !!!!!!!!!!!!!!!!!!!!!!!!
    !!   USE STATEMENTS   !!
    !!!!!!!!!!!!!!!!!!!!!!!!    
    USE PARAMETERS
    USE GLOBAL
    USE PROCESS
    USE AUXILIARY
    USE UTILITIES
    USE LAPACK95
    USE IFPORT
    USE OMP_LIB
    
    
    IMPLICIT NONE  

CONTAINS


SUBROUTINE LAUNCHCOMPUTEMODEL(trace_test)

    ! local vars
    REAL(DP)            ::  capsule_time_start, capsule_time_stop
    INTEGER, INTENT(IN) ::  trace_test
    
    CALL CREATE_GRIDS(0)
    CALL CREATE_SHOCKS(0)
    CALL CREATE_AUTVALUE(0)
        
    IF (readguess==1) THEN
        CALL READ_GUESS(0)
    END IF
    
    CALL create_guess(trace_test)
        
    WRITE (*,*) 'compute model (main)...'
    
    capsule_time_start    = DCLOCK() 
    CALL COMPUTEMODEL_MAIN
    capsule_time_stop     = DCLOCK()     
            
END SUBROUTINE LAUNCHCOMPUTEMODEL 

SUBROUTINE LAUNCHSIMULATEMODEL(trace_test)

    ! local vars
    REAL(DP)            ::  capsule_time_start, capsule_time_stop
    INTEGER, INTENT(IN) ::  trace_test
    
    CALL CREATE_GRIDS(0)
    CALL CREATE_SHOCKS(0)
    CALL CREATE_AUTVALUE(0)
    
    CALL READ_GUESS(0)
    CALL CREATE_GUESS(0)
    
    WRITE (*,*) 'simulate model (main)...'
    capsule_time_start    = DCLOCK() 
    CALL SIMULATEMODEL_MAIN
    capsule_time_stop     = DCLOCK()     
            
END SUBROUTINE LAUNCHSIMULATEMODEL 

SUBROUTINE READ_GUESS(trace_test)
    
    !! subroutine parameters
    INTEGER, INTENT(IN) ::  trace_test

    !! subroutine var
	CHARACTER (LEN = 80)    :: ierror
	INTEGER 				:: istat    
    INTEGER                 :: ios 
    CHARACTER (LEN=80)      :: inidir
    CHARACTER (LEN=80)      :: inifile
    CHARACTER (LEN=7)       :: cfg_str      
    
    !! begin
    WRITE (*,*) 'read_guess...'
       
    ! generate initial guess folder name
    WRITE (UNIT=cfg_str, FMT='(I<sfx>.<sfx>)') config
    inidir = TRIM(folder_ini) // TRIM(cfgpfx) // '_' // TRIM(cfg_str) // '/'   
    
200 FORMAT (' ', 'Read error: ', A, ' IOMSG = ', A )    

    ! READ value function -- chi value
    inifile = TRIM(inidir) // TRIM('chi.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) chi
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
    
    ! READ value function -- value function v
    inifile = TRIM(inidir) // TRIM('v.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) v
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)  
    
    ! READ value function -- value function vdef
    inifile = TRIM(inidir) // TRIM('vdef.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) vdef
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
        
    ! READ value function -- value function no default vnd
    inifile = TRIM(inidir) // TRIM('vnd.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) vnd
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
    
    ! READ price schedule - price schedule
    inifile = TRIM(inidir) // TRIM('p.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) p
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)    
    
    ! READ equilibrium prices - psav
    inifile = TRIM(inidir) // TRIM('psav.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) psav
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
    
    ! READ equilibrium prices -- psavnd
    inifile = TRIM(inidir) // TRIM('psavnd.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) psavnd
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)

    ! READ equilibrium prices -- def
    inifile = TRIM(inidir) // TRIM('def.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) def
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
    
    ! READ equilibrium prices -- pdef
    inifile = TRIM(inidir) // TRIM('pdef.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) pdef
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
        
    ! READ savings policy - sav
    inifile = TRIM(inidir) // TRIM('sav.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) sav
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)         

    ! READ equilibrium prices -- savnd
    inifile = TRIM(inidir) // TRIM('savnd.out')
    
    OPEN (UNIT=9, FILE=inifile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
	
    IF ( istat == 0 ) THEN
        READ (UNIT=9,FMT=*) savnd
	ELSE
        WRITE (*, 200) inifile, TRIM(ierror)
        readguess = 0
    END IF
    
    CLOSE (UNIT=9)
    
END SUBROUTINE READ_GUESS     

SUBROUTINE CREATE_GRIDS(trace_test)
    
    INTEGER, INTENT(IN) ::  trace_test
    
    WRITE (*,*) 'create_grids...'
    
    !! Construct bond grids
    !CALL bondgrid(bmin,bmax)    
    CALL BONDGRID_PIVOT(bmin,bmax,bpvt,apvt)    
    
    !CALL BPRIMEGRID_PIVOT(bmin,bmax,bpvt,apvt)  
    CALL BPRIMEGRID_PIVOT(bmin,bmax)    
    
    ! chi grid
    CALL CREATEGRID(chigrid, N_chi, chi_min, chi_min, chi_max, chi_pwr)

    ! create bprime bounds
    DO i_b=1,ngpb
        
        i_bp_min(i_b) = MAXVAL(MINLOC( ABS( (onedp-delta)*b(i_b) + issue_min    &
                                            - bprime                            &
                                        ) ))
            
        i_bp_max(i_b) = MINVAL(MINLOC( ABS( (onedp-delta)*b(i_b) + issue_max    &
                                            - bprime                            &
                                        ) ))
    END DO
    
    
    !! save bond grid
    !OPEN (unit=9, file="b.out", position="rewind")
    !WRITE (9, *) b
    !CLOSE (unit=9)
    !
    !OPEN (unit=9, file="bprime.out", position="rewind")
    !WRITE (9, *) bprime
    !CLOSE (unit=9)
    !
    !OPEN (unit=9, file="i_b_zero.out", position="rewind")
    !WRITE (9, *) i_b_zero
    !CLOSE (unit=9)
    
    IF (trace_test) THEN
        WRITE (*,*) 'bgrid'    
        WRITE (*,*) b
    
        WRITE (*,*) 'bprimegrid'
        WRITE (*,*) bprime
    END IF
    
END SUBROUTINE CREATE_GRIDS    

SUBROUTINE CREATE_SHOCKS(trace_test)

    INTEGER, INTENT(IN)             :: trace_test
    REAL (DP), DIMENSION(2,2)       :: A, varcovmat
    INTEGER                         :: t, i, j, i_ss, Ny_cut1, Ny_cut2 
    REAL (DP)                       :: cov, m_tau_cut1, m_tau_cut2
    REAL (DP)                       :: var_y, inc_factor, frac1, frac2, frac1_a, frac2_a
    
    !! begin
    WRITE (*,*) 'create_shocks...'

    IF (rgm == 2) THEN
        
        ! Define default cost regimes
        d1_rgm(1)       = d1_h
        d1_rgm(2)       = d1_l
    
        ! Define transition matrix for default cost regimes
        pr_rgm(1,1)     = rho_safer
        pr_rgm(1,2)     = onedp - rho_safer
        pr_rgm(2,1)     = onedp - rho_risky
        pr_rgm(2,2)     = rho_risky


        ! Define transition matrix for reentry prob
        ! theta_mat which is 2x2 matrix with theta in diagonals
        ! 1 off the diagonal if switching up
        theta_mat       = theta
        IF ((d1_h .GT. d1_l) .AND. (asymm_theta .EQ. 1)) THEN
            theta_mat(2,1)  = onedp
        ELSE IF ((d1_h .LT. d1_l) .AND. (asymm_theta .EQ. 1)) THEN
            theta_mat(2,1)  = onedp
        END IF
                
    ELSE

        ! Define default cost regimes
        d1_rgm(1)       = d1_h 
        
        ! Define transition matrix for default cost regimes
        pr_rgm(1,1)     = onedp

        ! Define transition matrix for reentry prob
        ! theta_mat which is 2x2 matrix with theta in diagonals
        ! 1 off the diagonals.
        theta_mat(1,1)  = theta
        
    END IF
    
    ! Define matrix of persistence and spillovers
    A(1,1) = rho_y
    A(1,2) = rho_piy
    A(2,1) = rho_ypi
    A(2,2) = rho_pi
    
    ! Define variance-covariance matrix
    varcovmat(1,1)  = sigma_y**2
    varcovmat(1,2)  = cov_piy
    varcovmat(2,1)  = cov_piy
    varcovmat(2,2)  = sigma_pi**2
    
    inc_factor      = 1.75_dp
    var_y           = MAX(eps,varcovmat(2,2))*A(1,2)**2/(1-A(1,1)**2)/(1-A(2,2)**2)  &
                        + MAX(eps,varcovmat(1,1))/(1-A(1,1)**2)
    
    m_tau_cut2      = MAX( -mtau, -0.5_dp )
    m_tau_cut1      = MIN( MIN ( -m_tau_cut2, -1.0_dp ) , MAX( -mtau, LOG(1.0_dp+d0)/SQRT(var_y) ) )

    ! fraction of points allocated to first region
    frac1           = (mtau+m_tau_cut1)/(2.0*mtau)
    ! to second region
    frac2           = (m_tau_cut2-m_tau_cut1)/(2.0*mtau)
    
    ! fraction adjusted up
    frac2_a         = inc_factor*frac2
    ! adjusted down
    frac1_a         = frac1*(onedp-frac2_a)/(onedp-frac2)
    
    IF ( m_tau_cut1 > -mtau ) THEN
        Ny_cut1     = 1 + MAX( 1, NINT(sy*frac1_a) )
    ELSE
        Ny_cut1     = 1
    END IF
    
    Ny_cut2 = MIN( sy, Ny_cut1 + NINT(sy*frac2_a) )
        
    IF (trace_test) THEN
100     FORMAT (1X, 2F10.6)
        PRINT *, 'variance covariance matrix: '    
        WRITE(*,100) ( varcovmat(1,i), i=1,2 )
        WRITE(*,100) ( varcovmat(2,i), i=1,2 )
        
        PRINT *, 'A = '    
        WRITE(*,100) ( A(1,i), i=1,2 )
        WRITE(*,100) ( A(2,i), i=1,2 )
    END IF
        
    
    ! Create transition matrix and shocks
    CALL BI_TAUCHEN_ALT(ygrid,pigrid,ptran,A,varcovmat,sy,spi,mtau,Ny_cut1,Ny_cut2,m_tau_cut1,m_tau_cut2)
	!CALL BI_TAUCHEN_ALT(ygrid,pigrid,ptran,A,varcovmat,sy,spi,mtau)
	!CALL BI_TAUCHEN(A,varcovmat,sy,spi,mtau,ygrid,pigrid,ptran,trace_test)
    
    IF (trace_test) THEN
        
        IF (ss<20) THEN
            WRITE (*,*) 'transition matrix'
            WRITE (*,*) ptran
        END IF
        
        PRINT *, "sum ptran(1,:):", SUM(ptran(1,:)) ! should sum to one
        
        !WRITE (*,*) 'raw income shocks'
        !WRITE (*,*) ygrid
        !
        !PRINT *, "pigrid"
        !PRINT *, pigrid
        
        PRINT *, "pigrid (low output)"
        PRINT *, (pigrid(i), i=1,sy*spi,sy )
                
        PRINT *, "pigrid (high output)"
        PRINT *, (pigrid(i), i=sy,sy*spi,sy )
    END IF
    
    ygrid = y*exp(ygrid)
    
    IF (trace_test) THEN
        
10      FORMAT (1X, 'income: ', F8.4, '| inflation: ',F8.4, '| exp inc:',F8.4, '| exp infl:',F8.4)
        DO i=1,ss
            WRITE (*,10) ygrid(i), pigrid(i), DOT_PRODUCT(ptran(i,:),ygrid), DOT_PRODUCT(ptran(i,:),pigrid)
        END DO
        
    END IF

	! Compute conditional expected inflation
    ex_infl     = zerodp
    DO i_ss=1,ss
        ex_infl(i_ss)   = DOT_PRODUCT(ptran(i_ss,:),pigrid)
    END DO   

    ! Compute conditional expected inflation
    ex_inflinv  = zerodp
    DO i_ss=1,ss
        ex_inflinv(i_ss) = onedp/(DOT_PRODUCT(ptran(i_ss,:), onedp/(onedp + pigrid))) - onedp
    END DO   
    
    ! Simulate output and inflation series
    cum_ptran = zerodp
    DO i=1,ss
        cum_ptran(i,1) = ptran(i,1)
        DO j=2,ss
            cum_ptran(i,j) = cum_ptran(i,j-1) + ptran(i,j)
        END DO
    END DO
    
!    IF (trace_test) THEN
!        ! Simulate output and inflation series
!        CALL DRNUN(N_S,shoc_simul)
!        y_simul = zerodp
!        pi_simul = zerodp
!    
!        cum_ptran = zerodp
!    
!        DO i=1,ss
!            cum_ptran(i,1) = ptran(i,1)
!            DO j=2,ss
!                cum_ptran(i,j) = cum_ptran(i,j-1) + ptran(i,j)
!            END DO
!        END DO
!        
!        i=1
!        y_simul(1) = ygrid(i)
!        pi_simul(1) = pigrid(i)
!
!        DO t=2,N_S
!            j = MAXVAL(MINLOC(ABS(cum_ptran(i,:)-shoc_simul(t))))
!            y_simul(t) = ygrid(j)
!            pi_simul(t) = pigrid(j)
!            i = j
!        END DO
!    
!        CALL COVARIANCE( N_S, y_simul, pi_simul, cov)
!        WRITE(*,*) 'simulated covariance(y,pi)', cov
!    END IF
    
	!! crisis cutoff
	yrec = 1.0_dp - yrec_sd*SQRT(var_y)
	
    !! Create bprime_inf 
    DO i_ssp=1,ss
        DO i_bp=1,ngpbprime
            bprime_inf(i_ssp,i_bp) = bprime(i_bp)/(1.0_dp + pigrid(i_ssp))
        END DO
    END DO
    
    OPEN (UNIT=unit_idx, file="bprime_inf.out", position="rewind")
    WRITE (unit_idx, *) bprime_inf
    CLOSE (UNIT=unit_idx)
    
    OPEN (UNIT=unit_idx, FILE='ptran.out', POSITION = "REWIND")
    WRITE (unit_idx, *) ptran
    CLOSE (UNIT=unit_idx)
    
    OPEN (UNIT=unit_idx, FILE='ygrid.out', POSITION = "REWIND")
    WRITE (unit_idx, *) ygrid
    CLOSE (UNIT=unit_idx)
    
    OPEN (UNIT=unit_idx, FILE='pigrid.out', POSITION = "REWIND")
    WRITE (unit_idx, *) pigrid
    CLOSE (UNIT=unit_idx)
    
END SUBROUTINE CREATE_SHOCKS

SUBROUTINE CREATE_AUTVALUE(trace_test)
    
    INTEGER, INTENT(IN) ::  trace_test

    CALL AUTARKY_VALUE
    
    IF (trace_test) THEN
        PRINT *, "vaut", vaut(1,1)
    END IF
    
END SUBROUTINE CREATE_AUTVALUE 

SUBROUTINE CREATE_GUESS(trace_test)
    
    INTEGER, INTENT(IN) ::  trace_test
        
    !! begin
    WRITE (*,*) 'create_guess...'
    
    CALL create_guess_p_V(trace_test)
    
    IF (trace_test) THEN
        WRITE (*,*) 'p(y_med,:)'
        WRITE (*,*) p(1, MAXVAL( MINLOC( ABS(ygrid-y)+ABS(pigrid) ) ), : )
        
        WRITE (*,*) 'p(y_low,:)'
        WRITE (*,*) p(1,MAXVAL(MINLOC(ygrid+ABS(pigrid))),:)
        
        WRITE (*,*) 'p(y_hi,:)'
        WRITE (*,*) p(1,MAXVAL(MAXLOC(ygrid+ABS(pigrid))),:)
    END IF
    
END SUBROUTINE CREATE_GUESS  

SUBROUTINE TEST_EIGEN(trace_test)

    !use lin_eig_self_int
    !use rand_gen_int
    
    INTEGER, INTENT(IN) ::  trace_test

    integer, parameter  :: n=2
    integer             :: i
    real(DP)             :: a(n,n), d(n), v_s(n,n), dm(n,n)
 
    !! Generate a random self-adjoint matrix.
    !call rand_gen(y)
    !a = reshape(y,(/n,n/))
    !a = a + transpose(a)
    
    a(1,1) = sigma_y**2
    a(1,2) = cov_piy
    a(2,1) = cov_piy
    a(2,2) = sigma_pi**2
    
    IF (trace_test) THEN
        WRITE (*,*) 'A'    
        WRITE (*,*) A
    END IF

    !! Compute the eigenvalues and eigenvectors (IMSL)
    !call lin_eig_self(a, d, v=v_s)
    !
    !dm = zerodp
    !DO i=1,n
    !    dm(i,i) = d(i)
    !END DO
    !
    !IF (trace_test) THEN
    !    WRITE (*,*) 'dm'    
    !    WRITE (*,*) dm
    !    WRITE (*,*) 'v_s'    
    !    WRITE (*,*) v_s      
    !    WRITE (*,*) 'A=v_s*dm*v_s^T'    
    !    WRITE (*,*) MATMUL(v_s,MATMUL(dm,TRANSPOSE(v_s)))   
    !END IF
    
    ! Compute the eigenvalues and eigenvectors (MKL)
    v_s = a
    call syevd( v_s , d ,'V')
    
    dm = zerodp
    DO i=1,n
        dm(i,i) = d(i)
    END DO
    
    IF (trace_test) THEN
        WRITE (*,*) 'dm'    
        WRITE (*,*) dm
        WRITE (*,*) 'v_s'    
        WRITE (*,*) v_s      
        WRITE (*,*) 'A=v_s*dm*v_s^T'    
        WRITE (*,*) MATMUL(v_s,MATMUL(dm,TRANSPOSE(v_s)))   
    END IF
    
    

END SUBROUTINE TEST_EIGEN

SUBROUTINE TEST_SOLVEMODEL(trace_test)

    ! local vars
    REAL(DP)            ::  capsule_time_start, capsule_time_stop
    INTEGER, INTENT(IN) ::  trace_test
    
    CALL create_grids(0)
    CALL create_shocks(0)
    CALL create_autvalue(0)
    
    IF (readguess==1) THEN
        CALL read_guess(0)
    END IF
    
    CALL create_guess(0)
    
    IF (conv_guess == 1) THEN
        
        WRITE (*,*) 'test_solvemodel...'
    
        capsule_time_start    = DCLOCK() 
        CALL SOLVEMODEL(1)
        capsule_time_stop     = DCLOCK()   
        
        CALL POST_PROCESS
        CALL SIMULATE_SIMPLE
        CALL SAVE_SIMPLE
    
    END IF
            
END SUBROUTINE TEST_SOLVEMODEL 

SUBROUTINE TEST_PARAMETERS(trace_test)
    
    INTEGER, INTENT(IN) ::  trace_test
    
    !! declarations
	CHARACTER (LEN = 80):: fileurl		! name of the file to read
    CHARACTER (LEN = 80):: prefix      ! name of the file to read
	INTEGER             :: numconfigs	! number of configurations contained in the file    
    INTEGER             :: aconfignum	! a configuration number 
    
    !! begin
    WRITE (*,*) 'load parameters...'

    !! read inputs
    WRITE (*,*) 'enter the filename'
    READ (*,*) fileurl
    fileurl = TRIM(fileurl)
    
    WRITE (*,*) 'enter the parameter file prefix'
    READ (*,*) prefix
    prefix = TRIM(prefix)    
    
    WRITE (*,*) 'enter the configuration to load'
    READ (*,*) aconfignum
    
    !! run subroutine
    CALL CREATE_PARAMFILES(fileurl, prefix, numconfigs)
    CALL LOAD_PARAMS(prefix, aconfignum)
        
END SUBROUTINE TEST_PARAMETERS     
 
SUBROUTINE TEST_CALIBRATE(trace_test)

    ! local vars 
    INTEGER, INTENT(IN) ::  trace_test
    REAL(8)             ::  capsule_time_start, capsule_time_stop
    INTEGER             ::  iter, i, ITMAX
    REAL(8)             ::  FTOL, FVALUE(4), XGUESS(4,3)
    
    IF (readguess==1) THEN
        conv_guess = 1
        CALL read_guess(0)
        CALL create_autvalue(0)
    ELSE
        CALL create_guess(0)
    END IF
      
    ! Set initial guess
    xguess(:,1)   = bita        ! borrower's discount factor
    xguess(:,2)   = d0 		    ! output loss coefficient (cutoff)
    xguess(:,3)   = d1_l        ! output loss coefficient (cost at mean)

    ! Set vertices of simplex
    xguess(2,1)   = 0.90_dp*bita        ! borrower's discount factor
    xguess(3,2)   = 0.75_dp*d0 		    ! output loss coefficient (cutoff)
    xguess(4,3)   = 0.75_dp*d1_l        ! output loss coefficient (cost at mean)

    !! begin
    WRITE (*,*) 'calibrate model...'
    
    MSE = 10.0_dp

99999 FORMAT ('  The best estimate for the minimum value of the', /, &
                '  function is X = (', 4F10.6, &
                ')', /, '  with MSE =', E13.6, //)
      
    OPEN (UNIT=10, FILE="CAL_PROGRESS.OUT", POSITION="REWIND")
    WRITE (10,*) " ------- Calibration Results -------"
    CLOSE (UNIT=10) 
    
    capsule_time_start    = DCLOCK() 
    DO i=1,4
        FVALUE(i) = SSFEVAL(XGUESS(i,:))
    END DO
    
    ftol    = 1.0D-6
    ITMAX   = 5000
    
    CALL AMOEBA(xguess,FVALUE,4,3,3,ftol,SSFEVAL,iter,ITMAX)
    WRITE (*,99999) (xguess(1,i),i=1,3), fvalue(1)
    capsule_time_stop     = DCLOCK() 
    
    OPEN (UNIT=10, FILE="CAL_RESULTS.OUT", POSITION="APPEND")
    WRITE (10,*) XGUESS(1,:), MSE
    CLOSE (UNIT=10)     

100 FORMAT (/1X, A, 'calibration took: ', F12.3, ' seconds')     
    WRITE (*,100) capsule_time_stop - capsule_time_start

     
    ! Save results
    CALL SAVE_SIMPLE
                      
END SUBROUTINE TEST_CALIBRATE

SUBROUTINE TEST_SIMULATE(trace_test)

    ! local vars
    INTEGER, INTENT(IN) ::  trace_test
    
    IF (readguess==1) THEN
        CALL read_guess(0)
        CALL create_autvalue(0)
        CALL SIMULATE_SIMPLE(1)
    ELSE
        CALL SIMULATE_SIMPLE(1)
    END IF
 
    !IF (TRACE_CODE) THEN
    !    WRITE (*,100) 'test_calibrate ', capsule_time_stop - capsule_time_start
    !END IF
    
END SUBROUTINE TEST_SIMULATE

SUBROUTINE RUN_AMOEBA(trace_test)

    ! local vars 
    INTEGER, INTENT(IN) ::  trace_test
    REAL(8)             ::  capsule_time_start, capsule_time_stop
    INTEGER             ::  iter, i, ITMAX
    REAL(8)             ::  FTOL, FVALUE(N_CAL+1), XGUESS(N_CAL+1,N_CAL)
    
    INTEGER                 :: ios          ! io status for system call
    CHARACTER (LEN=7)       :: cfg_str      ! configuration number
    CHARACTER (LEN=80)      :: outdir       ! output folder
    
    ! generate output folder name
    WRITE (UNIT=cfg_str, FMT='(I<sfx>.<sfx>)') config
    outdir = TRIM(folder_out) // TRIM(cfgpfx) // '_' // TRIM(cfg_str) // '/'      
    
    ! Setup folder for ouptut folder
    ios     = SYSTEM( 'mkdir -p ' // TRIM(outdir) )    
      
    ! Set initial guess
    IF (N_CAL == 3) THEN
        IF (B_CAL == 1) THEN
            xguess(:,1)   = bita        ! borrower's discount factor
        ELSE
            xguess(:,1)   = lambda      ! lambda
        END IF
        
        xguess(:,2)   = d0 		    ! output loss coefficient (cutoff)
        xguess(:,3)   = d1_l        ! output loss coefficient (cost at mean)
    ELSE
        xguess(:,1)   = d0 		    ! output loss coefficient (cutoff)
        xguess(:,2)   = d1_l        ! output loss coefficient (cost at mean)
    END IF
    
    ! Set vertices of simplex
    IF (N_CAL == 3) THEN
        IF (B_CAL == 1) THEN
            xguess(2,1)   = 0.90_dp*bita        ! borrower's discount factor
        ELSE
            xguess(2,1)   = 0.90_dp*lambda      ! lambda
        END IF
        
        xguess(3,2)   = 0.75_dp*d0 		    ! output loss coefficient (cutoff)
        xguess(4,3)   = 0.75_dp*d1_l        ! output loss coefficient (cost at mean)
    ELSE
        xguess(2,1)   = 0.90_dp*d0 		    ! output loss coefficient (cutoff)          
        xguess(3,2)   = 0.85_dp*d1_l        ! output loss coefficient (cost at mean) 		    
    END IF
    
    !! begin
    WRITE (*,*) 'calibrate model...'
    
    MSE     = 10.0_dp
    ftol    = 1.0D-6        ! ftol    = 1.0D-6
    ITMAX   = 100000        ! ITMAX   = 5000
    
      
    OPEN (UNIT=10, FILE= TRIM(outdir) // "CAL_PROGRESS.OUT", POSITION="REWIND")
    WRITE (10,*) "### ------- Calibration Results for configuration #: ", cfg_str
    CLOSE (UNIT=10) 
    
    capsule_time_start    = DCLOCK() 
    DO i=1,N_CAL+1
        FVALUE(i) = SSFEVAL(XGUESS(i,:))
    END DO
    

    
    CALL AMOEBA(XGUESS,FVALUE,N_CAL+1,N_CAL,N_CAL,ftol,SSFEVAL,iter,ITMAX)
    
    
99998 FORMAT ('  The best estimate for the minimum value of the', /, &
                '  function is X = (', 2F10.6, &
                ')', /, '  with MSE =', E13.6, //)    

    
99999           FORMAT ('  The best estimate for the minimum value of the', /, &
                '  function is X = (', 3F10.6, &
                ')', /, '  with MSE =', E13.6, //)    
    
    IF (N_CAL == 3) THEN            
        WRITE (*,99999) (xguess(1,i),i=1,N_CAL), fvalue(1)
    ELSE
        WRITE (*,99998) (xguess(1,i),i=1,N_CAL), fvalue(1)
    END IF
    
    capsule_time_stop     = DCLOCK() 
    
    OPEN (UNIT=10, FILE= TRIM(outdir) // "CAL_RESULTS.OUT", POSITION="APPEND")
    WRITE (10,*) XGUESS(1,:), MSE
    CLOSE (UNIT=10)     

100 FORMAT (/1X, A, 'calibration took: ', F12.3, ' seconds')     
    WRITE (*,100) capsule_time_stop - capsule_time_start

     
    ! Save results
    CALL SAVE_SIMPLE
                      
END SUBROUTINE RUN_AMOEBA

FUNCTION   SSFEVAL (X)
	!! Solves the model
    
	! INPUT & OUTPUT
    REAL(DP)        ::  X(N_cal), SSFEVAL
    REAL(DP)        ::  lambdamin_ss = 0.000000_dp
    REAL(DP)        ::  lambdamax_ss = 1.250000_dp
    REAL(DP)        ::  bitamin_ss  = 0.500000_dp
    REAL(DP)        ::  bitamax_ss  = 0.990000_dp
    REAL(DP)        ::  d0min_ss    = -.250000_dp
    REAL(DP)        ::  d0max_ss    = -.000001_dp
    REAL(DP)        ::  d1min_ss    = 0.001000_dp
    REAL(DP)        ::  d1max_ss    = 0.500000_dp
    REAL (DP)       ::  debt_lim    = 0.85_dp

    INTEGER                 :: ios          ! io status for system call
    CHARACTER (LEN=7)       :: cfg_str      ! configuration number
    CHARACTER (LEN=80)      :: outdir       ! output folder
    REAL (DP)               ::  prog_time_start, prog_time_stop

    ! generate output folder name
    WRITE (UNIT=cfg_str, FMT='(I<sfx>.<sfx>)') config
    outdir = TRIM(folder_out) // TRIM(cfgpfx) // '_' // TRIM(cfg_str) // '/'      
    
    ! Setup folder for ouptut folder
    ios     = SYSTEM( 'mkdir -p ' // TRIM(outdir) ) 
    
    ! Change parameter space
    IF (N_CAL == 3) THEN
        IF (B_CAL == 1) THEN
            bita    = X(1)       
            PARAMS_VEC%BITA_VAL		= bita
        ELSE
            lambda                  = X(1)
            PARAMS_VEC%LAMBDA_VAL   = lambda
        END IF
        d0          = X(2)
        d1_l        = X(3)
        
        PARAMS_VEC%D0_VAL       = d0
        PARAMS_VEC%D1_L_VAL     = d1_l
    ELSE
        d0          = X(1)
        d1_l        = X(2)        

        PARAMS_VEC%D0_VAL       = d0
        PARAMS_VEC%D1_L_VAL     = d1_l
    END IF
    
        
100 FORMAT ('   def prob (good) :',  F7.3,      &
            ' | def prob (bad)  :', F7.3,       &
            ' | avg spread 1: ', F7.3,          &
            ' | avg spread 2: ', F7.3) 

101 FORMAT (' bita  : ',  F12.8,                &
            ' | d0  : ', F12.8,                 &
            ' | d1_l: ', F12.8,                 &
            ' | MSE : ', ES11.3)   

103 FORMAT (' lambda: ',  F12.8,                &
            ' | d0  : ', F12.8,                 &
            ' | d1_l: ', F12.8,                 &
            ' | MSE : ', ES11.3)   
            
102 FORMAT (                                    &
            '   d0  : ', F12.8,                 &
            ' | d1_l: ', F12.8,                 &
            ' | MSE : ', ES11.3)  
            
    IF ((bita>bitamin_ss).AND.(bita<bitamax_ss)                     &
            .AND.(lambda>lambdamin_ss).AND.(lambda<lambdamax_ss)    &
            .AND.(d0>d0min_ss).AND.(d0<d0max_ss)                    &
            .AND.(d1_l>d1min_ss).AND.(d1_l<d1max_ss))               &
    THEN
        
    
        ! CREATE GRIDS
        CALL CREATE_GRIDS(0)
        CALL CREATE_SHOCKS(0)
        
        ! CREATE AUTARKY VALUE
        CALL CREATE_AUTVALUE(0)
        
        ! CREATE GUESS
        CALL CREATE_GUESS(0)
        
        IF (conv_guess == 1) THEN
           
            prog_time_start    = DCLOCK() 				! timer            
            CALL SOLVEMODEL 
            CALL POST_PROCESS
            CALL SIMULATE_SIMPLE
            prog_time_stop    = DCLOCK()               ! timer
    
            IF (calsprd1v2 == 1) THEN
                SSFEVAL = W_cal(1)*((results_simple%DPROB_GOOD_SIMUL_VAL - defprobg_t)/onedp)**2     &
                    + W_cal(2)*((results_simple%DPROB_BAD_SIMUL_VAL - defprobb_t)/onedp)**2          &
                    + W_cal(3)*((results_simple%SPREAD1_SIMUL_VAL - avgspread_t)/onedp)**2          
            
            ELSE IF (calsprd1v2 == 2) THEN
                SSFEVAL = W_cal(1)*((results_simple%DPROB_GOOD_SIMUL_VAL - defprobg_t)/onedp)**2     &
                    + W_cal(2)*((results_simple%DPROB_BAD_SIMUL_VAL - defprobb_t)/onedp)**2          &
                    + W_cal(3)*((results_simple%SPREAD2_SIMUL_VAL - avgspread_t)/onedp)**2

            ELSE
                SSFEVAL = W_cal(1)*((results_simple%DPROB_GOOD_SIMUL_VAL - defprobg_t)/onedp)**2     &
                    + W_cal(2)*((results_simple%DPROB_BAD_SIMUL_VAL - defprobb_t)/onedp)**2          &
                    + W_cal(3)*((results_simple%SPREAD1_SIMUL_VAL - avgspread_t)/onedp)**2
            END IF

            !! binding debt limit
            IF (4.0_dp*results_simple%DEBT_IN_SIMUL_VAL .GE. ABS(100.0_dp*debt_lim*bmin)) THEN
                PRINT *, "Debt limit bound issue"
                SSFEVAL = 1000.0_dp
            END IF
                        
            IF (N_CAL == 3) THEN
                IF (B_CAL == 1) THEN
                    WRITE(*,101) X, SSFEVAL
                ELSE
                    WRITE(*,103) X, SSFEVAL
                END IF
            ELSE
                WRITE(*,102) X, SSFEVAL
            END IF

            WRITE(*,100) results_simple%DPROB_GOOD_SIMUL_VAL, &
                        results_simple%DPROB_BAD_SIMUL_VAL, &
                        results_simple%SPREAD1_SIMUL_VAL, &
                        results_simple%SPREAD2_SIMUL_VAL

            IF ((SSFEVAL<MSE) .AND. (SSFEVAL<crit)          &
                              .AND. ( .NOT.ISNAN(SSFEVAL) ) &   ! added -- IK
                ) THEN
            
                MSE = SSFEVAL
                CALL SAVE_SIMPLE
            
                OPEN (UNIT=10, FILE= TRIM(outdir) // "CAL_PROGRESS.OUT", POSITION="APPEND")
                IF (N_CAL == 3) THEN
                    IF (B_CAL == 1) THEN
                        WRITE(*,101) X, SSFEVAL
                    ELSE
                        WRITE(*,103) X, SSFEVAL
                END IF
                ELSE
                    WRITE (10,102) X, MSE
                END IF
                
                CLOSE (UNIT=10) 
                
                PRINT *, "**************************************************"
	            PRINT *, "Code took: ", ceiling ((prog_time_stop - prog_time_start)/60), " minutes"
                PRINT *, "**************************************************"
	
            
            END IF 
        
        ELSE
            
                PRINT *, "**************************************************"
	            PRINT *, "High MSE ... ", SSFEVAL
                PRINT *, "**************************************************"
        END IF
    
    ELSE
        SSFEVAL = 1000.0_dp
    END IF
    
END FUNCTION SSFEVAL

END MODULE LAUNCH
