PROGRAM MAIN_HKP
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! PROGRAM: MAIN_HKP                                !!
    !!                                                  !!
    !! Date: Nov 15, 2016                               !!
    !!                                                  !!
    !! Profile:                                         !!
    !!  main_hkp                                        !!
    !!  main_hkp runmode setup params fnm prefix pfx    !!
    !!  main_hkp runmode create prefix pfx cfgnum cfg   !!
    !!  main_hkp runmode compute prefix pfx cfgnum cfg  !!
    !!  main_hkp runmode amoeba prefix pfx cfgnum cfg   !!
    !!  main_hkp runmode simuls prefix pfx cfgnum cfg   !!
    !!  main_hkp runmode gather params fnm prefix pfx & !!
    !!                          resfnm out              !!
    !!                                                  !!
    !! Inputs:                                          !!
    !! Inputs are pairs 'argtype argvalue' where        !!
    !!  argtype can be                                  !!
    !!  >> 'runmode' with possible values:              !!
    !!          'setup'  'create' 'compute' 'amoeba' 	!!  
    !!          'simuls' 'gather'                   	!!
    !!  >> 'params' is the parameter input file         !!
    !!  >> 'prefix' is the prefix of parameter folders  !!
    !!  >> 'cfgnum' is the parametrization (file #)     !!
    !!  >> 'resfnm' is the output file to be gathered   !!
    !!                                                  !!
    !! Changes:                                         !!
    !! >> 05/15/2015                                    !!
    !!  readguess                                       !!
    !!  - made sure the files are read from ini/        !!
    !!  saveresults                                     !!
    !! >> 03/16/2016                                    !!
    !!  - addprm mode added                             !! 
    !!  -- for appending of parameters in gather mode   !! 
    !! >> 05/04/2018                                    !!
    !!  - simul mode added                             	!!     
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    
    !!!!!!!!!!!!!!!!!!!!!!!!
    !!   USE STATEMENTS   !!
    !!!!!!!!!!!!!!!!!!!!!!!!    
    USE PARAMETERS
    USE GLOBAL
    USE PROCESS
    USE AUXILIARY
    USE UTILITIES
    USE LAUNCH
    USE LAPACK95
    USE IFPORT
    USE OMP_LIB
    
    
    !! REMOVE IMPLICIT VARIABLE TYPES
    IMPLICIT NONE  

    
    !! ARGUMENTS FROM COMMAND LINE
    CHARACTER (LEN = 80)    :: runmode          ! execution mode = solve, load
    CHARACTER (LEN = 80)    :: paramfile        ! filename for parameter configurations
    CHARACTER (LEN = 80)    :: filepfx          ! prefix of configuration files    
    INTEGER                 :: paramidx         ! number in configuration to use
    CHARACTER (LEN = 80)    :: outfnm           ! output filename to be gathered
    LOGICAL                 :: addprm = .TRUE.  ! whether or not to append parameter list (T/F. T by default)
    
    !! ARGUMENTS FOR PROGRAM
    CHARACTER (LEN = 80)    :: cmdline      ! generic command line    
    INTEGER                 :: ios          ! io status for system runs
    INTEGER                 :: num          ! case selected on menu
    CHARACTER (LEN = 80)    :: argtype      ! argument type inputted
    INTEGER                 :: argi         ! argument index
    INTEGER                 :: argtot       ! argument count
    INTEGER                 :: argpairs     ! argument pairs
    INTEGER                 :: status       ! read status variable
    CHARACTER (LEN = 80)    :: msg          ! i/o error message
    
    !! MISC ARGUMENTS
    INTEGER                 :: numcfgs      ! number of configurations found
    INTEGER             	:: number_of_threads
    
    !! MAIN PROGRAM    
    number_of_threads = OMP_get_max_threads()
    number_of_threads = OMP_get_max_threads() - 0
    
    CALL OMP_set_num_threads(number_of_threads)
    PRINT *, 'number of OMP threads', number_of_threads
	
	WRITE (*,*) ' MAIN MENU '

    ! Get command count
    argtot      = COMMAND_ARGUMENT_COUNT()
    argpairs    = argtot/2
    
    ! Setup folder for configuration files
    IF (argtot .EQ. 0) THEN
		folder_cfg = 'results'
        rc  = (onedp - bita_l)*(onedp - delta)
        readguess = 0
    ELSE
        ios     = SYSTEM( 'mkdir -p ' // TRIM(folder_cfg) )
	END IF
        
    ! Loop across input pairs and store them
    ifcmdargs: IF (argtot .EQ. 0) THEN
        
        doMenu: DO
            ! list selections
            WRITE (*,*) ' --***---***-- '
            WRITE (*,*) ' Enter the number of the routine to run   '
            WRITE (*,*) ' (01)  read guess  '  
            WRITE (*,*) ' (02)  create grids    '  
            WRITE (*,*) ' (03)  create shocks   '  
            WRITE (*,*) ' (04)  create autarky value   '  
            WRITE (*,*) ' (05)  create guess    '  
            WRITE (*,*) ' (06)  run solvemodel    '  
            WRITE (*,*) ' (07)  load parameters    '
		    WRITE (*,*) ' (08)  test simulate    '  
			WRITE (*,*) ' (10)  test eigenvalues    '  
			WRITE (*,*) ' (11)  test compute model    ' 
            WRITE (*,*) ' (12)  test calibrate model    ' 
            WRITE (*,*) ' (00)  QUIT        '
        
            ! read selection
            READ (*,*) num    
        
            SELECT CASE (num)
            CASE(1)
                CALL read_guess(1)
            CASE(2)
                CALL create_grids(1)
            CASE(3)
                CALL create_shocks(1)
            CASE(4)
                CALL create_autvalue(1)
            CASE(5)
                CALL create_guess(1)
            CASE(6)
                CALL test_solvemodel(1)
            CASE(7)
                CALL test_parameters(1)
			CASE(8)
				CALL test_simulate(1)        	
            CASE(10)
                CALL test_eigen(1)        	
            CASE(11)
                CALL launchcomputemodel(1)
            CASE(12)
                CALL test_calibrate(1)
            CASE DEFAULT
                EXIT
            END SELECT

        END DO doMenu
    
    ELSE
        
        doEachArg: DO argi = 1, argpairs
            
            CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi-1, VALUE = argtype)
            argtype = TRIM(argtype)
    
            SELECT CASE (argtype)
            
                CASE('runmode')
                
                    CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi, VALUE = cmdline)
                    runmode = TRIM(cmdline)
                    WRITE (*, *) ' Program run mode is: ', runmode
                
                CASE('params')
                
                    CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi, VALUE = cmdline)
                    paramfile = TRIM(cmdline)
                    WRITE (*, *) ' Parameter file name is: ', paramfile
            
                CASE('prefix')
                
                    CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi, VALUE = cmdline)
                    filepfx = TRIM(cmdline)
                    WRITE (*, *) ' Prefix for file is: ', filepfx
                
                CASE('cfgnum')
                
                    CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi, VALUE = cmdline)
                    cmdline = TRIM(cmdline)
                
                    READ (cmdline, FMT='(I7)', IOSTAT=status, IOMSG=msg ) paramidx
                
                    IF (status == 0) THEN
                        WRITE (*, *) ' Configuration number is: ', paramidx
                    ELSE
                        WRITE (*, *) ' Invalid input - number required.', TRIM(msg)
                    END IF               

                CASE('resfnm')
                
                    CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi, VALUE = cmdline)
                    outfnm = TRIM(cmdline)
                    WRITE (*, *) ' Result filename is : ', outfnm

                CASE('addprm')
                
                    CALL GET_COMMAND_ARGUMENT(NUMBER = 2*argi, VALUE = cmdline)
                    cmdline = TRIM(cmdline)
                    
                    READ (cmdline, FMT='(L7)', IOSTAT=status, IOMSG=msg ) addprm
                
                    IF (status == 0) THEN
                        WRITE (*, *) ' Add parameters (T/F)? : ', addprm
                    ELSE
                        addprm  = .TRUE.
                        WRITE (*, *) ' Invalid input - T/F required. (T assumed)', TRIM(msg)
                    END IF
                                
                CASE DEFAULT
                
                    WRITE (*, *) ' Command not recognized '
            
            END SELECT
                
        END DO doEachArg
        
        !! call relevant subprograms
        !! options are
        !! >> main_hkp runmode setup params fnm prefix pfx
        !! >> main_hkp runmode compute prefix pfx cfgnum cfg
        !! >> main_hkp runmode gather params fnm prefix pfx resfnm out
        
        ifmode: IF (runmode == 'setup') THEN
            
            CALL SETUP_PARAMFILE(paramfile)
            CALL CREATE_PARAMFILES(paramfile, filepfx, numcfgs)
        
        ELSE IF (runmode == 'create') THEN
            
            CALL CREATE_PARAMFILES(paramfile, filepfx, numcfgs)
        
        ELSE IF (runmode == 'compute') THEN
            
            CALL LOAD_PARAMS(filepfx, paramidx)
            CALL PRINT_PARAMS
            CALL LAUNCHCOMPUTEMODEL(1)
        
        ELSE IF (runmode == 'amoeba') THEN
            
            CALL LOAD_PARAMS(filepfx, paramidx)
            CALL PRINT_PARAMS
            CALL RUN_AMOEBA(1)
        
        ELSE IF (runmode == 'simuls') THEN
            
			simonly 	= 1
            readguess   = 1 
            
            CALL LOAD_PARAMS(filepfx, paramidx)
            CALL PRINT_PARAMS
            CALL LAUNCHSIMULATEMODEL(1)
            
        ELSE IF (runmode == 'gather') THEN  
            
            CALL GATHER_FILES(paramfile,filepfx, outfnm, addprm)
            
        END IF ifmode
                
    END IF ifcmdargs
    
END PROGRAM MAIN_HKP
