MODULE PARAMETERS
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! PROGRAM: PARAMETERS                              !!
    !!                                                  !!
    !! Last: Feb 23 2018                                !!      
    !!                                                  !!
    !!                                                  !!
    !! Changes:                                         !!
    !! >> 2017/09/05                                    !!
    !!    LT debt version                               !!
    !! >> 2018/02/23                                    !!
    !!    regime switching version                      !!
    !!                                                  !!
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    

INTEGER, PARAMETER  ::  DP          = SELECTED_REAL_KIND(P=13)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! BASELINE CALIBRATION !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

REAL (DP) 		 	::  bita        = 0.82652805_dp,    &       ! borrower's discount factor
                        bita_l      = 0.99502500_dp,    &       ! lender's discount factor
                        sigma       = 02.0_dp, 			&       ! borrower risk aversion 
                        sigma_l     = 08.0_dp,          &       ! lender risk aversion
                        ies_l       = 08.0_dp,          &       ! lender ies (if ies_l = sigma_l then ez = crra)
                        lambda      = 0.96_dp,          &       ! recovery rate in default
                        delta       = 1.000_dp,         &       ! maturity parameter
                        d0          = -0.02792534_dp,   &		! output loss coefficient (cutoff)
                        d1_h        = 0.15117401_dp,    &		! output loss coefficient (cost at mean) -- safer regime
                        d1_l        = 0.15117401_dp,    &		! output loss coefficient (cost at mean) -- risky regime
                        theta       = 0.100_dp,         &       ! reentry probability
                        cov_piy     = 0.0000000_dp		        ! covariance of income and inflation innovations (+-0.0000170_dp)


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! CALIBRATION TARGETS  !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

REAL (DP) 		 	        ::  defprobg_t  = 0.500_dp,     &       ! good times default probability (percent) -- 1.020_dp
                                avgspread_t = 0.740_dp,     &       ! average of annualized spread (percent) -- 0.740_dp
                                defprobb_t  = 3.500_dp              ! bad times default probability (percent) -- 2.580_dp 

INTEGER, PARAMETER          ::   EZ_mode    = 1                     ! set = 1 for EZ preferences (ies_l,sigma_l) vs = 0 for CRRA (sigma_l) 
INTEGER, PARAMETER          ::   N_cal      = 2                     ! set to the number of parameters to calibrate
INTEGER, PARAMETER          ::   B_cal      = 0                     ! b_cal = 1 if bita is calibrated and 1 and if lambda is
REAL (DP), DIMENSION(3)     ::   W_cal      = (/ 0.50_dp,   &       ! weight on good times def prob
                                                 0.50_dp,   &       ! weight on bad times def prob
                                                 0.00_dp /)         ! weight on spreads


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! CALIBRATION BOUNDS  !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

REAL(DP), PARAMETER ::  betamin     = 0.98750000_dp,   	&       ! government discount factor - min
                        betamax     = 0.98750000_dp,  	&       ! government discount factor - max
                        beta_lmin   = 0.99000000_dp,    &       ! lender discount factor - min
                        beta_lmax   = 0.99000000_dp,    &       ! lender discount factor - max
                        sigma_min 	= 02.00_dp,   	    &    	! risk aversion borrower - min
                        sigma_max 	= 02.00_dp,   	    &    	! risk aversion borrower - max
                        sigma_lmin 	= 59.00_dp,         &    	! epstein-zin risk aversion lender - min
						sigma_lmax 	= 59.00_dp,         &       ! epstein-zin risk aversion lender - max
                        ies_lmin 	= 01.00_dp,         &    	! epstein-zin ies lender - min
						ies_lmax 	= 01.00_dp,         &       ! epstein-zin ies lender - max
                        lambdamin   = 0.960_dp*1.0_dp,  &       ! recovery rate in default - min -- 0.960_dp
                        lambdamax   = 0.960_dp*1.0_dp,  &       ! recovery rate in default - max -- 0.960_dp
                        deltamin    = 1.0_dp/04.0_dp,   &       ! maturity parameter - min -- 1.0_dp/20.0_dp
                        deltamax    = 1.0_dp/1.00_dp,   &       ! maturity parameter - max -- 1.0_dp/20.0_dp
                        d0min       = -0.02750000_dp, 	&       ! output loss coefficient (linear) - min - 0.0823529_dp
                        d0max       = -0.02750000_dp, 	&       ! output loss coefficient (linear) - max
                        d1_hmin     = 0.20000000_dp, 	&       ! safer regime -- output loss coef. (loss @ mean) - min  - 0.095_dp
                        d1_hmax     = 0.200000000_dp, 	&       ! safer regime -- output loss coef. (loss @ mean) - max
                        d1_lmin     = 0.16000000_dp, 	&       ! risky regime -- output loss coef. (loss @ mean) - min  - 0.095_dp
                        d1_lmax     = 0.16000000_dp, 	&       ! risky regime -- output loss coef. (loss @ mean) - max
                        thetamin    = 0.100_dp,         &       ! reentry probability - min
                        thetamax    = 0.100_dp,         &       ! reentry probability - max
                        cov_piymin  = +0.0000170_dp*1.0_dp,	&   ! comovement cov _piy - min // +0.0000328_dp
                        cov_piymax  = -0.0000170_dp*1.0_dp      ! comovement cov _piy- max  // -0.0000328_dp

INTEGER, PARAMETER  ::  n_beta      = 01,               &       ! sample points for calibration - beta
                        n_beta_l    = 01,               &       ! sample points for calibration - beta_l
                        n_sigma     = 01,               &       ! sample points for calibration - sigma
                        n_sigma_l   = 01,               &       ! sample points for calibration - sigma_l
                        n_ies_l     = 01,               &       ! sample points for calibration - ies_l
                        n_lambda    = 01,               &       ! sample points for calibration - lambda - recovery rate in default
                        n_delta     = 02,               &       ! sample points for calibration - delta - maturity parameter
                        n_d0        = 01,               &       ! sample points for calibration - d0 - output loss cutoff
                        n_d1_h      = 01,               &       ! sample points for calibration - d1_h - safer regime -- output loss at mean 
                        n_d1_l      = 01,               &       ! sample points for calibration - d1_l - risky regime -- output loss at mean
                        n_theta     = 01,               &       ! sample points for calibration - theta - reentry probability
                        n_cov_piy   = 02                        ! sample points for calibration - cov_piy


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! CONSTANT PARAMETERS !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


REAL (DP), PARAMETER :: y           = 01.00_dp,         &       ! output mean
                        mu          = 1.000_dp,         &       ! relative size of lenders
                        alpha2      = 1.0_dp/mu,        &       ! if alpha2 = 0, comparable to Arellano extension
                        rho_y       = 0.800_dp,         &       ! persistence of income shocks
                        rho_pi      = 0.800_dp,         &       ! persistence of inflation shocks
                        rho_piy     = 0.000_dp,         &       ! spillover from inflation to income
                        rho_ypi     = 0.000_dp,         &       ! spillover from income to inflation
                        sigma_y     = 0.0100_dp,        &       ! standard deviation of income innovations (us: .0046)
                        sigma_pi    = 0.0100_dp,        &       ! standard deviation of inflation innovations (us:.0026)
                        rho_safer   = 0.992_dp,         &       ! persistence of safer regime (.980)
                        rho_risky   = 0.909_dp,         &       ! persistence of risky regime (.910)
                        mtau        = 3.0_dp,           &       ! determines the spacing of shock grids
                        chi_quad    = 02.00_dp,         &       ! adjustment costs - quadratic adjustments
                        chi_pwr     = 05.00_dp,         &       ! grid - adjustment costs - curvature on chi grid
                        chi_min     = 0.0_dp,           &       ! grid - adjustment costs - min
                        chi_max     = 10.00_dp,         &       ! grid - adjustment costs - max (10)
                        issue_max   = +0.50_dp*y,       &       ! debt issuance bounds - max (01.0_dp)
                        issue_min   = -0.50_dp*y                ! debt issuance bounds - min (01.0_dp)
         

INTEGER, PARAMETER ::   ngpb        = 0150,         &       !! number of grid points in B grid (100)
                        ngpbprime   = 0250,         &       !! number of grid points in B' grid (300)
                        sy          = 25,           &       !! number of output shock levels (23)
                        spi         = 09,           &       !! number of inflation shock levels (09)
                        ss          = sy*spi,       &       !! number (spi) * (output shock)
                        rgm         = 01,           &       !! number of default cost regimes (02)
                        pfitmax     = 400,          &       !! maximum number of loop iterations (400)
                        pfitmin     = 0005,         &       ! minimum number of loop iterations (005)
                        N_chi       = 10,           &       ! number of value on the chi adjustment grid (10)
                        pit_guess   = 1000,         &       !! maximum number of loop iterations on pricing kernel (200)
                        pitmax      = 400,          &       ! maximum number of loop iterations on pricing kernel
                        pitmin      = 025,          &       ! minimum number of loop iterations on pricing kernel
                        vfitmax     = 500,          &       ! maximum number of loop iterations
                        vlitmax     = 1000,         &       ! maximum number of loop iterations - lender's value function
                        N_S_simple  = 5000,         &       ! number of simulations -- long run (1000)
                        T_simul     = 20100,        &       ! number of periods per simulation -- long run
                        T_simul_erg = 0100,         &       ! min period used in for ergodic distribution -- long run
                        T_simul_large = 10000,      &       ! simulation for computing variance of output
                        T_simul_min = 0010,         &       ! min period used in for simulation stats -- with spells
                        T_simul_max = T_simul-10,   &       ! max period used in for simulation stats -- with spells
                        N_mom_stat  = 18,           &       ! number of stats for each simulation -- with spells
                        N_mom_prm   = 11,           &       ! number of parameters for each simulation -- with spells
                        N_moments   = N_mom_stat+N_mom_prm  ! total number of variables for simulation -- with spells

REAL (DP), PARAMETER :: bmax        = 0.00_dp,      &       ! upper bound on debt grid ( +0.00_dp >= 0 )
                        bmin        = -6.00_dp,     &       ! lower bound on debt grid ( -4.00_dp <= 0 ) 
                        bpvt        = -2.50_dp,     &       ! pivot point on debt grid ( -1.75_dp <= 0 ) 
                        apvt        = 0.50_dp,      &       ! scaling # debt grid points below pivot point (.35)
                        onepwr      = 1.00_dp,      &       ! curvature for even spacing
                        grdpwr      = 2.00_dp,      &       ! > 1 for more points near 0, < 1 otherwise, = 1 equal spacing
                        adjpwr      = 3.00_dp               ! curvature on ajusted weights

REAL (DP), PARAMETER :: critpavg    = 1.0D-6,       &       ! price schedule iteration criterion
                        critpguess  = 1.0D-6,       &       ! price schedule iteration criterion
                        critpsav    = 1.0D-5,       &       ! equilibrium price iteration criterion
                        critpmax    = 1.0D-3,       &       ! price iteration criterion
                        critsavg    = 1.0D-5,       &       ! savings iteration criterion
                        critv_l     = 1.0D-6,       &       ! lender's VF iteration criterion
                        weightnew   = 0.8_dp,       &       ! 
                        pweightnew  = 0.2_dp,       &       ! 
                        pweightmax  = 0.8_dp,       &       ! between 0 and 1
                        pweightmin  = 1.0D-8,       &       ! between 0 and 1
                        crit        = 1.0D-0,       &       ! save criteria
                        infty       = 1.0D12,       &       ! large positive value
                        onedp       = 1.0_dp,       &
                        zerodp      = 0.0_dp,       &
                        eps         = 1.0D-12               ! small positive value

REAL (DP)           ::  yrec_sd     = 0.0_dp,       &       ! yrec = 1 - yrec_sd*std(y) - simulation stats
                        pcrisis_med = 0.15_dp,      &       ! crisis probability cutoff - simulation stats
                        pcrisis_lo  = 0.02_dp,      &       ! crisis probability cutoff - simulation stats
                        pcrisis_hi  = 0.75_dp,      &       ! crisis probability cutoff - simulation stats
                        rcrisis     = 200.0_dp,     &       ! crisis spreads cutoff - simulation stats
                        rmax        = 0.75_dp               ! crisis spreads cutoff - simulation stats

REAL (DP)           ::  pdef_bad    = 0.10_dp               ! default prob cutoff -- very bad times
REAL (DP)           ::  tau_govt    = 0.193_dp              ! tax revenue share of output
REAL (DP)           ::  rs          = 1.010_dp              ! inverse of lender's discount factor [cannot be constant]
REAL (DP)           ::  rc          = 0.0_dp                ! coupon    

INTEGER             :: 	simonly 	= 0                     ! set to 1 if simulation only mode
                                               
INTEGER             ::  readguess   = 0,            &       ! readguess  = 1 if guess exists
                        readinichi  = 0,            &       ! readinichi = 1 yo start at stored chi value
                        zerocoupon  = 0,            &       ! set coupon to 1 if zerocoupon == 1 calsprd1v2 == 2
                        calsprd1v2  = 1,            &       ! whether to use spreads1 (=1) or spreads2 (=2) for calibration
                        crithybrid  = 20,           &       ! savings iteration criterion
                        nodefault   = 0                     ! set nodefault = 1 to force value of default to infinity 
                        
INTEGER             ::  unit_idx    = 9                     ! default unit number used

INTEGER             ::  maxparams   = 10000                 ! maximum numbers of parameter configurations allowed

INTEGER             ::  cds_mat      = 1                    ! value (quarters) for default prob stats -- use floor(onedp/delta) for maturity of bonds.
INTEGER             ::  asymm_theta  = 0                    ! = 1 if theta_mat(2,1) == 1 as opposed to theta 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!   RUNTIME CONTROLS   !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

INTEGER             ::  config      = 1                     ! configuration number - useful for multiple simulations
CHARACTER (LEN = 20)::  cfgpfx      = 'pfx'                 ! prefix for auto-generated calibration file
CHARACTER (LEN = 3) ::  ext         = 'in'                  ! file extension for parameter files
INTEGER, PARAMETER  ::  sfx         = 6                     ! width (digits) of the configuration index 
CHARACTER (LEN = 20)::  folder_cfg  = 'cfg/'                ! folder for parameter files created
CHARACTER (LEN = 20)::  folder_out  = 'out/'                ! folder for result files generated
CHARACTER (LEN = 20)::  folder_ini  = 'ini/'                ! folder for initial guess to be read

END MODULE PARAMETERS