MODULE GLOBAL
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! PROGRAM: GLOBAL                                  !!
    !!                                                  !!
    !! Last: Feb 02 2018                                !!     
    !!                                                  !!
    !!                                                  !!
    !! Changes:                                         !!
    !! >> 2017/09/08                                    !!
    !!    implemented LT debt with new lender SDF       !!
    !! >> 2018/02/23                                    !!
    !!    regime switching version                      !!
    !!                                                  !!
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
USE PARAMETERS


REAL (DP), DIMENSION (ngpb)                 ::  b, i_bp_min, i_bp_max
REAL (DP), DIMENSION (ngpbprime)            ::  bprime
REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  v, vo, vnd, vdef, vdefo,        &   ! Value functions (borrower)
                                                v_l, vo_l, vnd_l, vdef_l, 		&	! Value functions (lender)
														   vndo_l, vdefo_l, 	&	! Value functions (lender)
												vv_l, vvo_l, vvnd_l, vvdef_l,	&	! Value functions - v2 (lender)
														   vvndo_l, vvdefo_l		! Value functions - v2 (lender)
    
REAL (DP), DIMENSION (rgm,rgm)              ::  pr_rgm, theta_mat
REAL (DP), DIMENSION (rgm)                  ::  d1_rgm

REAL (DP), DIMENSION (ss, ngpbprime)        ::  bprime_inf
INTEGER                                     ::  i_ss, i_ssp, i_bp, i_b, &
                                                i_rgm, i_rgmp, &
                                                conv_guess, conv_ind

REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  sav, savo, savoo,               &   ! policy functions (assets)
                                                savnd, savndo,                  &   ! policy functions (assets, no default)
                                                def, defo,                      &   ! policy functions (default)
                                                cons, consnd, yy, payL, consL, 	&   ! policy functions (consumption)
                                                sav_aux, psav_aux, def_aux          ! auxiliary policy functions

REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  psav, psavo, psavnd,            &   ! equilibrium bond price
                                                psav_nodef, psav_def,			& 
												real_psav, real_psavnd,			&
                                                real_psav_nodef,                &
                                                real_psav_def,                  &
												real_pdef, real_pdef_old
		
REAL (DP), DIMENSION (ss)                   ::  p_guess, p1_guess, pdef_guess       ! price schedule -- guess under no default        
REAL (DP), DIMENSION (ss)                   ::  p_riskfree, p1_riskfree             ! price schedule -- no risk (default or inflation)       
REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  pdef, pdef1, pdef2                  ! price in default  
REAL (DP), DIMENSION (rgm,ss,ngpbprime)     ::  p, p1, p2,                      &   ! price schedule
												real_p, real_p_old,				&	! real price schedule
                                                p_nodef,                        &   ! price schedule (no default tomorrow)
                                                p_def,							&   ! price schedule (default tomorrow)
												real_p_nodef, real_p_def			! real prices

REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  real_psav_v2, real_psavnd_v2,   &
                                                real_psav_nodef_v2,             &
                                                real_psav_def_v2,               &
												real_pdef_v2, real_pdef_old_v2

REAL (DP), DIMENSION (rgm,ss,ngpbprime)     ::  real_p_v2, real_p_old_v2,       &	! real price schedule
												real_p_nodef_v2, real_p_def_v2      ! real prices

REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  dprob,                          &   ! default probability
                                                dprobnd,                        &   ! default probability (not in default)
                                                dprobid                             ! default probability (in default)

REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  realp_prod_mean,                &
                                                realp_mean_nodef,               &
                                                realp_mean_sdfl,                &
                                                realp_mean_defl,                &
                                                realp_mean_nodef_cov_sdfl_defl, &
                                                realp_mean_sdfl_cov_nodef_defl, &
                                                realp_mean_defl_cov_sdfl_nodef, &
                                                realp_prod_mean_nodef_alt,      &
                                                realp_prod_mean_indef_alt,      &
                                                realp_cov_defl_sdfl_nodef_alt,  &
                                                realp_cov_defl_sdfl_indef_alt

REAL (DP)                                   ::  c, assets, price, pricedef,izerobp, &
                                                i_b_zero, chi, MSE, issue
REAL (DP)                                   ::  yrec                                ! crisis output cutoff - simulation stats
REAL (DP), DIMENSION (N_chi)                ::  chigrid

REAL (DP), DIMENSION (ss,ss)                ::  sdf,                            &   ! lender's sdf
                                                sdf_nodef,                      &   ! lender's sdf (no default tomorrow)
                                                sdf_def                             ! lender's sdf (default tomorrow)

REAL (DP), DIMENSION (ss)                   ::  ygrid, pigrid, ex_infl, ex_inflinv, &
                                                ex_sdf, ex_sdf_nodef, ex_sdf_def

REAL (DP), DIMENSION (rgm,ss)               ::  conaut, vaut, utilaut
REAL (DP), DIMENSION (ss)                   ::  utildef, temp3, v_lrec, vo_lrec, Ev_lrec
REAL (DP), DIMENSION (ss)                   ::  qb
REAL (DP), DIMENSION (ss,ss)                ::  ident, temp, tempinv, temp1, tempinv2
REAL (DP), DIMENSION (ss,ss)                ::  ptran, cum_ptran
INTEGER,   DIMENSION (ss)                   ::  iptemp, iptemp2

REAL (DP), DIMENSION(N_S_simple)            ::  dprob_avg, dprob_avg_h, dprob_avg_l,            &
                                                debt_in_avg, debt_in_avg_h, debt_in_avg_l,      &
                                                debt_out_avg, debt_out_avg_h, debt_out_avg_l,   &
                                                spread1_avg, spread1_avg_h, spread1_avg_l,      &
                                                spread1_sdv, &
                                                spread2_avg, spread2_avg_h, spread2_avg_l,      &
                                                spread2_sdv, &
                                                spread__avg, &
                                                spread__sdv, &
                                                rfree_avg,  rfree_avg_h,  rfree_avg_l,          &
                                                rfreeg_avg, rfreeg_avg_h, rfreeg_avg_l,         &
                                                rfreeb_avg, rfreeb_avg_h, rfreeb_avg_l,         &
                                                spreadg_avg, spreadg_avg_h, spreadg_avg_l,      &
                                                spreadb_avg, spreadb_avg_h, spreadb_avg_l,      &
                                                sprdv2g_avg, sprdv2g_avg_h, sprdv2g_avg_l,      &
                                                sprdv2b_avg, sprdv2b_avg_h, sprdv2b_avg_l,      &
                                                dprobg_avg, dprobg_avg_h, dprobg_avg_l,         &
                                                dprobb_avg, dprobb_avg_h, dprobb_avg_l

REAL (DP), DIMENSION(N_S_simple)            ::  vlend_avg, vlend_avg_h, vlend_avg_l,            &
                                                vlendg_avg, vlendg_avg_h, vlendg_avg_l,         &
                                                vlendb_avg, vlendb_avg_h, vlendb_avg_l,         &
                                                vlendbb_avg, vlendbb_avg_h, vlendbb_avg_l

REAL (DP), DIMENSION(N_S_simple)            ::  vvlend_avg, vvlend_avg_h, vvlend_avg_l,         &
                                                vvlendg_avg, vvlendg_avg_h, vvlendg_avg_l,      &
                                                vvlendb_avg, vvlendb_avg_h, vvlendb_avg_l,      &
                                                vvlendbb_avg, vvlendbb_avg_h, vvlendbb_avg_l

REAL (DP), DIMENSION(N_S_simple)            ::  vborr_avg, vborr_avg_h, vborr_avg_l,            &
                                                vborrg_avg, vborrg_avg_h, vborrg_avg_l,         &
                                                vborrb_avg, vborrb_avg_h, vborrb_avg_l,         &
                                                vborrbb_avg, vborrbb_avg_h, vborrbb_avg_l


REAL (DP), DIMENSION(N_S_simple)            ::  realprice_avg, realprice_avg_h, realprice_avg_l,            &
                                                realprice_avg_gd, realprice_avg_gd_h, realprice_avg_gd_l,   &
                                                realprice_avg_bd, realprice_avg_bd_h, realprice_avg_bd_l

REAL (DP), DIMENSION(N_S_simple)            ::  realprice_sdv,      &
                                                realprice_sdv_gd,   &
                                                realprice_sdv_bd

REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_real_psav, &
                                                dcp_mean_real_psav_gd, dcp_mean_real_psav_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_real_psav_nodef, &
                                                dcp_mean_real_psav_nodef_gd, dcp_mean_real_psav_nodef_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_real_psav_def, &
                                                dcp_mean_real_psav_def_gd, dcp_mean_real_psav_def_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_prod_mean, &
                                                dcp_prod_mean_gd, dcp_prod_mean_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_nodef, &
                                                dcp_mean_nodef_gd, dcp_mean_nodef_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_sdfl, &
                                                dcp_mean_sdfl_gd, dcp_mean_sdfl_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_defl, &
                                                dcp_mean_defl_gd, dcp_mean_defl_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_nodef_cov_sdfl_defl, &
                                                dcp_mean_nodef_cov_sdfl_defl_gd, dcp_mean_nodef_cov_sdfl_defl_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_sdfl_cov_nodef_defl, &
                                                dcp_mean_sdfl_cov_nodef_defl_gd, dcp_mean_sdfl_cov_nodef_defl_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_mean_defl_cov_sdfl_nodef, &
                                                dcp_mean_defl_cov_sdfl_nodef_gd, dcp_mean_defl_cov_sdfl_nodef_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_prod_mean_nodef_alt, &
                                                dcp_prod_mean_nodef_alt_gd, dcp_prod_mean_nodef_alt_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_prod_mean_indef_alt, &
                                                dcp_prod_mean_indef_alt_gd,dcp_prod_mean_indef_alt_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_cov_defl_sdfl_nodef_alt, &
                                                dcp_cov_defl_sdfl_nodef_alt_gd, dcp_cov_defl_sdfl_nodef_alt_bd
REAL (DP), DIMENSION(N_S_simple)            ::  decompose_cov_defl_sdfl_indef_alt, &
                                                dcp_cov_defl_sdfl_indef_alt_gd, dcp_cov_defl_sdfl_indef_alt_bd

REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  erg_sav_shock, &
                                                erg_sav_shock_gd, erg_sav_shock_bd, erg_sav_shock_bb
REAL (DP), DIMENSION (rgm,ngpb,ss)          ::  erg_sav_shock_nd, &
                                                erg_sav_shock_nd_gd, erg_sav_shock_nd_bd
REAL (DP), DIMENSION (ngpb)                 ::  erg_sav, &
                                                erg_sav_gd, erg_sav_bd
REAL (DP), DIMENSION (ngpb)                 ::  erg_sav_nd, &
                                                erg_sav_nd_gd, erg_sav_nd_bd
!REAL (DP), DIMENSION (ngpb,ss)              ::  realpdef

!! CALIBRATION GRIDS
REAL(DP), DIMENSION (N_BETA)        :: BETA_GRID		! CALIBRATION GRID - BETA
REAL(DP), DIMENSION (N_BETA_L)      :: BETA_L_GRID		! CALIBRATION GRID - BETA_L
REAL(DP), DIMENSION (N_SIGMA)       :: SIGMA_GRID	    ! CALIBRATION GRID - SIGMA
REAL(DP), DIMENSION (N_SIGMA_L)     :: IES_L_GRID	    ! CALIBRATION GRID - IES_L
REAL(DP), DIMENSION (N_SIGMA_L)     :: SIGMA_L_GRID	    ! CALIBRATION GRID - SIGMA_L
REAL(DP), DIMENSION (N_LAMBDA)      :: LAMBDA_GRID	    ! CALIBRATION GRID - LAMBDA
REAL(DP), DIMENSION (N_DELTA)       :: DELTA_GRID	    ! CALIBRATION GRID - DELTA
REAL(DP), DIMENSION (N_D0)          :: D0_GRID	        ! CALIBRATION GRID - D0
REAL(DP), DIMENSION (N_D1_H)        :: D1_H_GRID        ! CALIBRATION GRID - D1_H
REAL(DP), DIMENSION (N_D1_L)        :: D1_L_GRID        ! CALIBRATION GRID - D1_L
REAL(DP), DIMENSION (N_THETA)       :: THETA_GRID	    ! CALIBRATION GRID - THETA
REAL(DP), DIMENSION (N_COV_PIY)     :: COV_PIY_GRID	    ! CALIBRATION GRID - COV_PI


TYPE :: PARAMS_SET
    ! CALIBRATION PARAMETERS
	INTEGER    	:: CONFIG_VAL
    REAL(DP)    :: BITA_VAL
    REAL(DP)    :: BITA_L_VAL
    REAL(DP)    :: SIGMA_VAL
    REAL(DP)    :: IES_L_VAL 
    REAL(DP)    :: SIGMA_L_VAL     
    REAL(DP)    :: LAMBDA_VAL
    REAL(DP)    :: DELTA_VAL
	REAL(DP)    :: D0_VAL
	REAL(DP)    :: D1_H_VAL
	REAL(DP)    :: D1_L_VAL
    REAL(DP)    :: THETA_VAL
    REAL(DP)    :: COV_PIY_VAL  
    
    ! MODEL CONSTANTS
    REAL(DP)    :: MU_VAL = mu    
    
END TYPE PARAMS_SET
TYPE (PARAMS_SET)   :: PARAMS_VEC	! object containing all parameter configuration

TYPE :: COMP_SET
    ! COMPUTATION SET
	INTEGER    	:: CONV_VF_VAL 
    REAL(DP)    :: CHI_VAL
    REAL(DP)    :: ADJCOST_VAL
    INTEGER     :: NGPB_VAL
    INTEGER     :: NGPBPRIME_VAL
    INTEGER     :: SS_VAL
    INTEGER     :: I_B_ZERO_VAL    
END TYPE COMP_SET
TYPE (COMP_SET)   :: compconv	! object containing computation info


TYPE :: RESULTS_SIM   
    ! SIMULATION RESULTS - SUMMARY STATS
    ! Here we store model-generated moments
    REAL(DP)    :: OUTPUT_SIMUL_VAL             ! simulation average - output
    REAL(DP)    :: INFLATION_SIMUL_VAL          ! simulation average - inflation
    REAL(DP)    :: DEBT_IN_SIMUL_VAL            ! simulation average - incoming debt
    REAL(DP)    :: DEBT_OUT_SIMUL_VAL           ! simulation average - outgoing debt
    REAL(DP)    :: DPROB_SIMUL_VAL              ! simulation average - default probability
    REAL(DP)    :: SPREAD1_SIMUL_VAL            ! simulation average - spreads 1
    REAL(DP)    :: SPREAD__SIMUL_VAL            ! simulation average - spreads _
    REAL(DP)    :: SPREAD2_SIMUL_VAL            ! simulation average - spreads 2
    REAL(DP)    :: RFREE_SIMUL_VAL              ! simulation average - "risk free" interest rate
    REAL(DP)    :: SPREAD_GOOD_SIMUL_VAL        ! simulation average - spreads in good times 
    REAL(DP)    :: SPREAD_BAD_SIMUL_VAL         ! simulation average - spreads in bad times
    REAL(DP)    :: SPRDV2_GOOD_SIMUL_VAL        ! simulation average - spreads in good times 
    REAL(DP)    :: SPRDV2_BAD_SIMUL_VAL         ! simulation average - spreads in bad times
    REAL(DP)    :: RFREE_GOOD_SIMUL_VAL         ! simulation average - "risk free" interest rate in good times 
    REAL(DP)    :: RFREE_BAD_SIMUL_VAL          ! simulation average - "risk free" interest rate in bad times
    REAL(DP)    :: DPROB_GOOD_SIMUL_VAL         ! simulation average - def prob. in good times 
    REAL(DP)    :: DPROB_BAD_SIMUL_VAL          ! simulation average - def prob. in bad times
    REAL(DP)    :: SDVSPREAD1_SIMUL_VAL         ! simulation average - std dev spreads 1
    REAL(DP)    :: SDVSPREAD__SIMUL_VAL         ! simulation average - std dev spreads _
    REAL(DP)    :: SDVSPREAD2_SIMUL_VAL         ! simulation average - std dev spreads 2
    REAL(DP)    :: DDEBT_IN_SIMUL_VAL           ! simulation average - change in debt
    REAL(DP)    :: DSPREAD1_SIMUL_VAL           ! simulation average - change in spreads 1
    REAL(DP)    :: DSPREAD2_SIMUL_VAL           ! simulation average - change in spreads 2
    REAL(DP)    :: LDEBT_IN_SIMUL_VAL           ! simulation average - initial debt  
    REAL(DP)    :: REALPRICE_SIMUL_VAL          ! simulation average - real price    
    REAL(DP)    :: REALPRICE_GD_SIMUL_VAL       ! simulation average - real price     
    REAL(DP)    :: REALPRICE_BD_SIMUL_VAL       ! simulation average - real price 
    REAL(DP)    :: SDVREALP_SIMUL_VAL           ! simulation average - std dev real price
    
    REAL(DP)    :: VINFLATION_SIMUL_VAL         ! simulation average - inflation variance
    REAL(DP)    :: VAGGCONS_SIMUL_VAL           ! simulation average - consumption growth variance
    REAL(DP)    :: CINFLCONS_SIMUL_VAL          ! simulation average - infl. and cons. growth covariance
      
    REAL(DP)    :: MEAN_REAL_PSAV_SIMUL_VAL         ! simulation average - real price decomposition - real price
    REAL(DP)    :: MEAN_REAL_PSAV_NODEF_SIMUL_VAL   ! simulation average - real price decomposition - real price
    REAL(DP)    :: MEAN_REAL_PSAV_DEF_SIMUL_VAL     ! simulation average - real price decomposition - real price
    REAL(DP)    :: PROD_MEAN_SIMUL_VAL              ! simulation average - real price decomposition - product of means (sdf lender, deflation res, no default prob)
    REAL(DP)    :: MEAN_NODEF_SIMUL_VAL             ! simulation average - real price decomposition - means (no default prob)
    REAL(DP)    :: MEAN_SDFL_SIMUL_VAL              ! simulation average - real price decomposition - means (sdf lender)
    REAL(DP)    :: MEAN_DEFL_SIMUL_VAL              ! simulation average - real price decomposition - means (deflation res)
    REAL(DP)    :: MEAN_NODEF_CSDFLDEFL_SIMUL_VAL   ! simulation average - real price decomposition - sdf lender and deflation residual covariance    
    REAL(DP)    :: MEAN_SDFL_CNODEFDEFL_SIMUL_VAL   ! simulation average - real price decomposition - no default prob and deflation residual covariance
    REAL(DP)    :: MEAN_DEFL_CSDFLNODEF_SIMUL_VAL   ! simulation average - real price decomposition - sdf lender and no default prob covariance    

    REAL(DP)    :: PROD_MEAN_NODEF_SIMUL_VAL        ! simulation average - real price decomposition - product of means (sdf lender, deflation res)
    REAL(DP)    :: PROD_MEAN_INDEF_SIMUL_VAL        ! simulation average - real price decomposition - product of means (sdf lender, deflation res)
    REAL(DP)    :: COV_DEFL_SDFL_NODEF_SIMUL_VAL    ! simulation average - real price decomposition - sdf lender and defl covariance        
    REAL(DP)    :: COV_DEFL_SDFL_INDEF_SIMUL_VAL    ! simulation average - real price decomposition - sdf lender and defl covariance 
    
    
    REAL(DP)    :: SDVREALP_GD_SIMUL_VAL           ! simulation average - std dev real price
    REAL(DP)    :: SDVREALP_BD_SIMUL_VAL            ! simulation average - std dev real price
              
    REAL(DP)    :: MEAN_REAL_PSAV_GD_SIMUL_VAL         ! simulation average - real price decomposition - real price
    REAL(DP)    :: MEAN_REAL_PSAV_NODEF_GD_SIMUL_VAL   ! simulation average - real price decomposition - real price
    REAL(DP)    :: MEAN_REAL_PSAV_DEF_GD_SIMUL_VAL     ! simulation average - real price decomposition - real price
    REAL(DP)    :: PROD_MEAN_GD_SIMUL_VAL              ! simulation average - real price decomposition - product of means (sdf lender, deflation res, no default prob)
    REAL(DP)    :: MEAN_NODEF_GD_SIMUL_VAL             ! simulation average - real price decomposition - means (no default prob)
    REAL(DP)    :: MEAN_SDFL_GD_SIMUL_VAL              ! simulation average - real price decomposition - means (sdf lender)
    REAL(DP)    :: MEAN_DEFL_GD_SIMUL_VAL              ! simulation average - real price decomposition - means (deflation res)
    REAL(DP)    :: MEAN_NODEF_CSDFLDEFL_GD_SIMUL_VAL   ! simulation average - real price decomposition - sdf lender and deflation residual covariance    
    REAL(DP)    :: MEAN_SDFL_CNODEFDEFL_GD_SIMUL_VAL   ! simulation average - real price decomposition - no default prob and deflation residual covariance
    REAL(DP)    :: MEAN_DEFL_CSDFLNODEF_GD_SIMUL_VAL   ! simulation average - real price decomposition - sdf lender and no default prob covariance    

    REAL(DP)    :: PROD_MEAN_NODEF_GD_SIMUL_VAL        ! simulation average - real price decomposition - product of means (sdf lender, deflation res)
    REAL(DP)    :: PROD_MEAN_INDEF_GD_SIMUL_VAL        ! simulation average - real price decomposition - product of means (sdf lender, deflation res)
    REAL(DP)    :: COV_DEFL_SDFL_NODEF_GD_SIMUL_VAL    ! simulation average - real price decomposition - sdf lender and defl covariance        
    REAL(DP)    :: COV_DEFL_SDFL_INDEF_GD_SIMUL_VAL    ! simulation average - real price decomposition - sdf lender and defl covariance 
       
          
    REAL(DP)    :: MEAN_REAL_PSAV_BD_SIMUL_VAL         ! simulation average - real price decomposition - real price
    REAL(DP)    :: MEAN_REAL_PSAV_NODEF_BD_SIMUL_VAL   ! simulation average - real price decomposition - real price
    REAL(DP)    :: MEAN_REAL_PSAV_DEF_BD_SIMUL_VAL     ! simulation average - real price decomposition - real price
    REAL(DP)    :: PROD_MEAN_BD_SIMUL_VAL              ! simulation average - real price decomposition - product of means (sdf lender, deflation res, no default prob)
    REAL(DP)    :: MEAN_NODEF_BD_SIMUL_VAL             ! simulation average - real price decomposition - means (no default prob)
    REAL(DP)    :: MEAN_SDFL_BD_SIMUL_VAL              ! simulation average - real price decomposition - means (sdf lender)
    REAL(DP)    :: MEAN_DEFL_BD_SIMUL_VAL              ! simulation average - real price decomposition - means (deflation res)
    REAL(DP)    :: MEAN_NODEF_CSDFLDEFL_BD_SIMUL_VAL   ! simulation average - real price decomposition - sdf lender and deflation residual covariance    
    REAL(DP)    :: MEAN_SDFL_CNODEFDEFL_BD_SIMUL_VAL   ! simulation average - real price decomposition - no default prob and deflation residual covariance
    REAL(DP)    :: MEAN_DEFL_CSDFLNODEF_BD_SIMUL_VAL   ! simulation average - real price decomposition - sdf lender and no default prob covariance    

    REAL(DP)    :: PROD_MEAN_NODEF_BD_SIMUL_VAL        ! simulation average - real price decomposition - product of means (sdf lender, deflation res)
    REAL(DP)    :: PROD_MEAN_INDEF_BD_SIMUL_VAL        ! simulation average - real price decomposition - product of means (sdf lender, deflation res)
    REAL(DP)    :: COV_DEFL_SDFL_NODEF_BD_SIMUL_VAL    ! simulation average - real price decomposition - sdf lender and defl covariance        
    REAL(DP)    :: COV_DEFL_SDFL_INDEF_BD_SIMUL_VAL    ! simulation average - real price decomposition - sdf lender and defl covariance 
    
!!	REAL(DP)    :: DEBT_IN_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime       
	REAL(DP)    :: DEBT_IN_H_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime     
	REAL(DP)    :: DEBT_IN_L_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime     
!!	REAL(DP)    :: DEBT_OUT_SIMUL_VAL                   ! simulation average -- (good/bad) time x (high/low) regime      
	REAL(DP)    :: DEBT_OUT_H_SIMUL_VAL                 ! simulation average -- (good/bad) time x (high/low) regime    
	REAL(DP)    :: DEBT_OUT_L_SIMUL_VAL                 ! simulation average -- (good/bad) time x (high/low) regime      
!!	REAL(DP)    :: DPROB_SIMUL_VAL                      ! simulation average -- (good/bad) time x (high/low) regime         
	REAL(DP)    :: DPROB_H_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime       
	REAL(DP)    :: DPROB_L_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime       
!!	REAL(DP)    :: SPREAD1_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime       
	REAL(DP)    :: SPREAD1_H_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime     
	REAL(DP)    :: SPREAD1_L_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime     
!!	REAL(DP)    :: SPREAD2_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime       
	REAL(DP)    :: SPREAD2_H_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime     
	REAL(DP)    :: SPREAD2_L_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime 
!!	REAL(DP)    :: RFREE_SIMUL_VAL                      ! simulation average -- (good/bad) time x (high/low) regime       
	REAL(DP)    :: RFREE_H_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime       
	REAL(DP)    :: RFREE_L_SIMUL_VAL                    ! simulation average -- (good/bad) time x (high/low) regime          
!!	REAL(DP)    :: REALPRICE_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime          
	REAL(DP)    :: REALPRICE_H_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime        
	REAL(DP)    :: REALPRICE_L_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime   
!!	REAL(DP)    :: RFREE_GOOD_SIMUL_VAL                 ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: RFREE_GOOD_H_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: RFREE_GOOD_L_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
!!	REAL(DP)    :: RFREE_BAD_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime	    
	REAL(DP)    :: RFREE_BAD_H_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime   
	REAL(DP)    :: RFREE_BAD_L_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime   
!!	REAL(DP)    :: SPREAD_GOOD_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPREAD_GOOD_H_SIMUL_VAL              ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPREAD_GOOD_L_SIMUL_VAL              ! simulation average -- (good/bad) time x (high/low) regime	
!!	REAL(DP)    :: SPREAD_BAD_SIMUL_VAL                 ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPREAD_BAD_H_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPREAD_BAD_L_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
!!	REAL(DP)    :: SPRDV2_GOOD_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPRDV2_GOOD_H_SIMUL_VAL              ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPRDV2_GOOD_L_SIMUL_VAL              ! simulation average -- (good/bad) time x (high/low) regime	
!!	REAL(DP)    :: SPRDV2_BAD_SIMUL_VAL                 ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPRDV2_BAD_H_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: SPRDV2_BAD_L_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
!!	REAL(DP)    :: DPROB_GOOD_SIMUL_VAL                 ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: DPROB_GOOD_H_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
	REAL(DP)    :: DPROB_GOOD_L_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime	
!!	REAL(DP)    :: DPROB_BAD_SIMUL_VAL                  ! simulation average -- (good/bad) time x (high/low) regime     
	REAL(DP)    :: DPROB_BAD_H_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime   
	REAL(DP)    :: DPROB_BAD_L_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime        
!!  REAL(DP)    :: REALPRICE_SIMUL_VAL                  ! simulation average - real price     
    REAL(DP)    :: REALPRICE_GD_H_SIMUL_VAL             ! simulation average - real price     
    REAL(DP)    :: REALPRICE_BD_H_SIMUL_VAL             ! simulation average - real price    
    REAL(DP)    :: REALPRICE_GD_L_SIMUL_VAL             ! simulation average - real price     
    REAL(DP)    :: REALPRICE_BD_L_SIMUL_VAL             ! simulation average - real price     
    
    REAL(DP)    :: VBORR_SIMUL_VAL              	    ! simulation average - 
    REAL(DP)    :: VBORR_GOOD_SIMUL_VAL         	    ! simulation average - good times 
    REAL(DP)    :: VBORR_BAD_SIMUL_VAL          	    ! simulation average - bad times
    REAL(DP)    :: VBORR_BB_SIMUL_VAL          	        ! simulation average - bad times x high default prob
    REAL(DP)    :: VBORR_H_SIMUL_VAL               	    ! simulation average -- (high/low) regime       
    REAL(DP)    :: VBORR_L_SIMUL_VAL               	    ! simulation average -- (high/low) regime   
    REAL(DP)    :: VBORR_GOOD_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime	
    REAL(DP)    :: VBORR_GOOD_L_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime	
    REAL(DP)    :: VBORR_BAD_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime   
    REAL(DP)    :: VBORR_BAD_L_SIMUL_VAL           	    ! simulation average -- (good/bad) time x (high/low) regime   
    REAL(DP)    :: VBORR_BB_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime x high default prob
    REAL(DP)    :: VBORR_BB_L_SIMUL_VAL           	    ! simulation average -- (good/bad) time x (high/low) regime x high default prob


    REAL(DP)    :: VLEND_SIMUL_VAL              	    ! simulation average - 
    REAL(DP)    :: VLEND_GOOD_SIMUL_VAL         	    ! simulation average - good times 
    REAL(DP)    :: VLEND_BAD_SIMUL_VAL          	    ! simulation average - bad times
    REAL(DP)    :: VLEND_BB_SIMUL_VAL            	    ! simulation average - bad times x high default prob
    REAL(DP)    :: VLEND_H_SIMUL_VAL               	    ! simulation average -- (high/low) regime       
    REAL(DP)    :: VLEND_L_SIMUL_VAL               	    ! simulation average -- (high/low) regime   
    REAL(DP)    :: VLEND_GOOD_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime	
    REAL(DP)    :: VLEND_GOOD_L_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime	
    REAL(DP)    :: VLEND_BAD_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime   
    REAL(DP)    :: VLEND_BAD_L_SIMUL_VAL           	    ! simulation average -- (good/bad) time x (high/low) regime
    REAL(DP)    :: VLEND_BB_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime x high default prob
    REAL(DP)    :: VLEND_BB_L_SIMUL_VAL           	    ! simulation average -- (good/bad) time x (high/low) regime x high default prob


    REAL(DP)    :: VVLEND_SIMUL_VAL              	    ! simulation average - 
    REAL(DP)    :: VVLEND_GOOD_SIMUL_VAL         	    ! simulation average - good times 
    REAL(DP)    :: VVLEND_BAD_SIMUL_VAL          	    ! simulation average - bad times
    REAL(DP)    :: VVLEND_BB_SIMUL_VAL          	    ! simulation average - bad times x high default prob
    REAL(DP)    :: VVLEND_H_SIMUL_VAL              	    ! simulation average -- (high/low) regime       
    REAL(DP)    :: VVLEND_L_SIMUL_VAL                   ! simulation average -- (high/low) regime   
    REAL(DP)    :: VVLEND_GOOD_H_SIMUL_VAL              ! simulation average -- (good/bad) time x (high/low) regime	
    REAL(DP)    :: VVLEND_GOOD_L_SIMUL_VAL              ! simulation average -- (good/bad) time x (high/low) regime	
    REAL(DP)    :: VVLEND_BAD_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime   
    REAL(DP)    :: VVLEND_BAD_L_SIMUL_VAL               ! simulation average -- (good/bad) time x (high/low) regime
    REAL(DP)    :: VVLEND_BB_H_SIMUL_VAL          	    ! simulation average -- (good/bad) time x (high/low) regime x high default prob
    REAL(DP)    :: VVLEND_BB_L_SIMUL_VAL                ! simulation average -- (good/bad) time x (high/low) regime x high default prob
    
END TYPE RESULTS_SIM

TYPE (RESULTS_SIM)  :: results_simple           ! RESULTS - simple

END MODULE GLOBAL