MODULE PROCESS

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! PROGRAM: PROCESS                                 !!
    !!                                                  !!
    !! Date: Sep 13, 2014                               !!
    !! Last: Feb 02 2018                                !!  
    !!                                                  !!
    !! Profile:                                         !!
    !!  UTILITY(c)                                      !!
    !!  UTILITYPRIME(c)                                 !!
    !!  UTILITY_L(c)                                    !!
    !!                                                  !!
    !!  BONDGRID(b_min,b_max)                           !!
    !!  BONDGRID_PIVOT(b_min,b_max,b_pivot,a_pivot)     !!
    !!  BPRIMEGRID(b_min,b_max)                         !!
    !!  BPRIMEGRID_PIVOT(b_min,b_max,b_pivot,a_pivot)   !!
    !!  CREATEGRID(grid, siz, grid_min, grid_pivot, &   !!
    !!                  grid_max, curvature)            !!
    !!                                                  !!
    !!  PENALTY(yshock)                                 !!
    !!  ADJCOST(oldb,newb,chi)                          !!
    !!                                                  !!
    !!  TAUCHEN(rho,sigma_rho,N,m_tau,shocks,p_tran)    !!
    !!  BI_TAUCHEN(A,varcovmat,N1,N2,m_tau,grid1, &     !!
    !!              grid2,p_tran,trace_test)            !!
    !!  BI_TAUCHEN_ALT(grid1,grid2,p_tran, &            !! 
    !!              A,varcov_in,N1,N2,m_tau_o, &        !!
    !!              N1_cut1_o,N1_cut2_o, &              !!
    !!              m_tau_cut1_o,m_tau_cut2_o)          !!
    !!                                                  !!
    !!  INTERPOLATE(x,y,siz,xi)                         !!
    !!  INTERPOLATEVEC(x,y,siz,xi,sizxi)                !!
    !!                                                  !!
    !!  COVARIANCE(a, b, n, mask)                       !!
    !!  AVG(a, n, mask)                                 !!
    !!  AVGBLK(A,N,M,mask)                              !!
    !!                                                  !!
    !!  AMOTRY(p,y,psum,mp,np,ndim,funk,ihi,fac)        !!
    !!  AMOEBA(p,y,mp,np,ndim,ftol,funk,iter,ITMAX)     !!
    !!                                                  !!
    !! Changes:                                         !!
    !! >> 02/09/2015                                    !!
    !!  bondgrid(b_min,b_max)                           !!
    !!  bprimegrid(b_min,b_max)                         !!
    !!  fixed bug for zero debt index when bmax = 0     !!
    !!                                                  !!
    !! >> 03/09/2015                                    !!
    !!  adjcost(oldb,newb)                              !!
    !!                                                  !!
    !! >> 05/15/2015                                    !!
    !!  adjcost(oldb,newb,chi)                          !!     
    !!                                                  !!
    !! >> 05/20/2015                                    !!
    !!  creategrid (grid, siz, grid_min, &              !!
    !!              grid_pivot, grid_max, curvature)    !!
    !! >> 09/10/2015                                    !!
    !!  created spells and frames to extract windows    !!
    !!  compute stats by simulated windows              !!
    !!                                                  !!
    !! >> 09/01/2016                                    !!
    !!  removed the code for spells and frames          !!
    !!                                                  !!
    !! >> 09/23/2016                                    !!
    !!  restored the code for spells and frames         !!    
    !!                                                  !!
    !! >> 11/15/2016                                    !!
    !!  amoeba & amotry                                 !!   
    !!                                                  !!
    !! >> 12/08/2016                                    !!
    !!  stop exit instead of pause in amoeba if itmax   !!  
    !!                                                  !!
    !! >> 12/29/2016                                    !!
    !!  new bi_tauchen method -- SH version             !!      
    !!                                                  !!
    !! >> 09/14/2017                                    !!
    !!  updated -- LT debt implementation               !!
    !!  updated -- removed spells code                  !!     
    !!                                                  !!
    !! >> 01/16/2018                                    !!
    !!  added AVGBLK function for block stats           !!
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    USE GLOBAL
    USE PARAMETERS
	USE LAPACK95

    IMPLICIT NONE 

    CONTAINS    
    
ELEMENTAL FUNCTION utility(c)       !! utility function
    REAL (DP), INTENT(IN)    :: c
    REAL (DP)                :: utility
    
    IF (c>zerodp) THEN
        IF (sigma == onedp) THEN
            utility  = LOG(c)
        ELSE        
            utility  = (c**(1-sigma))/(1-sigma)
        END IF
    ELSE
        utility = -infty
    END IF
    
END FUNCTION utility

ELEMENTAL FUNCTION utilityprime(c)  !! derivative of utility function
    REAL (DP), INTENT(IN)    :: c
    REAL (DP)                :: utilityprime
    IF (c > zerodp) THEN
        utilityprime  = c**(-sigma)
    ELSE
        utilityprime  = infty
    END IF
END FUNCTION utilityprime

ELEMENTAL FUNCTION utility_l(c)     !! utility function
    REAL (DP), INTENT(IN)    :: c
    REAL (DP)                :: utility_l
    IF (c>zerodp) THEN
        IF (sigma == onedp) THEN
            utility_l  = LOG(c)
        ELSE        
            utility_l  = (c**(1-sigma_l))/(1-sigma_l)
        END IF
    ELSE
        utility_l = -infty
    END IF
    
END FUNCTION utility_l

ELEMENTAL FUNCTION utilityprime_l(c) !! derivative of utility function
    REAL (DP), INTENT(IN)    :: c
    REAL (DP)                :: utilityprime_l
    IF (c > zerodp) THEN
        utilityprime_l  = c**(-sigma_l)
    ELSE
        utilityprime_l  = infty
    END IF
END FUNCTION utilityprime_l

ELEMENTAL FUNCTION penalty(yshock,regime)  !! default penalty function
    REAL (DP), INTENT(IN)   :: yshock
    INTEGER, INTENT(IN)     :: regime
    REAL (DP)               :: penalty

    REAL (DP)               :: d1
    
    d1      = d1_rgm(regime)
    
    penalty = MAX( zerodp,  (d1/d0)*yshock           &
                    + (d1 - (d1/d0))*yshock**2 )
    
END FUNCTION penalty

ELEMENTAL FUNCTION adjcost(oldb,newb,chi)   !! adjustment cost function
    REAL (DP), INTENT(IN)    :: oldb
    REAL (DP), INTENT(IN)    :: newb
    REAL (DP), INTENT(IN)    :: chi
    REAL (DP)                :: adjcost
    
    adjcost  =  chi*(oldb-newb)**chi_quad
    
END FUNCTION adjcost

SUBROUTINE  bondgrid(b_min,b_max)            !! creates bond grid
    INTEGER                                 :: i, n_neg, n_pos
    REAL (DP), DIMENSION(:), ALLOCATABLE     :: stops_neg, stops_pos
    REAL (DP), INTENT(IN)                    :: b_min,b_max
    
    n_neg       = FLOOR(ngpb*(-b_min/(b_max-b_min)))
    n_pos       = ngpb - n_neg + 1
    
    IF (n_pos>1) THEN
        ALLOCATE(stops_neg(n_neg), stops_pos(n_pos))
        FORALL (i=1:n_neg) stops_neg(i) = REAL((i-1))/(n_neg-1)     ! even grid between 0 and 1
        FORALL (i=1:n_pos) stops_pos(i) = REAL((i-1))/(n_pos-1)     ! even grid between 0 and 1

        stops_neg   = stops_neg**grdpwr                             ! distorts grid between 0 and 1
        stops_pos   = stops_pos**grdpwr                             ! distorts grid between 0 and 1
    
        FORALL (i=1:n_neg) b(n_neg - i + 1) = (b_min)*stops_neg(i)  ! points from bmin < 0 to 0
        i_b_zero       = n_neg + 1                                  ! sets index for b(i_b_zero) = 0
        FORALL (i=1:n_pos) b(i + n_neg - 1) = (b_max)*stops_pos(i)  ! points from 0 to bmax
    
        DEALLOCATE(stops_neg, stops_pos)
    ELSE
        ALLOCATE(stops_neg(ngpb))
        FORALL (i=1:ngpb) stops_neg(i) = REAL((i-1))/(ngpb-1)       ! even grid between 0 and 1

        stops_neg   = stops_neg**grdpwr                             ! distorts grid between 0 and 1
    
        FORALL (i=1:ngpb) b(ngpb - i + 1) = (b_min)*stops_neg(i)    ! points from bmin < 0 to 0
        i_b_zero       = ngpb                                       ! sets index for b(i_b_zero) = 0
    
        DEALLOCATE(stops_neg)
        
    END IF
    

END SUBROUTINE bondgrid

SUBROUTINE  bondgrid_pivot(b_min,b_max,b_pivot,a_pivot)            !! creates bond grid

    REAL (DP), INTENT(IN)                   :: b_min,b_max
    REAL (DP), INTENT(IN), OPTIONAL         :: b_pivot
    REAL (DP), INTENT(IN), OPTIONAL         :: a_pivot
    
    INTEGER                                 :: i, n_neg, n_pos
    REAL (DP)                               :: b_pvt
    REAL (DP)                               :: a_pvt
    REAL (DP), DIMENSION(:), ALLOCATABLE    :: stops_neg, stops_pos   

    ! Program
    IF (PRESENT(b_pivot)) THEN
        b_pvt = b_pivot
    ELSE 
        b_pvt = zerodp
    END IF
    
    IF (PRESENT(a_pivot)) THEN
        a_pvt = a_pivot
    ELSE 
        a_pvt = onedp
    END IF 
    
    CALL creategrid(b, ngpb, b_min, b_pvt, b_max, grdpwr, a_pvt)
    
    i_b_zero       = MINVAL(MINLOC(b, MASK = (b .GE. zerodp)))

END SUBROUTINE bondgrid_pivot

SUBROUTINE  bprimegrid_pivot(b_min,b_max,b_pivot,a_pivot)            !! creates bond grid

    REAL (DP), INTENT(IN)                   :: b_min,b_max
    REAL (DP), INTENT(IN), OPTIONAL         :: b_pivot
    REAL (DP), INTENT(IN), OPTIONAL         :: a_pivot
    
    INTEGER                                 :: i, n_neg, n_pos
    REAL (DP)                               :: b_pvt
    REAL (DP)                               :: a_pvt
    REAL (DP), DIMENSION(:), ALLOCATABLE    :: stops_neg, stops_pos    

    ! Program
    IF (PRESENT(b_pivot)) THEN
        b_pvt = b_pivot
    ELSE 
        b_pvt = zerodp
    END IF
    
    IF (PRESENT(a_pivot)) THEN
        a_pvt = a_pivot
    ELSE 
        a_pvt = onedp
    END IF    
    
    CALL creategrid(bprime, ngpbprime, b_min, b_pvt, b_max, grdpwr, a_pvt)
    
    izerobp       = MINVAL(MINLOC(b, MASK = (b .GE. zerodp)))

END SUBROUTINE bprimegrid_pivot

SUBROUTINE  bprimegrid(b_min,b_max)          !! creates bprime grid
    INTEGER                                 :: i, n_neg, n_pos
    REAL (DP), DIMENSION(:), ALLOCATABLE     :: stops_neg, stops_pos
    REAL (DP), INTENT(IN)                    :: b_min,b_max
    
    n_neg       = FLOOR(ngpbprime*(-b_min/(b_max-b_min)))
    n_pos       = ngpbprime - n_neg + 1
    
    IF (n_pos>1) THEN
        ALLOCATE(stops_neg(n_neg), stops_pos(n_pos))
    
        FORALL (i=1:n_neg) stops_neg(i) = REAL((i-1))/(n_neg-1)     ! even grid between 0 and 1
        FORALL (i=1:n_pos) stops_pos(i) = REAL((i-1))/(n_pos-1)     ! even grid between 0 and 1
    
        stops_neg   = stops_neg**grdpwr                             ! distorts grid between 0 and 1
        stops_pos   = stops_pos**grdpwr                             ! distorts grid between 0 and 1
    
        FORALL (i=1:n_neg) bprime(n_neg - i + 1) = (b_min)*stops_neg(i)     ! points from bmin < 0 to 0
        izerobp     = n_neg + 1                                             ! sets index for b(izero) = 0
        FORALL (i=1:n_pos) bprime(i + n_neg - 1) = (b_max)*stops_pos(i)     ! points from 0 to bmax
    
        DEALLOCATE(stops_neg, stops_pos)
    ELSE
        ALLOCATE(stops_neg(ngpbprime))
        FORALL (i=1:ngpbprime) stops_neg(i) = REAL((i-1))/(ngpbprime-1)       ! even grid between 0 and 1

        stops_neg   = stops_neg**grdpwr                             ! distorts grid between 0 and 1
    
        FORALL (i=1:ngpbprime) bprime(ngpbprime - i + 1) = (b_min)*stops_neg(i)    ! points from bmin < 0 to 0
        izerobp       = ngpbprime                                       ! sets index for b(i_b_zero) = 0
    
        DEALLOCATE(stops_neg)
        
    END IF
END SUBROUTINE bprimegrid

SUBROUTINE  creategrid(grid, siz, grid_min, grid_pivot, grid_max, curvature, adj_pivot)
	!! This SUBROUTINE fills the array grid 
	!! with siz points
	!! from grid_min to grid_max
	!! and a curvature around grid_pivot
	
	! INPUT & OUTPUT
    INTEGER, INTENT(IN) 					:: siz
	REAL(DP), INTENT(IN) 					:: grid_min, grid_max, grid_pivot
	REAL(DP), INTENT(IN) 					:: curvature
    REAL(DP), INTENT(IN), OPTIONAL          :: adj_pivot
    
	REAL(DP), DIMENSION(SIZ), INTENT(INOUT) :: grid
	
	! LOCAL VARIABLES
	INTEGER                                 :: i, n_med
    REAL (DP)                               :: adj_pvt
     
	! PROGRAM
    IF (PRESENT(adj_pivot)) THEN
        adj_pvt = adj_pivot
    ELSE 
        adj_pvt = onedp
    END IF
    
    ! initialization
    grid        =   grid_min
    
    IF_SINGLE: IF (siz .EQ. 1) THEN        
        ! if size of the grid is one
        ! then set to lower bound
        grid        =   grid_min
    ELSE

        n_med       = MAX(1,CEILING(adj_pvt*siz*((grid_pivot - grid_min)/ &
                            (grid_max - grid_min))))

        IF (n_med .GT. 1) THEN
            FORALL (i=1:n_med) grid(i) = DBLE(i-1)/DBLE(n_med-1)  	! [0,1] grid evenly split
            grid(1:n_med)	= 1.0_DP - grid(1:n_med)                ! [1,0] grid evenly split
            grid(1:n_med)	= 1.0_DP - grid(1:n_med)**curvature    	! [0,1] grid clustered at 1
            FORALL (i=1:n_med) grid(i) = grid_min + & 
            (grid_pivot-grid_min)*grid(i)   ! grid_min to grid_pivot, clustered
        END IF

        IF (n_med .LT. siz) THEN
            FORALL (i=n_med:siz) grid(i) = DBLE(i-n_med)/DBLE(siz-n_med)   ! even grid between 0 and 1
            grid(n_med:siz)	= grid(n_med:siz)**curvature            ! [0,1] grid clustered at 0
            FORALL (i=n_med:siz) grid(i) = grid_pivot + & 
            (grid_max-grid_pivot)*grid(i)   ! grid_min to grid_pivot, clustered
        END IF

    END IF IF_SINGLE
														
END SUBROUTINE creategrid

SUBROUTINE  TAUCHEN(rho,sigma_rho,N,m_tau,shocks,p_tran)

    INTEGER, INTENT(IN)     :: N
    REAL (DP), INTENT(IN)    :: rho, sigma_rho, m_tau
    REAL (DP), INTENT(OUT), DIMENSION(N)     :: shocks
    REAL (DP), INTENT(OUT), DIMENSION(N,N)   :: p_tran
    INTEGER                 :: i, j
    REAL (DP)                :: d, norm1, norm2
    
    shocks(sy) = m_tau*sigma_rho/(1-rho**2)**(0.5)
    shocks(1) = -shocks(N)
    d=(shocks(N)-shocks(1))/(N-1);
    
    DO i=2,N-1
        shocks(i) = shocks(i-1) + d
    END DO

    DO i=1,N
        DO j=2,N-1
            CALL VDCDFNORM(1,(shocks(j)+d/2-rho*shocks(i))/sigma_rho,norm1)
            CALL VDCDFNORM(1,(shocks(j)-d/2-rho*shocks(i))/sigma_rho,norm2)
            p_tran(i,j) = norm1-norm2
        END DO
        CALL VDCDFNORM(1,(shocks(1)+d/2-rho*shocks(i))/sigma_rho,norm1)
        CALL VDCDFNORM(1,(shocks(N)-d/2-rho*shocks(i))/sigma_rho,norm2)
        p_tran(i,1) = norm1
        p_tran(i,N) = 1-norm2
    END DO
END SUBROUTINE TAUCHEN

SUBROUTINE  BI_TAUCHEN(A,varcovmat,N1,N2,m_tau,grid1,grid2,p_tran,trace_test)
    
    !Z_t = AZ_{t-1} + Epsilon where Epsilon has variance-covariance matrix varcovmat
    
    !USE LIN_EIG_SELF_INT
    
    INTEGER, INTENT(IN)     :: N1, N2, trace_test
    REAL (DP), INTENT(IN)    :: m_tau
    REAL (DP), INTENT(IN), DIMENSION(2,2)            :: A, varcovmat
    REAL (DP), INTENT(OUT), DIMENSION(N1*N2)         :: grid1, grid2
    REAL (DP), INTENT(OUT), DIMENSION(N1*N2,N1*N2)   :: p_tran
    REAL (DP), DIMENSION(N1)                         :: tildegrid1
    REAL (DP), DIMENSION(N2)                         :: tildegrid2
    REAL (DP), DIMENSION(N1,N2)                      :: grid1_aux, grid2_aux
    REAL (DP), DIMENSION(N1,N2,N1,N2)                :: ptran_aux
    REAL (DP), DIMENSION(2,2)                        :: Lambda, Q, Atilde, QLQp
    REAL (DP), DIMENSION(2)                          :: d
    INTEGER                 :: i, j, k, l, ind1, ind2
    REAL (DP)                :: d1, d2, norm1, norm2, var_ztilde1, var_ztilde2, Pr1, Pr2
    
    ! Compute diagonal matrix (Lambda) and matrix of eigenvectors (Q)
    !CALL LIN_EIG_SELF(varcovmat, d, v=Q)
    Q = varcovmat
    CALL SYEVD( Q , d ,'V')

    Lambda = zerodp
    DO i=1,2
         Lambda(i,i) = d(i)
         IF (Lambda(i,i) .LE. 0) THEN
             PRINT *, 'Lambda is negative!'
             PAUSE
         END IF
    END DO
    
    ! Transform process Ztilde_t = AtildeZtilde_{t-1} + Epsilontilde (see appendix)
    Atilde = MATMUL(MATMUL(TRANSPOSE(Q),A),Q)
   
    ! Compute unconditional variance of ztilde1 and ztilde2
    var_ztilde1 = MAX(eps,Lambda(2,2))*Atilde(1,2)**2/(1-Atilde(1,1)**2)/(1-Atilde(2,2)**2)  &
                + MAX(eps,Lambda(1,1))/(1-Atilde(1,1)**2)
    var_ztilde2 = MAX(eps,Lambda(1,1))*Atilde(2,1)**2/(1-Atilde(1,1)**2)/(1-Atilde(2,2)**2)  &
                + MAX(eps,Lambda(2,2))/(1-Atilde(2,2)**2)
    
        
    ! Tauchen suggests to select endpoints of the grid to be a 
    ! multiple m of the unconditional standard deviation.
    tildegrid1(N1)  = m_tau*SQRT(var_ztilde1)
    tildegrid1(1)   = -tildegrid1(N1)
    tildegrid2(N2)  = m_tau*SQRT(var_ztilde2)
    tildegrid2(1)   = -tildegrid2(N2)
    
    d1=(tildegrid1(N1)-tildegrid1(1))/(N1-1)
    d2=(tildegrid2(N2)-tildegrid2(1))/(N2-1)
    
    DO i=2,N1-1
        tildegrid1(i) = tildegrid1(i-1) + d1
    END DO
    
    DO i=2,N2-1
        tildegrid2(i) = tildegrid2(i-1) + d2
    END DO
    
    ! Compute transition matrix
    DO i=1,N1
        DO j=1,N2
            DO k=1,N1
                IF (k==1) THEN
                    CALL VDCDFNORM(1, ( tildegrid1(k) + d1/2 - Atilde(1,1)*tildegrid1(i) - Atilde(1,2)*tildegrid2(j) )/SQRT(MAX(eps,Lambda(1,1))), norm1)
                    norm2 = 0
                ELSE IF (k==N1) THEN
                    CALL VDCDFNORM(1, ( tildegrid1(k) - d1/2 - Atilde(1,1)*tildegrid1(i) - Atilde(1,2)*tildegrid2(j) )/SQRT(MAX(eps,Lambda(1,1))), norm2)
                    norm1 = 1
                ELSE
                    CALL VDCDFNORM(1, ( tildegrid1(k) + d1/2 - Atilde(1,1)*tildegrid1(i) - Atilde(1,2)*tildegrid2(j) )/SQRT(MAX(eps,Lambda(1,1))), norm1)
                    CALL VDCDFNORM(1, ( tildegrid1(k) - d1/2 - Atilde(1,1)*tildegrid1(i) - Atilde(1,2)*tildegrid2(j) )/SQRT(MAX(eps,Lambda(1,1))), norm2)
                END IF
                Pr1 = norm1 - norm2
                DO l=1,N2
                    IF (l==1) THEN
                        CALL VDCDFNORM(1, ( tildegrid2(l) + d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))),norm1)
                        norm2 = 0
                    ELSE IF (l==N2) THEN
                        CALL VDCDFNORM(1, ( tildegrid2(l) - d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))),norm2)
                        norm1 = 1
                    ELSE
                        CALL VDCDFNORM(1, ( tildegrid2(l) + d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))), norm1)
                        CALL VDCDFNORM(1, ( tildegrid2(l) - d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))), norm2)
                    END IF
                    Pr2 = norm1 - norm2
                    ptran_aux(i,j,k,l) = Pr1*Pr2                
                END DO
            END DO
        END DO  
    END DO
        
    ! Recover grids for z1 and z2
    DO i=1,N1
        DO j=1,N2
            grid1_aux(i,j) = DOT_PRODUCT( Q(1,:) , (/tildegrid1(i),tildegrid2(j)/) )
            grid2_aux(i,j) = DOT_PRODUCT( Q(2,:) , (/tildegrid1(i),tildegrid2(j)/) )
        END DO
    END DO

    p_tran = RESHAPE( ptran_aux, (/ N1*N2, N1*N2 /) )
    grid1 = RESHAPE( grid1_aux, (/ N1*N2 /) )
    grid2 = RESHAPE( grid2_aux, (/ N1*N2 /) )
    
    IF (trace_test) THEN
        100 FORMAT (1X, 2F10.6)
        
        PRINT *, 'Lambda = '
        WRITE(*,100) ( Lambda(1,i), i=1,2 )
        WRITE(*,100) ( Lambda(2,i), i=1,2 )
    
        PRINT *, 'Q = '
        WRITE(*,100) ( Q(1,i), i=1,2 )
        WRITE(*,100) ( Q(2,i), i=1,2 )
    
        QLQp = MATMUL(MATMUL(Q,Lambda),TRANSPOSE(Q))

        PRINT *, 'QLambdaQ'': '
        WRITE(*,100) ( QLQp(1,i),  i=1,2 )
        WRITE(*,100) ( QLQp(2,i), i=1,2 )
        
        PRINT *, 'Atilde (Q''AQ) = '
        WRITE(*,100) ( Atilde(1,i), i=1,2 )
        WRITE(*,100) ( Atilde(2,i), i=1,2 )
        
        PRINT *, 'var_ztilde1 = ',var_ztilde1
        PRINT *, 'var_ztilde2 = ',var_ztilde2
    
        PRINT *, 'tildegrid1: '
        PRINT *, tildegrid1
        PRINT *, 'tildegrid2: '
        PRINT *, tildegrid2
        
        PRINT *, 'grid1: '
        PRINT *, grid1
        PRINT *, 'grid2: '
        PRINT *, grid2
    END IF
    
END SUBROUTINE BI_TAUCHEN

SUBROUTINE  BI_TAUCHEN_ALT(grid1,grid2,p_tran,A,varcov_in,N1,N2,m_tau_o,N1_cut1_o,N1_cut2_o,m_tau_cut1_o,m_tau_cut2_o)
	!! This SUBROUTINE creates the grids and transition matrices for 2-D VAR(1) process
    ! Inputs: A,varcovmat,N1,N2,
    ! Optional inputs: m_tau_o, N1_cut1_o, N1_cut2_o, m_tau_cut1_o, m_tau_cut2_o
    ! Outputs: grid1, grid2, p_tran
	
	! INPUT & OUTPUT
    INTEGER, INTENT(IN)                             :: N1, N2
    REAL (DP), INTENT(IN), DIMENSION(2,2)           :: A, varcov_in
    REAL (DP), INTENT(OUT), DIMENSION(N1*N2)        :: grid1, grid2
    REAL (DP), INTENT(OUT), DIMENSION(N1*N2,N1*N2)  :: p_tran
    INTEGER                                         :: N1_cut1, N1_cut2
    REAL (DP)                                       :: m_tau, m_tau_cut1, m_tau_cut2
    INTEGER, INTENT(IN), OPTIONAL                   :: N1_cut1_o, N1_cut2_o
    REAL (DP), INTENT(IN), OPTIONAL                 :: m_tau_o, m_tau_cut1_o, m_tau_cut2_o

    
    REAL (DP), DIMENSION(N1)            :: tildegrid1
    REAL (DP), DIMENSION(N2)            :: tildegrid2
    REAL (DP), DIMENSION(N1,N2)         :: grid1_aux, grid2_aux
    REAL (DP), DIMENSION(N1,N2,N1,N2)   :: ptran_aux
    REAL (DP), DIMENSION(2,2)           :: Lambda, Atilde, varcovaux
    REAL (DP)                           :: covar
    
    INTEGER     :: i, j, k, l, ind1, ind2
    REAL (DP)   :: d1_d, d1_s1, d1_s2, d11, d12, d2
    REAL (DP)   :: norm1, norm2, var_ztilde1, var_ztilde2, Pr1, Pr2
    
    ! Program
    IF (PRESENT(m_tau_o)) THEN
        m_tau = m_tau_o
    ELSE 
        m_tau = 2.65_dp
    END IF
    IF (PRESENT(N1_cut1_o)) THEN
        N1_cut1 = N1_cut1_o
    ELSE 
        N1_cut1 = 1
    END IF
    IF (PRESENT(N1_cut2_o)) THEN
        N1_cut2 = N1_cut2_o
    ELSE 
        N1_cut2 = N1
    END IF
    IF (PRESENT(m_tau_cut1_o)) THEN
        m_tau_cut1 = m_tau_cut1_o
    ELSE 
        m_tau_cut1 = -m_tau
    END IF
    IF (PRESENT(m_tau_cut2_o)) THEN
        m_tau_cut2 = m_tau_cut2_o
    ELSE 
        m_tau_cut2 = m_tau
    END IF
    
    IF ((N1_cut2 .LE. N1_cut1).OR.(m_tau_cut2 .LE. m_tau_cut1)) THEN
        PRINT *, 'Error!'
        RETURN
    END IF
    
    
    ! First, turn off covariance
    ! This implies that diagonal matrix is the var/cov matrix 
    ! and the eigenvector matrix is the identity matrix
    varcovaux = varcov_in
    covar = varcovaux(2,1)
    varcovaux(1,2) = zerodp
    varcovaux(2,1) = zerodp
    Lambda = varcovaux
    Atilde = A
    
    !! Compute diagonal matrix (Lambda) and matrix of eigenvectors (Q)
    !CALL LIN_EIG_SELF(varcovmat, d, v=Q)
    !Q = varcovmat
    !CALL SYEVD( Q , d ,'V')
    !
    !Lambda = zerodp
    !DO i=1,2
    !     Lambda(i,i) = d(i)
    !     IF (Lambda(i,i) .LE. 0) THEN
    !         PRINT *, 'Lambda is negative!'
    !         PAUSE
    !     END IF
    !END DO
    !! Transform process Ztilde_t = AtildeZtilde_{t-1} + Epsilontilde (see appendix)
    !Atilde = MATMUL(MATMUL(TRANSPOSE(Q),A),Q)
   
    ! Compute unconditional variance of ztilde1 and ztilde2
    var_ztilde1 = MAX(eps,Lambda(2,2))*Atilde(1,2)**2/(1-Atilde(1,1)**2)/(1-Atilde(2,2)**2)  &
                + MAX(eps,Lambda(1,1))/(1-Atilde(1,1)**2)
    var_ztilde2 = MAX(eps,Lambda(1,1))*Atilde(2,1)**2/(1-Atilde(1,1)**2)/(1-Atilde(2,2)**2)  &
                + MAX(eps,Lambda(2,2))/(1-Atilde(2,2)**2)
        
    ! Select endpoints of grids to be a 
    ! multiple m of the unconditional standard deviation.
    tildegrid1(N1)  = m_tau*SQRT(var_ztilde1)
    tildegrid1(1)   = -tildegrid1(N1)
    tildegrid2(N2)  = m_tau*SQRT(var_ztilde2)
    tildegrid2(1)   = -tildegrid2(N2)
    
    tildegrid1(N1_cut1)  = m_tau_cut1*SQRT(var_ztilde1)
    tildegrid1(N1_cut2)  = m_tau_cut2*SQRT(var_ztilde1)
    
    ! dense distance
    d1_d = (tildegrid1(N1_cut2) - tildegrid1(N1_cut1))/DBLE(N1_cut2 - N1_cut1)

    ! sparse distance 1 
    IF (N1_cut1 - 1 > 0) THEN
        d1_s1 = (tildegrid1(N1_cut1) - tildegrid1(1))/DBLE(N1_cut1 - 1)
    ELSE
        d1_s1 = d1_d
    END IF

    ! sparse distance 2
    IF (N1 - N1_cut2 > 0) THEN
        d1_s2 = (tildegrid1(N1) - tildegrid1(N1_cut2))/DBLE(N1 - N1_cut2)
    ELSE
        d1_s2 = d1_d
    END IF
    
    d2 = (tildegrid2(N2) - tildegrid2(1))/(N2 - 1)
    
    DO i=2,N1_cut1-1
        tildegrid1(i) = tildegrid1(i-1) + d1_s1
    END DO
    
    DO i=N1_cut1+1,N1_cut2-1
        tildegrid1(i) = tildegrid1(i-1) + d1_d
    END DO
    
    DO i=N1_cut2+1,N1-1
        tildegrid1(i) = tildegrid1(i-1) + d1_s2
    END DO
    
    DO i=2,N2-1
        tildegrid2(i) = tildegrid2(i-1) + d2
    END DO
    
    ! Compute transition matrix
    DO i=1,N1
        DO j=1,N2
            DO k=1,N1
                IF (k==1) THEN
                    CALL VDCDFNORM(1, ( tildegrid1(k) + d1_s1/2 &
                        - Atilde(1,1)*tildegrid1(i)             &
                        - Atilde(1,2)*tildegrid2(j) )           &
                        /SQRT(MAX(eps,Lambda(1,1))), norm1)
                    norm2 = 0
                ELSE IF (k==N1) THEN
                    CALL VDCDFNORM(1, ( tildegrid1(k) - d1_s2/2 &
                        - Atilde(1,1)*tildegrid1(i)             &
                        - Atilde(1,2)*tildegrid2(j) )           &
                        /SQRT(MAX(eps,Lambda(1,1))), norm2)
                    norm1 = 1
                ELSE
                    IF (k<N1_cut1) THEN
                        d11 = d1_s1
                        d12 = d1_s1
                    ELSEIF (k.EQ.N1_cut1) THEN
                        d11 = d1_d
                        d12 = d1_s1
                    ELSEIF (k<N1_cut2) THEN
                        d11 = d1_d
                        d12 = d1_d
                    ELSEIF (k.EQ.N1_cut2) THEN
                        d11 = d1_s2
                        d12 = d1_d
                    ELSEIF (k<N1) THEN
                        d11 = d1_s2
                        d12 = d1_s2
                    END IF
                    CALL VDCDFNORM(1, ( tildegrid1(k) + d11/2   &
                        - Atilde(1,1)*tildegrid1(i)             &
                        - Atilde(1,2)*tildegrid2(j) )           &
                        /SQRT(MAX(eps,Lambda(1,1))), norm1)
                    CALL VDCDFNORM(1, ( tildegrid1(k) - d12/2   &
                        - Atilde(1,1)*tildegrid1(i)             &
                        - Atilde(1,2)*tildegrid2(j) )           &
                        /SQRT(MAX(eps,Lambda(1,1))), norm2)
                END IF
                Pr1 = norm1 - norm2
                DO l=1,N2
                    IF (l==1) THEN
                        CALL VDCDFNORM(1, ( tildegrid2(l) + d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))),norm1)
                        norm2 = 0
                    ELSE IF (l==N2) THEN
                        CALL VDCDFNORM(1, ( tildegrid2(l) - d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))),norm2)
                        norm1 = 1
                    ELSE
                        CALL VDCDFNORM(1, ( tildegrid2(l) + d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))), norm1)
                        CALL VDCDFNORM(1, ( tildegrid2(l) - d2/2 - Atilde(2,2)*tildegrid2(j) - Atilde(2,1)*tildegrid1(i) )/SQRT(MAX(eps,Lambda(2,2))), norm2)
                    END IF
                    Pr2 = norm1 - norm2
                    ptran_aux(i,j,k,l) = Pr1*Pr2                
                END DO
            END DO
        END DO  
    END DO
    
    ! Recover grids for z1 and z2, adjusting for covariance
    DO i=1,N1
        DO j=1,N2
            !grid1_aux(i,j) = DOT_PRODUCT( Q(1,:) , (/tildegrid1(i),tildegrid2(j)/) )
            !grid2_aux(i,j) = DOT_PRODUCT( Q(2,:) , (/tildegrid1(i),tildegrid2(j)/) )
            grid1_aux(i,j) = tildegrid1(i)
            grid2_aux(i,j) = tildegrid2(j) + 1.0D4*covar*tildegrid1(i)
        END DO
    END DO
    
    p_tran = RESHAPE( ptran_aux, (/ N1*N2, N1*N2 /) )
    grid1 = RESHAPE( grid1_aux, (/ N1*N2 /) )
    grid2 = RESHAPE( grid2_aux, (/ N1*N2 /) )
   
    
END SUBROUTINE BI_TAUCHEN_ALT

FUNCTION    INTERPOLATE(X,Y,siz,xi)
	!! Interpolates to find the values of the underlying function Y at the point xi.
	!! X and Y must be one-DIMENSIONal arrays with the same DIMENSION siz
	!! xi must be a scalar
	!! if xi exceeds the bounds of X, i.e. requiring extrapolation, then Yi is set to the closest point
	
	! INPUT & OUTPUT
    INTEGER, INTENT(IN) 					:: siz
    REAL(8), DIMENSION(siz), INTENT(IN)	    :: X, Y
    REAL(8), INTENT(IN)	                    :: xi
    REAL(8)				                    :: INTERPOLATE
    INTEGER								    :: ind, i
    
	! PROGRAM
	ind = MINLOC( ABS( xi - X ) , 1 )
	outer: IF ( xi >= X(ind) ) THEN
		inner1: IF  ( ind == siz ) THEN
			INTERPOLATE = Y(ind)
		ELSE
			INTERPOLATE =	&
				( Y(ind+1)*(xi-X(ind)) + Y(ind)*(X(ind+1)-xi) )	&
				/( X(ind+1) - X(ind) )
		END IF inner1
    ELSE ! IF ( xi < X(ind) )
		inner2: IF  ( ind == 1 ) THEN
			INTERPOLATE = Y(ind)
		ELSE
			INTERPOLATE =	&
				( Y(ind)*(xi-X(ind-1)) + Y(ind-1)*(X(ind)-xi) )	&
				/( X(ind) - X(ind-1) )
		END IF inner2
    END IF outer
	
END FUNCTION INTERPOLATE 

FUNCTION    INTERPOLATEVEC(X,Y,siz,xi,sizxi)
	!! Interpolates to find the values of the underlying function Y at the points of vector xi.
	!! X and Y must be one-DIMENSIONal arrays with the same length siz
	!! xi must be a one-DIMENSIONal array of length sizxi
	!! if xi exceeds the bounds of X, i.e. requiring extrapolation, then Yi is set to the closest point
	
	! INPUT & OUTPUT
    INTEGER, INTENT(IN) 					:: siz, sizxi
    REAL(8), DIMENSION(siz), INTENT(IN)	    :: X, Y
    REAL(8), DIMENSION(sizxi), INTENT(IN)	:: xi
    REAL(8), DIMENSION(sizxi)               :: INTERPOLATEVEC
    INTEGER								    :: ind, i
    
	! PROGRAM
    DO i=1,sizxi
	    ind = MINLOC( ABS( xi(i) - X ) , 1 )
	    outer: IF ( xi(i) >= X(ind) ) THEN
		    inner1: IF  ( ind == siz ) THEN
			    INTERPOLATEVEC = Y(ind)
		    ELSE
			    INTERPOLATEVEC =	&
				    ( Y(ind+1)*(xi(i)-X(ind)) + Y(ind)*(X(ind+1)-xi(i)) )	&
				    /( X(ind+1) - X(ind) )
		    END IF inner1
        ELSE ! IF ( xi(i) < X(ind) )
		    inner2: IF  ( ind == 1 ) THEN
			    INTERPOLATEVEC = Y(ind)
		    ELSE
			    INTERPOLATEVEC =	&
				    ( Y(ind)*(xi(i)-X(ind-1)) + Y(ind-1)*(X(ind)-xi(i)) )	&
				    /( X(ind) - X(ind-1) )
		    END IF inner2
        END IF outer
    END DO
	
END FUNCTION INTERPOLATEVEC

FUNCTION    COVARIANCE(A, B, N, mask)

  ! Computes the covariance of two series A and B of length
  ! N. 

  IMPLICIT NONE

  INTEGER, INTENT(IN)                   :: N
  REAL(DP), DIMENSION(N), INTENT(IN)    :: A 
  REAL(DP), DIMENSION(N), INTENT(IN)    :: B
  LOGICAL, DIMENSION(N),INTENT(IN),OPTIONAL          :: mask  
  REAL(DP)                              :: COVARIANCE
  
  REAL(DP), DIMENSION(N)                :: Adev
  REAL(DP), DIMENSION(N)                :: Bdev
  REAL(DP), DIMENSION(N)                :: AdevBdev

  REAL(DP)                              :: xmn
  REAL(DP)                              :: ymn
  
  IF (PRESENT(mask)) THEN
      
      xmn = SUM(A,mask) / REAL(COUNT(mask),DP)
      ymn = SUM(B,mask) / REAL(COUNT(mask),DP)

      Adev = A - xmn
      Bdev = B - ymn

      AdevBdev = Adev * Bdev

      COVARIANCE = SUM(AdevBdev,mask)/REAL(COUNT(mask)-1,DP)

  ELSE

      xmn = SUM(A) / REAL(N,DP)
      ymn = SUM(B) / REAL(N,DP)

      Adev = A - xmn
      Bdev = B - ymn

      AdevBdev = Adev * Bdev

      COVARIANCE = SUM(AdevBdev)/REAL(N-1,DP)

  END IF 
  
END FUNCTION COVARIANCE 

FUNCTION    AVG(A,N,mask)

  ! Computes the mean of a vector A of length
  ! N. 

  IMPLICIT NONE

  REAL(DP), DIMENSION(N), INTENT(IN)                :: A 
  INTEGER, INTENT(IN)                               :: N
  LOGICAL,  DIMENSION(N), INTENT(IN),OPTIONAL       :: mask
  REAL(DP)                                          :: AVG
  
  ! local
  LOGICAL,  DIMENSION(N)                            :: valid
  
  valid     = NOT(ISNAN(A))
  
  IF (PRESENT(mask)) THEN
    AVG = SUM(A,mask .AND. valid) / REAL(COUNT(mask .AND. valid),DP)
  ELSE
    AVG = SUM(A, valid) / REAL(COUNT(valid),DP)
  END IF
  
END FUNCTION AVG

FUNCTION    AVGBLK(A,N,M,mask)

  ! Computes the mean of a vector A of length N
  ! using a block-level summary of A  
  ! across non-overlapping blocks of size M

  ! average across non overlapping blocks of length M
  ! for each block, take the (block) summary if mask is > 0
  ! here the summary statistic is akin to "ANY"

  IMPLICIT NONE

  REAL(DP), DIMENSION(N), INTENT(IN)                :: A 
  INTEGER, INTENT(IN)                               :: N
  INTEGER, INTENT(IN)                               :: M
  LOGICAL,  DIMENSION(N), INTENT(IN),OPTIONAL       :: mask
  REAL(DP)                                          :: AVGBLK
  
  ! Internal variables
  INTEGER                                           :: n_blk, t_blk  
  REAL(DP)                                          :: blk_eql    
  LOGICAL,  DIMENSION(N)                            :: valid
  
  ! initialization
  blk_eql   = onedp         ! use to implement "if (any (A == blk_eql))"
  
  n_blk     = 0
  AVGBLK    = zerodp
  
  valid     = NOT(ISNAN(A))  
  
  IF (PRESENT(mask)) THEN
    
    DO t_blk = 1,N-M-1,M+1
        IF ( mask(t_blk) .AND. valid(t_blk) ) THEN  
            n_blk   = n_blk + 1
            IF ( ANY(A(t_blk+1:t_blk+M) .EQ. blk_eql) ) THEN
                AVGBLK  = AVGBLK + onedp
            END IF
        END IF
    END DO
    AVGBLK  = AVGBLK / REAL(n_blk,DP)    
    
  ELSE
    DO t_blk = 1,N-M-1,M+1
        IF (valid(t_blk) ) THEN  
        n_blk   = n_blk + 1
            IF ( ANY(A(t_blk+1:t_blk+M) .EQ. blk_eql)  ) THEN
                AVGBLK  = AVGBLK + onedp
            END IF
        END IF
    END DO
    AVGBLK  = AVGBLK / REAL(n_blk,DP)  
  
  END IF
  
END FUNCTION AVGBLK

FUNCTION    AMOTRY(p,y,psum,mp,np,ndim,funk,ihi,fac)
    INTEGER   :: ihi,mp,ndim,np,NMAX
    REAL(DP)  :: AMOTRY,fac,p(mp,np),psum(np),y(mp),funk
    PARAMETER (NMAX=20)
    EXTERNAL funk
    INTEGER   :: j
    REAL(DP)  :: fac1,fac2,ytry,ptry(NMAX)
      
    fac1=(1.-fac)/ndim
    fac2=fac1-fac
    DO j=1,ndim
        ptry(j)=psum(j)*fac1-p(ihi,j)*fac2
    END DO
    ytry=funk(ptry)
    IF (ytry.lt.y(ihi)) THEN
        y(ihi)=ytry
        DO j=1,ndim
            psum(j)=psum(j)-p(ihi,j)+ptry(j)
            p(ihi,j)=ptry(j)
        END DO
    ENDIF
    AMOTRY=ytry
    RETURN
END FUNCTION AMOTRY
    
SUBROUTINE  AMOEBA(p,y,mp,np,ndim,ftol,funk,iter,ITMAX)
    INTEGER     :: iter,mp,ndim,np,NMAX,ITMAX
    REAL(DP)    :: ftol,p(mp,np),y(mp),funk
    PARAMETER (NMAX=20)
    EXTERNAL funk
    INTEGER     :: i,ihi,ilo,inhi,j,m,n
    REAL(DP)    :: rtol,sum,swap,ysave,ytry,psum(NMAX)
    
    iter=0
1   DO n=1,ndim
        sum=0.
        DO m=1,ndim+1
            sum=sum+p(m,n)
        END DO
        psum(n)=sum
    END DO
2   ilo=1
    IF (y(1).gt.y(2)) THEN
        ihi=1
        inhi=2
    ELSE
        ihi=2
        inhi=1
    ENDIF
    DO i=1,ndim+1
        IF(y(i).le.y(ilo)) ilo=i
        IF(y(i).gt.y(ihi)) THEN
            inhi=ihi
            ihi=i
        ELSE IF(y(i).gt.y(inhi)) THEN
            IF(i.ne.ihi) inhi=i
        ENDIF
    END DO
    rtol=2.*abs(y(ihi)-y(ilo))/(abs(y(ihi))+abs(y(ilo)))
    IF (rtol.lt.ftol) THEN
        swap=y(1)
        y(1)=y(ilo)
        y(ilo)=swap
        DO n=1,ndim
            swap=p(1,n)
            p(1,n)=p(ilo,n)
            p(ilo,n)=swap
        END DO
        RETURN
    ENDIF
    IF (iter.ge.ITMAX) STOP 'ITMAX exceeded in amoeba' ! PAUSE statement in original cause
    iter=iter+2
    ytry=AMOTRY(p,y,psum,mp,np,ndim,funk,ihi,-1.0_dp)
    IF (ytry.le.y(ilo)) THEN
        ytry=AMOTRY(p,y,psum,mp,np,ndim,funk,ihi,2.0_dp)
    ELSE IF (ytry.ge.y(inhi)) THEN
        ysave=y(ihi)
        ytry=AMOTRY(p,y,psum,mp,np,ndim,funk,ihi,0.5_dp)
        IF (ytry.ge.ysave) THEN
            DO i=1,ndim+1
                IF(i.ne.ilo) THEN
                    DO j=1,ndim
                        psum(j)=0.5_dp*(p(i,j)+p(ilo,j))
                        p(i,j)=psum(j)
                    END DO
                    y(i)=funk(psum)
                ENDIF
            END DO
            iter=iter+ndim
            GOTO 1
        ENDIF
    ELSE
        iter=iter-1
    ENDIF
    GOTO 2
END SUBROUTINE AMOEBA
   
END MODULE PROCESS