MODULE UTILITIES
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! PROGRAM: UTILITIES                               !!
    !!                                                  !!
    !! Last: Feb 02 2018                                !!        
    !!                                                  !!
    !! Profile:                                         !!
    !!  LOAD_PARAMS                                     !!
    !!      (prefix, aconfignum)                        !!
    !!  CREATE_PARAMFILES                               !!
    !!      (masterfile, prefix, numconfigs)            !!
    !!  GATHER_FILES                                    !!
    !!      (masterfile, prefix, filenm)                !!
    !!  SETUP_PARAMFILE                                 !!
    !!      (masterfile)                                !!
    !!                                                  !!
    !! Changes:                                         !!
    !! >> 2017/09/05                                    !!    
    !!    LT debt setup                                 !!
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    !!!!!!!!!!!!!!!!!!!!!!!!
    !!   USE STATEMENTS   !!
    !!!!!!!!!!!!!!!!!!!!!!!!    
    USE PARAMETERS
    USE GLOBAL
    USE PROCESS
    
    IMPLICIT NONE 

    !!!!!!!!!!!!!!!!!!!!!!!!
    !!    DECLARATIONS    !!
    !!!!!!!!!!!!!!!!!!!!!!!!    


CONTAINS

SUBROUTINE PRINT_PARAMS	
	! This routine reads a set of variables 
	! that are stored in the memory
    ! and prints it.

	IMPLICIT NONE

  
            						
110 	FORMAT(' ', 'line # = ', I5,  &
			' bita    = ', F10.7, ' bita_l  = ', F10.7, ' sigma   = ', F10.7, &
            ' sigma_l = ', F10.7, ' ies_l   = ', F10.7, ' lambda  = ', F10.7, &
            ' delta   = ', F10.3, ' d0      = ', F10.7, ' d1_h    = ', F10.7, &
            ' d1_h    = ', F10.7, ' theta   = ', F10.7, ' cov_piy = ', F10.7) 

        
	! print line
    WRITE (*,*) 'printing parameters stored'
	WRITE(*, 110)   PARAMS_VEC%CONFIG_VAL,  &
		            PARAMS_VEC%BITA_VAL,	&
                    PARAMS_VEC%BITA_L_VAL,  &
		            PARAMS_VEC%SIGMA_VAL,	&
		            PARAMS_VEC%SIGMA_L_VAL,	&
                    PARAMS_VEC%IES_L_VAL,	&
                    PARAMS_VEC%LAMBDA_VAL,	&
                    PARAMS_VEC%DELTA_VAL,	&
		            PARAMS_VEC%D0_VAL,      &
		            PARAMS_VEC%D1_H_VAL,    &
		            PARAMS_VEC%D1_L_VAL,    &
                    PARAMS_VEC%THETA_VAL,   &
		            PARAMS_VEC%COV_PIY_VAL

210 	FORMAT(' ', 'line # = ', I5,  &
			' bita    = ', F10.7, ' bita_l  = ', F10.7, ' sigma   = ', F10.7, &
            ' sigma_l = ', F10.7, ' ies_l   = ', F10.7, ' lambda  = ', F10.7, &
            ' delta   = ', F10.3, ' d0      = ', F10.7, ' d1_h    = ', F10.7, &
            ' d1_l    = ', F10.7, ' theta   = ', F10.7, ' cov_piy = ', F10.7, &
            ' r_s     = ', F10.7) 
        
    ! set parameters
	! print line
    WRITE (*,*) 'printing parameters used'
	WRITE(*, 210)   config,     &
		            bita,	    &
                    bita_l,     &
		            sigma,	    &
		            sigma_l,	&
                    ies_l,	    &
                    lambda,	    &
                    delta,	    &
		            d0,         &
		            d1_h,       &
		            d1_l,       &
                    theta,      &
		            cov_piy,	&
                    rs
  
END SUBROUTINE PRINT_PARAMS	

SUBROUTINE LOAD_PARAMS (prefix, aconfignum)
	! This routine reads a set of variables 
	! from the single-line file named
    ! prefix_aconfignum.ext
    ! and changes the global parameters

	IMPLICIT NONE

	CHARACTER (LEN = *), INTENT(IN) 	:: prefix		! name of the file to read
	INTEGER, INTENT(IN)					:: aconfignum	! number of configurations contained in the file
  
	! PROGRAM
	INTEGER 							:: istat
    CHARACTER (LEN = 80)                :: ierror
	CHARACTER (LEN=300) 				:: line
    CHARACTER (LEN=7) 				    :: lineno_str
    CHARACTER (LEN=80)                  :: newfile
	
    REAL(DP)                            :: bita_in
    REAL(DP)                            :: bita_l_in
    REAL(DP)                            :: sigma_in
    REAL(DP)                            :: sigma_l_in     
	REAL(DP)                            :: ies_l_in
    REAL(DP)                            :: lambda_in
    REAL(DP)                            :: delta_in
	REAL(DP)                            :: d0_in
	REAL(DP)                            :: d1_h_in
	REAL(DP)                            :: d1_l_in
    REAL(DP)                            :: theta_in
    REAL(DP)                            :: cov_piy_in     
  
    ! generate filename
    WRITE (UNIT=lineno_str, FMT='(I<sfx>.<sfx>)') aconfignum
    newfile = TRIM(folder_cfg) // TRIM(prefix) // '_' // TRIM(lineno_str) // '.' // ext   
            
	! Read and set parameters
    OPEN (UNIT=11, FILE=newfile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
					
	openif: IF ( istat == 0 ) THEN
        	
		! read line
        WRITE (*,*) 'reading file', newfile

100     FORMAT(1X, 12F13.7)            
		READ (UNIT=11,FMT=100)  bita_in,    &
                                bita_l_in,  &
                                sigma_in,   &
                                sigma_l_in, &
	                            ies_l_in,   &
                                lambda_in,  &
                                delta_in,   &
	                            d0_in,      &
	                            d1_h_in,    &
	                            d1_l_in,    &
                                theta_in,   &
                                cov_piy_in
			
        WRITE (*,*) 'storing configuration', aconfignum
            
		! store line
		PARAMS_VEC%CONFIG_VAL 	= aconfignum
		PARAMS_VEC%BITA_VAL		= bita_in
        PARAMS_VEC%BITA_L_VAL   = bita_l_in
		PARAMS_VEC%SIGMA_VAL	= sigma_in
		PARAMS_VEC%SIGMA_L_VAL	= sigma_l_in
        PARAMS_VEC%IES_L_VAL	= ies_l_in
        PARAMS_VEC%LAMBDA_VAL	= lambda_in
        PARAMS_VEC%DELTA_VAL	= delta_in
		PARAMS_VEC%D0_VAL       = d0_in
		PARAMS_VEC%D1_H_VAL     = d1_h_in
		PARAMS_VEC%D1_L_VAL     = d1_l_in
        PARAMS_VEC%THETA_VAL	= theta_in
		PARAMS_VEC%COV_PIY_VAL	= cov_piy_in
		
			
        WRITE (*,*) 'printing parameters loaded'
            
		! print line
		WRITE(*, 110)   PARAMS_VEC%CONFIG_VAL,  &
		                PARAMS_VEC%BITA_VAL,	&
                        PARAMS_VEC%BITA_L_VAL,  &
		                PARAMS_VEC%SIGMA_VAL,	&
		                PARAMS_VEC%SIGMA_L_VAL,	&
                        PARAMS_VEC%IES_L_VAL,	&
                        PARAMS_VEC%LAMBDA_VAL,	&
                        PARAMS_VEC%DELTA_VAL,	&
		                PARAMS_VEC%D0_VAL,      &
		                PARAMS_VEC%D1_H_VAL,    &
		                PARAMS_VEC%D1_L_VAL,    &
                        PARAMS_VEC%THETA_VAL,	&
		                PARAMS_VEC%COV_PIY_VAL
							
110 	FORMAT(' ', 'line # = ', I5,  &
				' bita    = ', F10.7, ' bita_l  = ', F10.7, ' sigma   = ', F10.7, &
                ' sigma_l = ', F10.7, ' ies_l   = ', F10.7, ' lambda  = ', F10.7, &
                ' delta   = ', F10.3, ' d0      = ', F10.7, ' d1_h    = ', F10.7, &
                ' d1_l    = ', F10.7, ' theta   = ', F10.7, ' cov_piy = ', F10.7) 
		
        ! set parameters
        config      = PARAMS_VEC%CONFIG_VAL
        bita        = PARAMS_VEC%BITA_VAL
        bita_l      = PARAMS_VEC%BITA_L_VAL
	    sigma       = PARAMS_VEC%SIGMA_VAL
	    sigma_l     = PARAMS_VEC%SIGMA_L_VAL
        ies_l       = PARAMS_VEC%IES_L_VAL
	    lambda      = PARAMS_VEC%LAMBDA_VAL
        delta       = PARAMS_VEC%DELTA_VAL
        d0          = PARAMS_VEC%D0_VAL
        d1_h        = PARAMS_VEC%D1_H_VAL
        d1_l        = PARAMS_VEC%D1_L_VAL
        theta       = PARAMS_VEC%THETA_VAL
	    cov_piy     = PARAMS_VEC%COV_PIY_VAL
        
        ! set global dependent variables
        rs          = 1.0_dp/bita_l
        
        ! coupon rate --- adjusted if calsprd specified
        rc          = zerodp
        
        IF ( zerocoupon == 0 ) THEN
            rc          = (1.0_dp - bita_l)*(1.0_dp - delta)
        ELSE  IF ( zerocoupon == 1 ) THEN
            rc          = zerodp    ! see new definition of "real discounted" price
        END IF
        
        IF (calsprd1v2 == 1) THEN
            rc          = (1.0_dp - bita_l)*(1.0_dp - delta)
            zerocoupon  = 0
        ELSE IF (calsprd1v2 == 2) THEN
            rc          = zerodp
            zerocoupon  = 1
        END IF
        	    
        ! set prefix
        cfgpfx      = prefix
               
	ELSE openif
	
		WRITE (*, 200) newfile, ierror
200		FORMAT (' ', 'Error opening file: ', A, ' IOMSG = ', A )
	
    END IF openif
        
    CLOSE (UNIT=11)  
  
END SUBROUTINE LOAD_PARAMS	

SUBROUTINE CREATE_PARAMFILES (masterfile, prefix, numconfigs)
	! This routine generates parameter files from a master file
    ! 
    ! It reads a set of variables 
	! from the file 'masterfile'
	! and create a file for each linea file named prefix_$$$.ext
    ! where $$$ is the row number of the configuration
    !
	! The file starts with comments line that are ignored

	IMPLICIT NONE

	CHARACTER (LEN = *), INTENT(IN) 	:: masterfile   ! name of the file to read
    CHARACTER (LEN = *), INTENT(IN)     :: prefix       ! prefix to be used for file names
    INTEGER, INTENT(OUT), OPTIONAL      :: numconfigs   ! number of configurations 
  
	! PROGRAM
	CHARACTER (LEN = 80)                :: ierror
	INTEGER 							:: istat
    INTEGER 							:: ios
	INTEGER 							:: lineno
    INTEGER 							:: linetot
	CHARACTER (LEN=7) 				    :: lineno_str
	CHARACTER (LEN=300) 				:: line
    CHARACTER (LEN=80)                  :: newfile
    CHARACTER (LEN=80)                  :: mstfile
	
	! Read and set parameters
    mstfile = TRIM(folder_cfg) // masterfile // '.' // ext     
    OPEN (UNIT=11, FILE=mstfile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istat, IOMSG=ierror)
					
	openif: IF ( istat == 0 ) THEN
	     WRITE (*,*) 'successfully opened master parameter file'
		
         skipcomments: DO
			! skip comment lines
			READ (UNIT=11, FMT='(A)') line
			IF (line (1:1) /= '#') EXIT skipcomments
        END DO skipcomments
		
        WRITE (*,*) 'comments all skipped'
        
		! go to last line: first data entry
		BACKSPACE(UNIT=11)
        
        linetot = 0	
		storeloop: DO lineno = 1, maxparams
		
			! read line
            READ (UNIT=11, FMT='(A)', IOSTAT = ios) line
            
            IF ( ios .GT. 0) THEN
                WRITE (*,*) ' Error reading record.'
                EXIT
            ELSE IF (ios .LT. 0) THEN
                WRITE (*,*) ' EOF or EOR found.'
                EXIT
            END IF
            
            linetot = linetot + 1
            
            ! generate filename corresponding to line number
            WRITE (UNIT=lineno_str, FMT='(I<sfx>.<sfx>)') lineno
            newfile = TRIM(folder_cfg) // TRIM(prefix) // '_' // TRIM(lineno_str) // '.' // ext           
            
	        ! store line into new file
            OPEN (UNIT=22, FILE=newfile, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ios)          
            WRITE (UNIT=22, FMT='(A)') line
			CLOSE (UNIT=22, STATUS='KEEP')
            
            ! warning for large files
            IF ( lineno .EQ. maxparams ) THEN
                WRITE (*,*) ' Error: Max number of records reached.'
                EXIT
            END IF
                        
        END DO storeloop
        
        IF ( PRESENT (numconfigs)) THEN
            numconfigs = linetot
        END IF
		
    ELSE openif
        
		WRITE (*, 200) mstfile, ierror
200		FORMAT (' ', 'Error opening file: ', A, ' IOMSG = ', A )        
	
    END IF openif
        
    WRITE (*,*) 'done. generated ', numconfigs, ' files.'
    
    CLOSE (UNIT=11)  
  
END SUBROUTINE CREATE_PARAMFILES

SUBROUTINE GATHER_FILES (masterfile, prefix, filenm, addparams)
	! This routine creates a new file "filenm"
    ! in the output folder <<folder_out/>>
    ! 
    ! by reading a set of variables from the file "filenm"
    ! in each subdirectory folder_out/prefix_$$$$/
    ! where $$$ is the configuration inside masterfile
    !
    ! if addparams is .TRUE. the corresponding line in "masterfile" is appended

	IMPLICIT NONE

	CHARACTER (LEN = *), INTENT(IN) 	:: masterfile   ! name of the file to read
    CHARACTER (LEN = *), INTENT(IN)     :: prefix       ! prefix to be used for file names
    CHARACTER (LEN = *), INTENT(IN)     :: filenm       ! the individual file to be gathered
    LOGICAL, INTENT(IN), OPTIONAL       :: addparams    ! specifies whether or not to append list of parameters
  
	! PROGRAM
	CHARACTER (LEN = 80)                :: ierrorin
    CHARACTER (LEN = 80)                :: ierrorout
	INTEGER 							:: istatin
    INTEGER 							:: istatout
    INTEGER 							:: ios
	INTEGER 							:: lineno
    INTEGER 							:: linetot
	CHARACTER (LEN=7) 				    :: lineno_str
	CHARACTER (LEN=450) 				:: newline
    CHARACTER (LEN=450) 				:: linein
    CHARACTER (LEN=450) 				:: lineout
    CHARACTER (LEN=80)                  :: newfile
    CHARACTER (LEN=80)                  :: outfile
    CHARACTER (LEN=80)                  :: mstfile
    LOGICAL                             :: appendprms

    IF (PRESENT(addparams)) THEN
        appendprms  = addparams
    ELSE
        appendprms  = .TRUE.
    END IF

	! open parameters file (masterfile)
    mstfile = TRIM(folder_cfg) // masterfile // '.' // ext     
    OPEN (UNIT=11, FILE=mstfile, POSITION='REWIND', STATUS='OLD', & 
						ACTION='READ', IOSTAT=istatin, IOMSG=ierrorin)
	
    ! open new output file
    outfile = TRIM(folder_out) // TRIM(filenm)
    OPEN (UNIT=12, FILE=outfile, STATUS='REPLACE', & 
						ACTION='WRITE', IOSTAT=istatout, IOMSG=ierrorout)    
					
	openif: IF (( istatin == 0 ) .AND. ( istatout == 0 )) THEN
	     WRITE (*,*) 'successfully opened files'
		
         skipcomments: DO
			! skip comment lines
			READ (UNIT=11, FMT='(A)') linein
			IF (linein (1:1) /= '#') EXIT skipcomments
        END DO skipcomments
		
        WRITE (*,*) 'comments all skipped'
        
		! go to last line: first data entry
		BACKSPACE(UNIT=11)
        
        linetot = 0	
		storeloop: DO lineno = 1, maxparams
		
			! read line
            linein  = ''
            READ (UNIT=11, FMT='(A)', IOSTAT = ios) linein
            
            IF ( ios .GT. 0) THEN
                WRITE (*,*) ' Error reading record.'
                EXIT
            ELSE IF (ios .LT. 0) THEN
                WRITE (*,*) ' EOF or EOR found.'
                EXIT
            END IF
            
            linetot = linetot + 1
            
            ! generate path to file name
            WRITE (UNIT=lineno_str, FMT='(I<sfx>.<sfx>)') lineno
            newfile  = TRIM(folder_out) // TRIM(prefix) // '_' // &
                            TRIM(lineno_str) // '/' // TRIM(filenm)    
                        
	        ! read line into new file
            lineout = ''
            OPEN (UNIT=22, FILE=newfile, POSITION='REWIND', STATUS='OLD', &
                            ACTION='READ', IOSTAT=ios)          
            
            IF ( ios == 0) THEN
                
                skipcomments2: DO
			    ! skip comment lines
			    READ (UNIT=22, FMT='(A)') lineout
			    IF (lineout (1:1) /= '#') EXIT skipcomments2
                END DO skipcomments2
                
                ! return to first non comment line
                BACKSPACE(UNIT=22)
                
                ! read non-comment first line
                READ (UNIT=22, FMT='(A)') lineout
            
            END IF
            
            CLOSE (UNIT=22)
            
            ! write into new file
            IF (appendprms) THEN
                newline = TRIM(lineno_str) // ' ' // TRIM(linein) // ' ' // TRIM(lineout)
                newline = TRIM(newline)
            ELSE
                newline = TRIM(lineno_str) // ' ' // TRIM(lineout)
                newline = TRIM(newline)
            ENDIF
            
            WRITE (UNIT=12, FMT='(A)') newline
            
            ! warning for large files
            IF ( lineno .EQ. maxparams ) THEN
                WRITE (*,*) ' Error: Max number of records reached.'
                EXIT
            END IF
                        
        END DO storeloop
        
	ELSE openif
	
        WRITE (*, 200) mstfile, TRIM(ierrorin)
		WRITE (*, 200) outfile, TRIM(ierrorout)
200		FORMAT (' ', 'Possible error: ', A, ' IOMSG = ', A )  
	
    END IF openif
        
    WRITE (*,*) 'done. gathered ', linetot, ' files.'
    
    CLOSE (UNIT=22, STATUS='KEEP')
    CLOSE (UNIT=11)  
  
END SUBROUTINE GATHER_FILES

SUBROUTINE SETUP_PARAMFILE (masterfile)

	! This routine generates a main parameter file to be used for calibration
    ! These files are the configuration folder folder_cfg
    ! 
    ! It generates a set of parameters
	! and creates a file for all parameters, named masterfile.$ext$
    !

	IMPLICIT NONE

    CHARACTER (LEN = *), INTENT(IN) 	:: masterfile   ! name of the file to read
  
	! PROGRAM
    INTEGER                             :: i_beta       = 1
    INTEGER                             :: i_beta_l     = 1
    INTEGER                             :: i_sigma      = 1
    INTEGER                             :: i_sigma_l    = 1
    INTEGER                             :: i_ies_l      = 1
    INTEGER                             :: i_lambda     = 1
    INTEGER                             :: i_delta      = 1 
    INTEGER                             :: i_d0         = 1
    INTEGER                             :: i_d1_h       = 1 
    INTEGER                             :: i_d1_l       = 1 
    INTEGER                             :: i_theta      = 1
    INTEGER                             :: i_cov_piy    = 1
    
    INTEGER 							:: ios
    INTEGER                             :: linetot
	CHARACTER (LEN=12) 				    :: lineno_str
    CHARACTER (LEN=80)                  :: filenm
    
    ! CREATE GRIDS FOR PARAMETERS    
    ! BUILD PARAMETER GRID FOR BETA
    CALL creategrid(BETA_GRID,N_BETA,BETAMIN,BETAMIN,BETAMAX,ONEPWR)

    ! BUILD PARAMETER GRID FOR BETA_L
    CALL creategrid(BETA_L_GRID,N_BETA_L,BETA_LMIN,BETA_LMIN,BETA_LMAX,ONEPWR)    
    
    ! BUILD PARAMETER GRID FOR SIGMA
    CALL creategrid(SIGMA_GRID,N_SIGMA,SIGMA_MIN,SIGMA_MIN,SIGMA_MAX,ONEPWR)

    ! BUILD PARAMETER GRID FOR SIGMA_L
    CALL creategrid(SIGMA_L_GRID,N_SIGMA_L,SIGMA_LMIN,SIGMA_LMIN,SIGMA_LMAX,ONEPWR)
    
    ! BUILD PARAMETER GRID FOR IES_L
    CALL creategrid(IES_L_GRID,N_IES_L,IES_LMIN,IES_LMIN,IES_LMAX,ONEPWR)

    ! BUILD PARAMETER GRID FOR LAMBDA
    CALL creategrid(LAMBDA_GRID,N_LAMBDA,LAMBDAMIN,LAMBDAMIN,LAMBDAMAX,ONEPWR)

    ! BUILD PARAMETER GRID FOR DELTA
    CALL creategrid(DELTA_GRID,N_DELTA,DELTAMIN,DELTAMIN,DELTAMAX,ONEPWR)
    
    ! BUILD PARAMETER GRID FOR D0
    CALL creategrid(D0_GRID,N_D0,D0MIN,D0MIN,D0MAX,ONEPWR)

    ! BUILD PARAMETER GRID FOR D1_H
    CALL creategrid(D1_H_GRID,N_D1_H,D1_HMIN,D1_HMIN,D1_HMAX,ONEPWR)

    ! BUILD PARAMETER GRID FOR D1_L
    CALL creategrid(D1_L_GRID,N_D1_L,D1_LMIN,D1_LMIN,D1_LMAX,ONEPWR)
    
        
    ! BUILD PARAMETER GRID FOR THETA
    CALL creategrid(THETA_GRID,N_THETA,THETAMIN,THETAMIN,THETAMAX,ONEPWR)
    
    ! BUILD PARAMETER GRID FOR COV_PIY
    CALL creategrid(COV_PIY_GRID,N_COV_PIY,COV_PIYMIN,COV_PIYMIN,COV_PIYMAX,ONEPWR)
    
    
    ! OPEN
    filenm = TRIM(folder_cfg) // masterfile // '.' // ext        
    OPEN (UNIT=22, FILE=filenm, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ios)          
100 FORMAT(1X, 12F13.7)    
    
    WRITE (*,*) 'begin - writing in masterfile ...', TRIM(filenm)
    
    linetot = 0
    DO i_beta    = 1, N_BETA
    DO i_beta_l  = 1, N_BETA_L
    DO i_sigma   = 1, N_SIGMA
    DO i_sigma_l = 1, N_SIGMA_L
    DO i_ies_l   = 1, N_IES_L
    DO i_lambda  = 1, N_LAMBDA
    DO i_delta   = 1, N_DELTA
    DO i_d0      = 1, N_D0
    DO i_d1_h    = 1, N_D1_H
    DO i_d1_l    = 1, N_D1_L
    DO i_theta   = 1, N_THETA
    DO i_cov_piy = 1, N_COV_PIY
        
        linetot     = linetot + 1
        WRITE (*,*) 'configuration ', linetot
        
	    ! store values into new file
		WRITE (UNIT=22,FMT=100) BETA_GRID(i_beta),      &
                                BETA_L_GRID(i_beta_l),  &
                                SIGMA_GRID(i_sigma),    &
                                SIGMA_L_GRID(i_sigma_l),&
	                            IES_L_GRID(i_ies_l),    &
                                LAMBDA_GRID(i_lambda),  &
                                DELTA_GRID(i_delta),    &
	                            D0_GRID(i_d0),          &
	                            D1_H_GRID(i_d1_h),      &
	                            D1_L_GRID(i_d1_l),      &
                                THETA_GRID(i_theta),    &  
                                COV_PIY_GRID(i_cov_piy)
          
    END DO
    END DO
    END DO
    END DO
    END DO
    END DO
    END DO
    END DO
    END DO                        
    END DO
    END DO
    END DO
  
    WRITE (*,*) 'done - writing in masterfile ...  ', TRIM(filenm)
	! CLOSE
    CLOSE (UNIT=22, STATUS='KEEP')
    
    WRITE (*,*) 'closed masterfile ...              ', TRIM(filenm)
        
END SUBROUTINE SETUP_PARAMFILE

END MODULE UTILITIES
