MODULE AUXILIARY
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! PROGRAM: AUXILIARY                               !!
    !!                                                  !!
    !! Date: Sep 13, 2014                               !!
    !! Last: Feb 02 2018                                !!        
    !!                                                  !!
    !! Profile:                                         !!
    !!  AUTARKY_VALUE                                   !!
    !!  CREATE_GUESS_P_V                                !!
    !!  SOLVEMODEL                                      !!
    !!  POST_PROCESS                                    !!
    !!  SAVERESULTS                                     !!
    !!  SIMULATE                                        !!
    !!  RUNSIMUL                                        !!    
    !!                                                  !!
    !!                                                  !!
    !! Changes:                                         !!
    !! >> 02/09/2015                                    !!
    !!  solvemodel                                      !!
    !!  - record convergence & print more details       !!
    !!  saveresults                                     !!
    !!  - save last iteration prices and savings        !!
    !! >> 03/09/2015                                    !!
    !!  - bonds adjustments costs                       !!
    !! >> 09/15/2015                                    !!
    !!  - spell-level simulation dataset                !!
    !!  - new simulation moments                        !!  
    !! >> 12/15/2015                                    !!
    !!  - simulation results (see *_simple routines)    !!
    !! >> 03/07/2016                                    !!
    !!  - added business cycle stats                    !!
    !!  - added price decomposition by state (B.s)      !!    
    !!  - ergodic distribution of (B.s)                 !!
    !!                                                  !!
    !! >> 09/23/2016                                    !!
    !!  - added spells and frame back                   !!    
    !!                                                  !!
    !! >> 11/15/2016                                    !!
    !!  - added ssfeval (used with amoeba)              !!
    !!  - removed spells -- TBD: to add back            !!
    !!                                                  !!
    !! >> 12/08/2016                                    !!
    !!  - corrected ssfeval for nan                     !! 
    !!                                                  !!
    !! >> 09/14/2017                                    !!
    !!  - modified algorithm -- simpler lender SDF      !! 
    !!  - removed spells simulation code                !!
    !!                                                  !!
    !! >> 2018/02/23                                    !!
    !!    regime switching version                      !!    
    !!                                                  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!   USE STATEMENTS - CUSTOM  !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!    
    USE PARAMETERS
    USE PROCESS
    USE GLOBAL
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!   USE STATEMENTS - GENERIC   !!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!     
    USE LAPACK95
    USE IFPORT
    
    IMPLICIT NONE 

    !!!!!!!!!!!!!!!!!!!!!!!!
    !!    DECLARATIONS    !!
    !!!!!!!!!!!!!!!!!!!!!!!!    
    INTEGER                         ::  i_bp_star, i, ite, iter, iter2, j, gap3, ioerror,   &
                                        ibpinfl, piter, i_blambda, iterchi, tmp1,           &
                                        j_bp_min, j_bp_max, arg2(3)

    REAL (DP)                       ::  evo, ev, evv, evp, evp_nodef, evp_def,              &
                                        gappavg, gapsavg, gappsav, gappmax, gapsmax,        &
                                        gapl, gapll, gapld, gappdefavg, gapdchg,            &
                                        b00, bpinfl, smin,                                  &
                                        cons_l, cons_lp, rsdf_loc, y_def,                   &
                                        utils, dutils, dutils_l, dutils_lp,                 &
                                        defp, defpr, qdet, qdef,                            &
                                        avgadjcost, avg_error, adjweight, avg_pfit_err
    
    
    
    CONTAINS
 

SUBROUTINE AUTARKY_VALUE

    !! Calculate utility of permament autarky 
    !! only used as a guess 
    !! so we compute it only for each regime
    !! so as not to invert the larger transition matrix
    
    
    DO i_rgm=1,rgm
        
        conaut(i_rgm,:)     = ygrid(:) - penalty(ygrid(:),i_rgm)
        utilaut(i_rgm,:)    = utility(conaut(i_rgm,:))
        
        ! get autarky value = (I - beta*P)^(-1)*U_aut
        ident = zerodp
        FORALL (i_ss=1:ss) ident(i_ss,i_ss) = onedp  ! Construct identity matrix
        
        temp = (ident-bita*ptran)
        tempinv = temp
        
        CALL GETRF(tempinv,iptemp)
        CALL GETRI(tempinv,iptemp)
        
        vaut(i_rgm,:)    = MATMUL(tempinv,utilaut(i_rgm,:))
    
    END DO
      
    
    !DO i_rgm=1,rgm      ! default cost regime
    !    DO i=1,ss
    !        conaut(i) = ygrid(i) - penalty(ygrid(i),i_rgm)
    !        PRINT *, ygrid(i), conaut(i)
    !    END DO
    !END DO
    
    !utilaut(:)  =  utility(conaut(:))           ! autarky period utility in low cost regime
    
    !! get autarky value = (I - beta*P)^(-1)*U_aut
    !ident = zerodp
    !FORALL (i_ss=1:ss) ident(i_ss,i_ss) = onedp  ! Construct identity matrix
    !temp = (ident-bita*ptran)
    !tempinv = temp
    !CALL GETRF(tempinv,iptemp)
    !CALL GETRI(tempinv,iptemp)
    !vaut    = MATMUL(tempinv,utilaut)
    
END SUBROUTINE AUTARKY_VALUE

SUBROUTINE CREATE_GUESS_P_V(showdetails)

    IMPLICIT NONE 
    
    INTEGER, INTENT(IN), OPTIONAL   :: showdetails
    
    ! local variable
    LOGICAL                         :: showdetails_prm
    
    IF (PRESENT(showdetails)) THEN
        showdetails_prm  = (showdetails .EQ. 1)
    ELSE
        showdetails_prm  = .FALSE.
    END IF
    
    conv_guess      = 0

    !! non-stochastic price with no default or inflation
    qdet = bita_l*(rc + delta)/(onedp - bita_l*(onedp - delta)) 
    !! non-stochastic default price with no inflation
    qdef = theta*lambda*qdet
    
    IF (showdetails_prm) THEN
        PRINT *, 'qdet=',qdet
        PRINT *, 'qdef=',qdef
    END IF
    
    
    !! Lender recursive preferences -- computation
    
    !! Lender recursive preferences -- computation -- initialization
    IF (EZ_mode == 1) THEN
        IF (ies_l == onedp) THEN
            !v_lrec = EXP( (onedp-bita_l)*LOG(ygrid) )
            v_lrec = (ygrid)**(onedp-bita_l)
        ELSE
            v_lrec = (onedp-bita_l)**(onedp/(onedp-ies_l))
        END IF
    ELSE
        v_lrec = onedp
    END IF
    
    gapl = onedp
    
    
    !! Lender recursive preferences -- computation -- iteration
    LENDER: DO iter = 1,vlitmax        
        ! Keep previous value function
        vo_lrec = v_lrec
                         
        ! update lender value function
        DO i_ss=1,ss   
            evp = zerodp                   
            
            !expected value  E[ W^(1-\gamma) ]
            DO i_ssp=1,ss
                evp = evp + ptran(i_ss,i_ssp)*vo_lrec(i_ssp)**(onedp-sigma_l)
            END DO
            
            ! updated value W
            ! W^(1\-rho) = {(1-bita)u^(1-\rho) + bita*E[W^(1-\gamma)]^(1\rho)/(1-\gamma) }
            IF (ies_l == onedp) THEN
                v_lrec(i_ss) = EXP((onedp-bita_l)*LOG(ygrid(i_ss)) + bita_l/(onedp-sigma_l)*LOG(evp))
            ELSE
                v_lrec(i_ss) = (                                                &
                                (onedp-bita_l)*(ygrid(i_ss))**(onedp-ies_l)     &
                                + bita_l*evp**((onedp-ies_l)/(onedp-sigma_l))   &
                                )**(onedp/(onedp-ies_l))
            END IF
            
        END DO
    
        ! lender value function convergence criterion
        gapl = SUM(ABS (v_lrec - vo_lrec )) / DBLE(ss) 
        IF ( (gapl<critv_l) .OR. (iter==vlitmax) ) THEN
50          FORMAT ('iteration: ',I5, ' avg errors (v_l): ',F9.4)         
            WRITE(*,50) iter, gapl
            EXIT LENDER
        END IF
        
    END DO LENDER
    
    !! Risk-adjusted expected value of lender [E W^(1-\gamma)]^1/(1-\gamma)
    Ev_lrec     = zerodp
    DO i_ss=1,ss   
        evp     = zerodp  
        DO i_ssp=1,ss
                evp = evp +                 &
                    ptran(i_ss,i_ssp)       &
                    *(v_lrec(i_ssp)**(onedp-sigma_l))                
        END DO
        Ev_lrec(i_ss) = evp**(onedp/(onedp-sigma_l))
    END DO
    
    
    !! SDF lender
    !!
    sdf_nodef   = onedp
    sdf_def     = onedp
    sdf         = onedp
    DO i_ss=1,ss                                    ! loop over income states s = (y,pi)
        DO i_ssp=1,ss                               ! loop over future states s'= (y',pi')
            
            ! stochastic discount factor 
            IF (EZ_mode == 1) THEN
                sdf(i_ss,i_ssp) =   bita_l*                                     &
                                    ((ygrid(i_ssp)/ygrid(i_ss))**(-ies_l))*   &
                                    (v_lrec(i_ssp)/Ev_lrec(i_ss))**(ies_l-sigma_l) 
            ELSE
                sdf(i_ss,i_ssp) =   bita_l*                                     &
                                    ((ygrid(i_ssp)/ygrid(i_ss))**(-sigma_l))
            END IF
            
        END DO 
    END DO
    sdf_nodef   = sdf
    sdf_def     = sdf_nodef
    
    !! Compute conditional expected sdf
    ex_sdf_nodef    = onedp
    ex_sdf          = onedp
    ex_sdf_def      = onedp
    
    DO i_ss=1,ss
        ex_sdf_nodef(i_ss) = DOT_PRODUCT(ptran(i_ss,:), sdf(i_ss,:)) 
    END DO 
    ex_sdf      = ex_sdf_nodef
    ex_sdf_def  = ex_sdf_nodef
    
    IF (readguess==0) THEN    
    !!
    !! bond price schedule under no default 
    !!
        p_guess     = qdet
        p1_guess    = qdet
    
        ! iterate on no default pricing kernel
        PGUESS_LOOP: DO iter=1,pit_guess
            !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(evp)
            !$OMP DO COLLAPSE(1)
            DO i_ss=1,ss                                ! loop over income states s = (y,pi)
                evp	    = zerodp                        ! initialize cost of borrowing q(s) -- no default so no debt level
                DO i_ssp=1,ss                           ! loop over future states s'
                    ! incrementally update borrowing cost
                    evp = evp + ptran(i_ss,i_ssp)                   &
                        *sdf_nodef(i_ss,i_ssp)                      &
                        *(rc+delta+(onedp-delta)*p_guess(i_ssp))    &
                        /(onedp + pigrid(i_ssp)) 
                END DO 
                p1_guess(i_ss) = evp  ! implied borrowing cost
            END DO
            !$OMP END DO
            !$OMP END PARALLEL
        

    
            ! convergence check
            gappavg = (SUM (ABS (p1_guess-p_guess))) / DBLE(ss)
            gappmax = (MAXVAL (ABS (p1_guess-p_guess))) 
            IF (showdetails) THEN
                PRINT *, " no default guess -- max abs errors (price)  ", gappmax 
            END IF
        
            CONV_PK: IF (gappavg<critpguess)  THEN
                IF (showdetails) THEN
                PRINT *, "NO DEFAULT GUESS -- PRICE CONVERGED!"
                END IF
                conv_guess  = 1
                EXIT PGUESS_LOOP
            ELSE IF ( ISNAN(gappavg)) THEN
                IF (showdetails) THEN
                PRINT *, "NO DEFAULT GUESS --  NaN error "
                END IF
                conv_guess  = 0
                EXIT PGUESS_LOOP
            ELSE
                ! update guess
                p_guess = pweightnew*p1_guess + (onedp-pweightnew)*p_guess
            END IF CONV_PK
        END DO PGUESS_LOOP
    
        !!   
        !! update the default price
        !!
        pdef_guess = theta*lambda*p_guess
    
        !!
        !! Initialize the no-default and default parts of the bond price
        !! p_nodef = p
        !! p_def = zerodp
        !! 
    
        FORALL (i_rgm=1:rgm,i_bp=1:ngpbprime)   p(i_rgm,:,i_bp)     = p_guess(:)
        FORALL (i_rgm=1:rgm,i_b =1:ngpb)        pdef(i_rgm,i_b,:)   = pdef_guess(:)

    END IF
    
    IF (readguess==0) THEN
    !!
    !! set the initial equilibrium price guessing that bstar=b
    !!

        DO i_rgm=1,rgm
            DO i_b=1,ngpb
                i_bp = MAXVAL(MINLOC(ABS(bprime-b(i_b))))
                psav(i_rgm,i_b,:)   = RESHAPE(p(i_rgm,:,i_bp),SHAPE(psav(i_rgm,i_b,:)))
            END DO
        END DO

    END IF
    
    IF (readguess==0) THEN  
    !!
    !! set value function guess to autarky 
    !! v repayment and vdef
    !!
    
        FORALL (i_rgm=1:rgm,i_ss=1:ss) v(i_rgm,:,i_ss)    = vaut(i_rgm,i_ss)
        vdef    = v
    
    END IF
    
    
    !!
    !! risk free bond price schedule 
    !! under no default  and no inflation
    !!
    p_riskfree     = qdet
    p1_riskfree    = qdet
    
    ! iterate on risk free pricing kernel
    P_RISKFREE_LOOP: DO iter=1,pit_guess
        !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(evp)
        !$OMP DO COLLAPSE(1)
        DO i_ss=1,ss                                ! loop over income states s = (y,pi)
            evp	    = zerodp                        ! initialize cost of borrowing q(s) -- no default  & no inflation
            DO i_ssp=1,ss                           ! loop over future states s'
                ! incrementally update borrowing cost
                evp = evp + ptran(i_ss,i_ssp)           &
                    *sdf_nodef(i_ss,i_ssp)              &
                    *(rc+delta+(onedp-delta)*p_riskfree(i_ssp))
            END DO 
            p1_riskfree(i_ss) = evp                            ! implied borrowing cost
        END DO
        !$OMP END DO
        !$OMP END PARALLEL
            
        ! convergence check
        gappavg = (SUM (ABS (p1_riskfree-p_riskfree))) / DBLE(ss)
        gappmax = (MAXVAL (ABS (p1_riskfree-p_riskfree))) 
        
        CONV_PRF: IF (gappavg<critpguess)  THEN
            PRINT *, " "
            PRINT *, "RISK FREE PRICE CONVERGED"
            PRINT *, " "

            EXIT P_RISKFREE_LOOP
        ELSE IF ( ISNAN(gappavg)) THEN
            EXIT P_RISKFREE_LOOP
        ELSE
            ! update guess
            p_riskfree = pweightnew*p1_riskfree + (onedp-pweightnew)*p_riskfree
        END IF CONV_PRF
        
        
    END DO P_RISKFREE_LOOP
    
    
    ! Case when convergence of guess does not occur
    IF ((conv_guess == 0) .AND. (readguess==0)) THEN
        PRINT *, "NO DEFAULT GUESS -- PRICING KERNEL GUESS DID NOT CONVERGE :( ... "        
        RETURN
    END IF
    
    
END SUBROUTINE CREATE_GUESS_P_V

SUBROUTINE COMPUTEMODEL_MAIN
    ! launch key computation subroutines
    ! (a) solve model
    ! (b) simulate
    ! (c) save results
    
    IMPLICIT NONE
    
    REAL (DP)           ::  prog_time_start, prog_time_stop
    
    prog_time_start    = DCLOCK() 				! timer

    CALL    SOLVEMODEL
    CALL    POST_PROCESS   
    
    CALL    SIMULATE_SIMPLE
    CALL    SAVE_SIMPLE
       
    prog_time_stop    = DCLOCK()               ! timer
	PRINT *, "**************************************************"
	PRINT *, "**************************************************"
	PRINT *, "Code took: ", ceiling ((prog_time_stop - prog_time_start)/60), " minutes"
    PRINT *, "**************************************************"
	PRINT *, "**************************************************"
                
END SUBROUTINE COMPUTEMODEL_MAIN

SUBROUTINE SIMULATEMODEL_MAIN
    ! simulate model without solving, using initial guess/solution
    ! (a) simulate
    ! (b) save results
    
    IMPLICIT NONE
    
    REAL (DP)           ::  prog_time_start, prog_time_stop
    
    prog_time_start    = DCLOCK() 				! timer

    CALL    POST_PROCESS       
    CALL    SIMULATE_SIMPLE
    CALL    SAVE_SIMPLE
       
    prog_time_stop    = DCLOCK()               ! timer
	PRINT *, "**************************************************"
	PRINT *, "**************************************************"
	PRINT *, "Code took: ", ceiling ((prog_time_stop - prog_time_start)/60), " minutes"
    PRINT *, "**************************************************"
	PRINT *, "**************************************************"
                
END SUBROUTINE SIMULATEMODEL_MAIN

SUBROUTINE SOLVEMODEL(showdetails)

    IMPLICIT NONE

    INTEGER, INTENT(IN), OPTIONAL :: showdetails
    
    REAL (DP)           ::  inichi
    REAL (DP)           ::  loop_time_start, loop_time_stop, &
                            total_time_start, total_time_stop

    LOGICAL     ::  showdetails_prm
    
    IF (PRESENT(showdetails)) THEN
        showdetails_prm  = (showdetails .EQ. 1)
    ELSE
        showdetails_prm  = .TRUE.
    END IF    
    
    PRINT *, " BEGIN - SOLVE MODEL"
    99  FORMAT (/1X,'beta = ', F9.5,    &
                ' | d0   = ', F9.5,     &
                ' | d1_h = ', F9.5,     &
                ' | d1_l = ', F9.5) 
    WRITE(*,99) bita, d0, d1_h, d1_l
    
    ! Initialize
    IF (readguess .EQ. 0) THEN
        sav = zerodp
        def = zerodp
    END IF

    IF ((readguess .EQ. 1) .AND. (readinichi .EQ. 1)) THEN
    !! get starting value to chi
        inichi = MAXVAL(MINLOC(ABS(chigrid-chi)))
    ELSE
        inichi = 1
    END IF
        
    ! Begin value function and price function iteration
    conv_ind            = 0
    total_time_start    = DCLOCK() ! timer
    
    CHI_LOOP: DO iterchi = inichi,N_chi
        PRINT *, " OUTER OUTER ADJ LOOP -  CHI ITER     =       ", iterchi
        chi = chigrid(iterchi)
        avg_pfit_err = 5.0_dp
        
        OUTER_LOOP: DO iter = 1,pfitmax
            IF (showdetails_prm) THEN
                PRINT *, "      OUTER PRICE FCN LOOP - PFIT     =       ", iter
            ELSE IF ((iter.EQ.10).OR.(iter.EQ.25).OR.(iter.EQ.50).OR.(iter.EQ.100)) THEN
                PRINT *, "      OUTER PRICE FCN LOOP - PFIT     =       ", iter
            END IF
            loop_time_start = DCLOCK()   ! timer
        
            psavo   = psav
            savoo   = sav
            defo    = def
        
            INNER_LOOP: DO iter2 = 1,vfitmax

                !PRINT *, "            INNER LOOP VF - VF IT     =       ", iter2
                vo = v
                savo = sav
                vdefo = vdef
                
                !!
                !! Compute the value of repayment
                !!
                !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(assets,evo,i_bp_star,price,c,ev,j_bp_min,j_bp_max)
                !$OMP DO COLLAPSE(3)
                LOOP_R: DO i_rgm=1,rgm                          ! loop over current default cost regime
                    LOOP_B: DO i_b=1,ngpb                       ! loop over incoming asset level  B
                        LOOP_S: DO i_ss=1,ss                    ! loop over state s=(y,pi)

                            assets      = b(i_b)
                            evo         = -infty                ! initial guess for maxval V(B,s,:)
                            i_bp_star   = 1                     ! initial guess for B'(B,s)
                            
                            j_bp_min   = i_bp_min(i_b)
                            j_bp_max   = i_bp_max(i_b)

                            FIND_BPRIME: DO i_bp=j_bp_min,j_bp_max      ! loop over next asset level B' to update
                                price   = p(i_rgm,i_ss, i_bp)           ! cost of borrowing B' given B & s
                                issue   = bprime(i_bp) - (onedp-delta)*assets
                                c       = ygrid(i_ss) + assets*(rc+delta)               &
                                            - price*issue &
                                            - adjcost(savoo(i_rgm,i_b,i_ss),bprime(i_bp),chi)

                                NEG_CONS: IF ( c<= zerodp ) THEN
                                    ev  = -infty			    ! set value function to -\infty
                                ELSE IF (ABS(issue) > issue_max) THEN
                                    ev  = -infty
                                ELSE							! ELSE: feasible choice
                                    ev = utility(c)             ! flow utility
                                    DO i_ssp=1,ss               ! compute implied value for (B,s,B',s')
                                        DO i_rgmp=1,rgm         ! regime  next period
                                        ev = ev +                       &
                                                bita                    &
                                                *ptran(i_ss,i_ssp)      &
                                                *pr_rgm(i_rgm,i_rgmp)   &
                                                *INTERPOLATE(           &
                                                    b ,                     &
                                                    vo(i_rgmp,:,i_ssp) ,    &
                                                    ngpb ,                  &
                                                    bprime_inf(i_ssp,i_bp)  &
                                                )
                                        ! INTERPOLATE(X,Y,siz,xi)
                                        END DO
                                    END DO
                                END IF NEG_CONS

                                NEW_MAXVAL: IF (ev >= evo) THEN
                                    evo = ev                    ! update the new maxval V(B,s,:)
                                    i_bp_star = i_bp            ! update corresponding argmax B'(B,s)
                                END IF NEW_MAXVAL

                            END DO FIND_BPRIME

                            ! update the policy functions
                            sav(i_rgm, i_b, i_ss)   = bprime(i_bp_star)         ! optimal assets
                            psav(i_rgm, i_b, i_ss)  = p(i_rgm,i_ss, i_bp_star)  ! price given optimal asset
                            v(i_rgm, i_b, i_ss)     = evo                       ! optimal value

                            !! Compute the value of default
                            IF (nodefault == 0) THEN
                                ev = utilaut(i_rgm,i_ss)              ! flow utility
                                
                                DO i_ssp=1,ss                   ! compute implied value for (B,s,B',s')
                                    DO i_rgmp=1,rgm             ! regime  next period
                                    ev  = ev +                      &
                                            bita*                   &
                                            ptran(i_ss,i_ssp)       &
                                            *pr_rgm(i_rgm,i_rgmp)   &
                                            *(                      &
                                                INTERPOLATE(        &
                                                    b,              &
                                                    theta_mat(i_rgm,i_rgmp)*vo(i_rgmp,:,i_ssp) +                    &
                                                        (onedp-theta_mat(i_rgm,i_rgmp))*vdefo(i_rgmp,:,i_ssp),      &
                                                    ngpb , lambda*assets/(1+pigrid(i_ssp))                          &
                                                        )            &
                                            )
                                    ! INTERPOLATE(X,Y,siz,xi)
                                    END DO
                                END DO
                                
                                vdef(i_rgm, i_b,i_ss) =  ev
                            
                            END IF

                        END DO LOOP_S
                    END DO LOOP_B
                END DO LOOP_R
                !$OMP END DO
                !$OMP END PARALLEL
        
                !!
                !! update the no-default policy functions
                !!
                savnd   = sav   ! recall no default was assumed
                psavnd  = psav  ! recall no default was assumed
                vnd     = v     ! recall no default was assumed
        
                IF (nodefault == 1) THEN
                    vdef    = -infty
                    def     = zerodp
                ELSE
                    !! Compute v = max (v_no_default, v_default)
                    def     = zerodp                            ! Assume no default. Update below
                    DO i_rgm=1,rgm
                        DO i_ss=1,ss                                ! loop over income states s = (y,pi)
                            DO i_b=1,ngpb                           ! loop over incoming assets B
                                IF (v(i_rgm, i_b, i_ss) < vdef(i_rgm, i_b, i_ss)) THEN
                                    v(i_rgm, i_b, i_ss)    = vdef(i_rgm, i_b, i_ss)
                                    def(i_rgm, i_b, i_ss)  = onedp         ! default IF v_c < v_d
                                    sav(i_rgm, i_b, i_ss)  = b(i_b)        ! assets = B
                                    psav(i_rgm, i_b, i_ss) = eps           ! cost of borrowing infinite
                                END IF
                            END DO
                        END DO
                    END DO

                END IF
            
                !! POLICY FUNCTION CONVERGENCE
                gapsavg = (SUM (ABS (sav-savo))) / DBLE(rgm*ngpb*ss)
                gapsmax = (MAXVAL (ABS (sav-savo))) 
        
                !PRINT *, " "
                !PRINT *, "            INNER LOOP VF - VF IT     =       ", iter2, " - END "
                !PRINT *, " avg abs errors (savings)  ", gapsavg  
                !PRINT *, " max abs errors (savings)  ", gapsmax
                !arg2 = MAXLOC(ABS(sav-savo))
                !i_b  = arg2(1)
                !i_ss = arg2(2)
                !PRINT *, "maximum change at B=",b(i_b)," y=",ygrid(i_s s)
                !PRINT *, "old B'(B,s) = ", savo(i_b,i_ss)
                !PRINT *, "new B'(B,s) = ", sav(i_b,i_ss)      
                
                CONV_VF: IF ( gapsavg<critsavg ) THEN
                    IF (showdetails_prm) THEN
                        PRINT *, " avg abs errors (savings)  ", gapsavg 
                        PRINT *, "INNER LOOP VF done after iteration ", iter2
                    END IF
                    EXIT INNER_LOOP
                END IF CONV_VF
                
                ! UNNECESSARY AVERAGING ** MARKED FOR DELETION **
                !sav = weightnew*sav + (onedp-weightnew)*savo
        
            END DO INNER_LOOP
            
            !! PRICE FUNCTION UPDATE
            IF (showdetails_prm) THEN
                PRINT *, " "
                PRINT *, "            -- UPDATING DEFAULT PRICE --      "
            END IF
            
            ! iterate on default price
            pdef2 = pdef
            IF (nodefault==1) THEN
                pdef1 = pdef
            ELSE
                
                PDEFK_LOOP: DO piter=1,pitmax
                    
                    !PRINT *, "            INNER LOOP PDEF - IT      =       ", piter
                                
                    !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(assets,evp,bpinfl,defpr,price,pricedef)
                    !$OMP DO COLLAPSE(3)
                    DO i_rgm=1,rgm                                      ! regime
                        DO i_b=1,ngpb                                   ! loop over incoming assets B
                            DO i_ss=1,ss                                ! loop over income states s = (y,pi)

                                assets  = b(i_b)
                                evp	    = zerodp                                ! initialize cost of borrowing q(s,B')

                                DO i_rgmp=1,rgm                                     ! regime  next period
                                    DO i_ssp=1,ss                                   ! loop over future states s'
                                        bpinfl  = lambda*assets/(1+pigrid(i_ssp))   ! inflated & diluted debt
                                        defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl )

                                        ! Equilibrium price next period under NO default
                                        price   = INTERPOLATE(b, psavnd(i_rgmp,:,i_ssp), ngpb, bpinfl )

                                        ! Bond price next period in DEFAULT
                                        pricedef = INTERPOLATE(b, pdef2(i_rgmp,:,i_ssp),ngpb,bpinfl )

                                        ! update borrowing cost
                                        evp = evp +                                         &
                                                ptran(i_ss,i_ssp)                           &
                                                *pr_rgm(i_rgm,i_rgmp)                       &
                                                *sdf(i_ss,i_ssp)                                            &
                                                *lambda/(onedp + pigrid(i_ssp))                             &
                                                *(                                                          &
                                                    theta_mat(i_rgm,i_rgmp)*(onedp-defpr)*((rc+delta) + (onedp-delta)*price)    &
                                                    + (onedp-theta_mat(i_rgm,i_rgmp)+theta_mat(i_rgm,i_rgmp)*defpr)*pricedef    &
                                                )
                                    END DO
                                END DO
                            
                                pdef1(i_rgm,i_b,i_ss) = evp    ! implied borrowing cost
                            
                            END DO
                        END DO
                    END DO
                    !$OMP END DO
                    !$OMP END PARALLEL
                    
                    gappdefavg = (SUM (ABS (pdef1-pdef2))) / DBLE(rgm*ngpb*ss)
                    !PRINT *, " avg abs errors (price)  ", gappdefavg 
        
                    CONV_PDEFK: IF (gappdefavg<critpavg)  THEN
                        IF (showdetails_prm) THEN
                            PRINT *, " avg abs errors (price)  ", gappdefavg 
                            PRINT *, "DEFAULT PRICE UPDATING DONE after iteration", piter
                        END IF
                        EXIT PDEFK_LOOP
                    ELSE IF (gappdefavg>infty)  THEN
                        IF (showdetails_prm) THEN
                            PRINT *, "DEFAULT PRICE BLEW UP :("
                        END IF
                        !EXIT PDEFK_LOOP
                        CYCLE CHI_LOOP
                    ELSE IF (ISNAN(gappdefavg))  THEN
                        IF (showdetails_prm) THEN
                            PRINT *, "DEFAULT PRICE NaN :("
                            END IF
                        !EXIT PDEFK_LOOP
                        CYCLE CHI_LOOP
                    ELSE
                        ! Update at slower rate with more iterations
                        adjweight = pweightnew*( onedp - REAL(piter-1)/REAL(pitmax) )
                        pdef2 = adjweight*pdef1 + (onedp-adjweight)*pdef2   ! update price
                    END IF CONV_PDEFK
                END DO PDEFK_LOOP
            END IF            

            IF (showdetails_prm) THEN
                PRINT *, " "
                PRINT *, "            -- UPDATING PRICE SCHEDULE --     "
            END IF
            
            ! iterate on pricing kernel
            p2 = p
            avg_error = 5.0_dp
            
            PK_LOOP: DO piter=1,pitmax
                
                !PRINT *, "            INNER LOOP PRICE FCN - IT =       ", piter

                !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(evp_nodef,evp_def,bpinfl,defpr,price,pricedef)
                !$OMP DO COLLAPSE(3)
                DO i_rgm=1,rgm                                  ! regime
                    DO i_ss=1,ss                                    ! loop over income states s = (y,pi)
                        DO i_bp=1,ngpbprime                         ! loop candidate outgoing assets B'

                            evp_nodef   = zerodp                    ! initialize no default part of q
                            evp_def     = zerodp                    ! initialize default part of q

                            DO i_rgmp=1,rgm                             ! regime  next period
                                DO i_ssp=1,ss                               ! loop over future states s'
                                    bpinfl  = bprime_inf(i_ssp,i_bp)        ! inflated debt
                                    defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl )

                                    ! Equilibrium price next period under NO DEFAULT
                                    price       = INTERPOLATE(b, psavnd(i_rgmp,:,i_ssp), ngpb, bpinfl )

                                    ! Bond price next period in DEFAULT
                                    pricedef    = INTERPOLATE(b, pdef2(i_rgmp,:,i_ssp),ngpb,bpinfl )

                                    ! update borrowing cost (no default next period)
                                    evp_nodef   = evp_nodef +                       &
                                                    ptran(i_ss,i_ssp)               &
                                                    *pr_rgm(i_rgm,i_rgmp)           &
                                                    *sdf(i_ss,i_ssp)                &
                                                    *(onedp - defpr)*               &
                                                    ((rc+delta)+(onedp-delta)*price)   &
                                                    /(onedp + pigrid(i_ssp))

                                    ! update borrowing cost (default next period)
                                    evp_def     = evp_def +                         &
                                                    ptran(i_ss,i_ssp)               &
                                                    *pr_rgm(i_rgm,i_rgmp)           &
                                                    *sdf(i_ss,i_ssp)                &
                                                    *defpr*                         &
                                                    pricedef                        &
                                                    /(onedp + pigrid(i_ssp))
                                END DO
                            END DO

                            p1(i_rgm,i_ss,i_bp)         = evp_nodef + evp_def   ! implied borrowing cost
                            p_nodef(i_rgm,i_ss,i_bp)    = evp_nodef             ! implied borrowing cost (future no default states)
                            p_def(i_rgm,i_ss,i_bp)      = evp_def               ! implied borrowing cost (future default states)

                        END DO
                    END DO
                END DO
                !$OMP END DO
                !$OMP END PARALLEL
                
                gappavg = (SUM (ABS (p1-p2))) / DBLE(rgm*ss*ngpbprime)
                avg_error = 0.6_dp*avg_error + 0.4_dp*gappavg
                
                !PRINT *, " avg abs errors (price)  ", gappavg 
                !PRINT *, " cumulative avg error    ", avg_error 
        
                CONV_PK: IF (gappavg<critpavg)  THEN
                    IF (showdetails_prm) THEN
                        PRINT *, "PRICING KERNEL UPDATING DONE"
                    END IF
                    EXIT PK_LOOP
                ELSE IF (gappavg>infty)  THEN
                    IF (showdetails_prm) THEN
                        PRINT *, "PRICING KERNEL BLEW UP :("
                    END IF
                    !EXIT PK_LOOP
                    CYCLE CHI_LOOP
                ELSE IF (ISNAN(gappavg))  THEN
                    IF (showdetails_prm) THEN
                        PRINT *, "PRICING KERNEL NaN :("
                    END IF
                    !EXIT PK_LOOP
                    CYCLE CHI_LOOP
                ELSE IF ((gappavg>avg_error).AND.(piter>pitmin))  THEN
                    IF (showdetails_prm) THEN
                        PRINT *, "ERRORS NOT DECREASING :("
                    END IF
                    !EXIT PK_LOOP
                    CYCLE CHI_LOOP
                ELSE
                    ! Update at slower rate with more iterations
                    adjweight = pweightnew*( onedp - REAL(piter-1)/REAL(pitmax) )
                    ! update price
                    p2 = adjweight*p1 + (onedp-adjweight)*p2    
                END IF CONV_PK
            END DO PK_LOOP
                    
            loop_time_stop  = DCLOCK()                  ! timer
            
            gappavg         = (SUM (ABS (p1-p))) / DBLE(rgm*ss*ngpbprime)
            gappdefavg      = (SUM (ABS (pdef1-pdef))) / DBLE(rgm*ngpb*ss)
            gappsav         = (SUM (ABS (psav-psavo))) / DBLE(rgm*ngpb*ss)
            gappmax         = (MAXVAL (ABS (p1-p))) 
            gapdchg         = SUM (ABS (def-defo))
            gapsavg         = (SUM (ABS (savoo-sav)))/ DBLE(rgm*ngpb*ss)
            avg_pfit_err    = 0.75_dp*avg_pfit_err + 0.25_dp*gappavg
            
            IF (showdetails_prm) THEN
                PRINT *, " avg abs errors (price)  ", gappavg 
                PRINT *, " cumulative avg error    ", avg_pfit_err 
            END IF
            
10          FORMAT (1X, 'P iteration: ', I3, ' |  chi: ',F8.4)
            
            IF (showdetails_prm) THEN
                PRINT *, ""
                WRITE (*,10) iter, chi
                PRINT *, "iteration took  ", loop_time_stop - loop_time_start, " seconds"
                PRINT *, "Number of price iterations:       ", piter
                PRINT *, "avg abs errors (price schedule)   ", gappavg
                PRINT *, "avg abs errors (def price)        ", gappdefavg
                PRINT *, "avg abs errors (eqm. price)       ", gappsav
                PRINT *, "tot # changes  (default policy)   ", gapdchg
                PRINT *, "avg abs errors (savings policy)   ", gapsavg
            END IF
            
            CONV_PRICE: IF (gappavg<critpavg .AND. gappdefavg<critpavg .AND.gappsav<critpsav ) THEN
                p           = p1            ! update price
                pdef        = pdef1
                conv_ind    = 1
                
                total_time_stop    = DCLOCK()               ! timer
                
                IF (showdetails_prm) THEN
                    PRINT *, " "
                    PRINT *, "**************************************************"
                    PRINT *, "CHI: ", chi
                    PRINT *, "CONVERGED: ", ceiling ((total_time_stop - total_time_start)/60), " minutes"
                    PRINT *, "**************************************************"
                END IF
                EXIT CHI_LOOP
            
            ELSE IF ( ISNAN(gappavg) .OR. ISNAN(gappdefavg) .OR. ISNAN(gappsav) ) THEN
                IF (showdetails_prm) THEN
                    PRINT *, " "
                    PRINT *, "      NaN error with price changes       "
                    PRINT *, " "
                END IF
                !EXIT CHI_LOOP
                CYCLE CHI_LOOP
            ELSE IF (iter == pfitmax .AND. iterchi == N_chi) THEN
                IF (showdetails_prm) THEN
                    PRINT *, " "
                    PRINT *, "      RUN OUT OF ITERATIONS AND ADJ. COST. ITERATIONS       "
                    PRINT *, " "
                END IF
                
            ELSE IF ((gappavg>avg_pfit_err).AND.(iter>pfitmin))  THEN
                IF (showdetails_prm) THEN
                    PRINT *, "ERRORS NOT DECREASING :("
                END IF
                !EXIT PK_LOOP
                CYCLE CHI_LOOP
                    
            ELSE
                ! Update at slower rate with more iterations
                adjweight = MAX( pweightmin, pweightmax                 &
                    *( onedp - REAL(iter-1)/REAL(pfitmax) )**adjpwr )
                IF (showdetails_prm) THEN
                    PRINT *, 'weight: ', adjweight
                    PRINT *, " "
                END IF
                ! update price
                p       = (onedp-adjweight)*p + adjweight*p1            
                pdef    = (onedp-adjweight)*pdef + adjweight*pdef1
            END IF CONV_PRICE
        
        END DO OUTER_LOOP
    END DO CHI_LOOP
    
    IF (CHI>0) THEN
        !!
        !! Run one more optimization with chi=0
        !! Compute the value of repayment
        !!
    
        !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(assets,evo,i_bp_star,price,c,ev,j_bp_min,j_bp_max)
        !$OMP DO COLLAPSE(3)
        DO i_rgm=1,rgm                                  ! regime
            DO i_b=1,ngpb                               ! loop over incoming asset level  B
                DO i_ss=1,ss                            ! loop over state s=(y,pi)
                    assets      = b(i_b)
                    evo         = -infty                ! initial guess for maxval V(B,s,:)
                    i_bp_star   = 1                     ! initial guess for B'(B,s)

                    j_bp_min   = i_bp_min(i_b)
                    j_bp_max   = i_bp_max(i_b)

                    DO i_bp=j_bp_min,j_bp_max           ! loop over next asset level B' to update
                        price   = p(i_rgm,i_ss, i_bp)   ! cost of borrowing B' given B & s

                        c       = ygrid(i_ss) + assets*(rc+delta)               &
                                    - price*( bprime(i_bp) - (onedp-delta)*assets )

                        IF ( c<= zerodp ) THEN
                            ev  = -infty			    ! set value function to -\infty
                        ELSE							! ELSE: feasible choice
                            ev = utility(c)             ! flow utility
                            
                            DO i_rgmp=1,rgm                 ! regime  next period
                                DO i_ssp=1,ss               ! compute implied value for (B,s,B',s')
                                    ev = ev +                       &
                                            bita                    &
                                            *ptran(i_ss,i_ssp)      &
                                            *pr_rgm(i_rgm,i_rgmp)   &
                                            *INTERPOLATE(           &
                                                b ,                 &
                                                v(i_rgmp,:,i_ssp) , &
                                                ngpb ,              &
                                                bprime_inf(i_ssp,i_bp)  &
                                                        )
                                    ! INTERPOLATE(X,Y,siz,xi)
                                END DO
                            END DO
                            
                        END IF

                        IF (ev >= evo) THEN
                            evo = ev                    ! update the new maxval V(B,s,:)
                            i_bp_star = i_bp            ! update corresponding argmax B'(B,s)
                        END IF

                    END DO

                    ! update the policy functions
                    savo(i_rgm,i_b, i_ss)     = bprime(i_bp_star)         ! optimal assets

                END DO
            END DO
        END DO
        !$OMP END DO
        !$OMP END PARALLEL

        !! CONVERGENCE
        gapsavg = (SUM (ABS (sav-savo))) / DBLE(rgm*ngpb*ss)
        gapsmax = (MAXVAL (ABS (sav-savo))) 
                
        gap3 = 0
        !DO i_rgm=1,rgm                                      ! regime
        !    DO i_ss=1,ss                                    ! loop over income states s = (y,pi)
        !        DO i_b=1,ngpb                               ! loop over incoming assets B
        !            IF ( def(i_rgm,i_b, i_ss) == zerodp ) THEN
        !                IF (sav(i_rgm,i_b,i_ss) .NE. savo(i_rgm,i_b,i_ss)) THEN
        !                    gap3 = gap3 + 1
        !                END IF
        !            END IF
    	   !     END DO
        !    END DO
        !END DO
        gap3 = COUNT( (def .EQ. zerodp) .AND. (sav .NE. savo) )
        
        arg2    = MAXLOC(ABS(sav-savo))
        i_rgm   = arg2(1)
        i_b     = arg2(2)
        i_ss    = arg2(3)
    
        IF (showdetails_prm) THEN
            PRINT *, " "
            PRINT *, " -- Reoptimization with chi=0 -- "
            PRINT *, " avg abs errors (savings)        ", gapsavg  
            PRINT *, " max abs errors (savings)        ", gapsmax
            PRINT *, "# points with changes in savings ", gap3
            PRINT *, "maximum change at B=",b(i_b)," y=",ygrid(i_ss)," in d1(rgm)=",d1_rgm(i_rgm)
            PRINT *, "B'(B,s) with chi > 0: ", sav(i_rgm,i_b,i_ss)
            PRINT *, "B'(B,s) with chi = 0: ", savo(i_rgm, i_b,i_ss)  
        END IF 
    
        ! Compute implicit adjustment cost
        avgadjcost  = onedp
        tmp1        = 0
        
        DO i_rgm=1,rgm                                  ! regime
            DO i_ss=1,ss                                ! loop over income states s = (y,pi)
                DO i_b=1,ngpb                           ! loop over incoming assets B
                    IF ( def(i_rgm,i_b, i_ss) == zerodp ) THEN
                        tmp1        = tmp1 + 1
                        c           = ygrid(i_ss)  + b(i_b)         &            ! consumption under policy with chi>0
                                        - psav(i_rgm,i_b,i_ss)*sav(i_rgm,i_b,i_ss) 
                    
                        avgadjcost = avgadjcost         &                       ! implicit ajustment cost (percent consumption)
                                        *(onedp +       &
                                            adjcost(sav(i_rgm,i_b,i_ss),    &
                                                    savo(i_rgm,i_b,i_ss),   &
                                                    chi)                    &   
                                        /c) 
                    END IF
                END DO
            END DO
        END DO
        
        avgadjcost = 100.0_dp*(avgadjcost**(onedp/REAL(tmp1,DP))-onedp)  ! geometric mean
        
        PRINT *, "Implied adjustment cost (percent consumption): ", avgadjcost
        
    END IF
    
    ! Computation information
    compconv%conv_vf_val    = conv_ind
    compconv%chi_val        = chi
    compconv%adjcost_val    = avgadjcost
    compconv%ngpb_val       = ngpb
    compconv%ngpbprime_val  = ngpbprime
    compconv%ss_val         = ss
    compconv%i_b_zero_val   = i_b_zero   
    
    IF (showdetails_prm) THEN
        PRINT *, " "
        PRINT *, " END - SOLVING MODEL"
        PRINT *, " "
    END IF
    
END SUBROUTINE SOLVEMODEL

SUBROUTINE POST_PROCESS
    ! This routine runs after the model has been solved
    ! Given the policy functions, the routine does the following
    ! (a) compute expected default probabilities
    ! (b) compute borrower consumption
    ! (c) compute lender value function
    ! (d) compute lender sdf
    
    
    REAL (DP)   ::  x_,y_,z_
    REAL (DP)   ::  x_m,y_m,z_m
    REAL (DP)   ::  xy_m,yz_m,xz_m
	REAL (DP)	::	real_evp_nodef,real_evp_def, real_price, real_pricedef

    
    PRINT *, "Post-iteration processing ..."

	
    !! Compute consumption decision rules and next period default probabilities
    DO i_rgm=1,rgm                                  ! regime
        DO i_ss=1,ss
            DO i_b=1,ngpb
            
                IF (def(i_rgm, i_b, i_ss) == onedp ) THEN

                    ! Compute BORROWER consumption
                    cons(i_rgm, i_b, i_ss) = conaut(i_rgm, i_ss)
                                                    
                    ! Compute LENDER repayment
                    payL(i_rgm,i_b, i_ss) 	= zerodp
					consL(i_rgm,i_b, i_ss) 	= conaut(i_rgm, i_ss)                   &
                                                    *((onedp - tau_govt)/tau_govt)  &
												+ payL(i_rgm,i_b, i_ss)
                    
                    defp    = zerodp                      	! initialize Pr (def | B,s)
                    DO i_rgmp=1,rgm  
                        DO i_ssp=1,ss                                   ! loop over future states s'
                            bpinfl = lambda*b(i_b)          &
                                        /(onedp+pigrid(i_ssp))
                            
                            defp = defp +                   &
                                    ptran(i_ss,i_ssp)       &           
                                    *pr_rgm(i_rgm,i_rgmp)   &
                                    *INTERPOLATE(           &
                                        b ,                 &
                                        def(i_rgmp,:,i_ssp) ,& 
                                        ngpb ,              &
                                        bpinfl )
                        END DO
                    END DO
    			    dprob(i_rgm,i_b,i_ss) = onedp - theta + theta*defp
                    
                ELSE
                    ! Compute BORROWER consumption
                    cons(i_rgm, i_b, i_ss) = ygrid(i_ss) +              &
                                            (rc+delta)*b(i_b)           &
                                            - psav(i_rgm, i_b, i_ss)    &
                                                *(sav(i_rgm,i_b,i_ss)   &
                                            -(1-delta)*b(i_b)) 
                    
                                
                    ! Compute LENDER repayment flows
                    payL(i_rgm,i_b, i_ss)  = - (rc+delta)*b(i_b)           	&
                                                + psav(i_rgm, i_b, i_ss)    &
                                                    *(sav(i_rgm,i_b,i_ss)   &
                                                -(1-delta)*b(i_b))
					
					consL(i_rgm,i_b, i_ss) 	= ygrid(i_ss)                           &
                                                    *((onedp - tau_govt)/tau_govt)  &
												+ payL(i_rgm,i_b, i_ss)
					
    			    defp    = zerodp                      ! initialize Pr (def | B,s)
                    DO i_rgmp=1,rgm  
    			        DO i_ssp=1,ss                           ! loop over future states s'
                            bpinfl = sav(i_rgm,i_b,i_ss)    &
                                        /(onedp+pigrid(i_ssp))
                            
                            defp = defp +                   &
                                    ptran(i_ss,i_ssp)       &  
                                    *pr_rgm(i_rgm,i_rgmp)   &
                                    *INTERPOLATE(           &
                                        b ,                 &
                                        def(i_rgmp,:,i_ssp) ,&
                                        ngpb ,              &
                                        bpinfl )        
                        END DO
                    END DO
    			    dprob(i_rgm,i_b,i_ss) = defp
                    
                END IF
            
                ! Compute output
                yy(i_rgm, i_b, i_ss) = ygrid(i_ss)
            
                ! Compute no default consumption
                consnd(i_rgm,i_b, i_ss) = ygrid(i_ss) +                 &
                                            (rc+delta)*b(i_b)           &
                                            - psavnd(i_rgm, i_b, i_ss)  &
                                            *(savnd(i_rgm,i_b,i_ss)     &
                                            -(1-delta)*b(i_b))
            
                ! Compute in-default future expected default prob
                defp    = zerodp                      	! initialize Pr (def | B,s)
                DO i_rgmp=1,rgm  
                    DO i_ssp=1,ss                           ! loop over future states s'
                        bpinfl = lambda*b(i_b)              &
                                    /(onedp+pigrid(i_ssp))
                        defp = defp +                       &
                                ptran(i_ss,i_ssp)           &  
                                *pr_rgm(i_rgm,i_rgmp)       &
                                *INTERPOLATE(               &
                                    b ,                     &
                                    def(i_rgmp,:,i_ssp) ,   &
                                    ngpb ,                  &
                                    bpinfl )
                    END DO
                END DO
                dprobid(i_rgm,i_b,i_ss) = onedp - theta + theta*defp

                ! Compute no-default future expected default prob
                defp    = zerodp                      	! initialize Pr (def | B,s)
                DO i_rgmp=1,rgm  
                    DO i_ssp=1,ss                           ! loop over future states s'
                        bpinfl = savnd(i_rgm,i_b,i_ss)      &
                                    /(onedp+pigrid(i_ssp))
                        
                        defp = defp +                       &
                                ptran(i_ss,i_ssp)           &
                                *pr_rgm(i_rgm,i_rgmp)       &
                                *INTERPOLATE(               &
                                    b ,                     &
                                    def(i_rgmp,:,i_ssp) ,   &
                                    ngpb ,                  &
                                    bpinfl )
                    END DO
                END DO
                dprobnd(i_rgm,i_b,i_ss) = defp          

            END DO
        END DO 
    END DO
    
    !! LENDER VALUE FUNCTION
    
	! payment only value stream
    gapl 		= onedp
	v_l 		= zerodp
	vnd_l 		= zerodp
	vdef_l 		= zerodp
	
	! consumption value stream
	gapll 		= onedp
	vv_l 		= zerodp
	vvnd_l 		= zerodp
	vvdef_l		= zerodp
	
    LENDER: DO iter = 1,vlitmax
            
        ! Keep previous value function
		vo_l 		= v_l
		vvo_l 		= vv_l
		
		vndo_l 		= vnd_l
		vvndo_l 	= vvnd_l
	
		vdefo_l 	= vdef_l
		vvdefo_l 	= vvdef_l
		
        !$OMP PARALLEL DEFAULT(SHARED) PRIVATE(assets,ev,evv,bpinfl,defpr)
        !$OMP DO COLLAPSE(3)        
        DO i_rgm=1,rgm                                  
		DO i_ss=1,ss
		DO i_b=1,ngpb
			
			assets      = b(i_b)
			
			! no default
			ev 	= payL(i_rgm,i_b, i_ss)						! payments only
			evv = consL(i_rgm,i_b, i_ss) 					! consumption level
			
				DO i_rgmp=1,rgm  
				DO i_ssp=1,ss
					bpinfl = savnd(i_rgm,i_b,i_ss)      	&
							/(onedp+pigrid(i_ssp))
					defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl )
					
					ev 	= ev +                       		&
							ptran(i_ss,i_ssp)           	&
							*pr_rgm(i_rgm,i_rgmp)       	&
							*sdf(i_ss,i_ssp)       			&
							*(								&
								(1-defpr)					&
								*INTERPOLATE(               &
									b ,                     &
									vnd_l(i_rgmp,:,i_ssp),	&
									ngpb ,                  &
									bpinfl )				&
								+ defpr						&
								*INTERPOLATE(               &
									b ,                     &
									vdef_l(i_rgmp,:,i_ssp),	&
									ngpb ,                  &
									bpinfl )				&	
							)
							
					
					
					evv = evv +                       		&
							ptran(i_ss,i_ssp)           	&
							*pr_rgm(i_rgm,i_rgmp)       	&
							*sdf(i_ss,i_ssp)       			&
							*(								&
								(1-defpr)					&
								*INTERPOLATE(               &
									b ,                     &
									vvnd_l(i_rgmp,:,i_ssp),	&
									ngpb ,                  &
									bpinfl )				&
								+ defpr						&
								*INTERPOLATE(               &
									b ,                     &
									vvdef_l(i_rgmp,:,i_ssp),&
									ngpb ,                  &
									bpinfl )				&	
							)		
				END DO
				END DO
				
			vnd_l(i_rgm,i_b,i_ss) 	= ev 					! payments 
			vvnd_l(i_rgm,i_b,i_ss) 	= evv 					! consumption 
			
			! default
			ev 	= zerodp
			evv	= zerodp
			
				DO i_rgmp=1,rgm  
				DO i_ssp=1,ss
					bpinfl = lambda*assets/(onedp+pigrid(i_ssp))
					defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl )
					
					ev 	= ev +                       		&
							ptran(i_ss,i_ssp)           	&
							*pr_rgm(i_rgm,i_rgmp)       	&
							*sdf(i_ss,i_ssp)       			&
							*(								&
								theta*(1-defpr)				&
								*INTERPOLATE(               &
									b ,                     &
									vnd_l(i_rgmp,:,i_ssp),	&
									ngpb ,                  &
									bpinfl )				&
								+ (1-theta + theta*defpr)	&
								*INTERPOLATE(               &
									b ,                     &
									vdef_l(i_rgmp,:,i_ssp),	&
									ngpb ,                  &
									bpinfl )				&	
							)
							
					
					evv	= evv +                       		&
							ptran(i_ss,i_ssp)           	&
							*pr_rgm(i_rgm,i_rgmp)       	&
							*sdf(i_ss,i_ssp)       			&
							*(								&
								theta*(1-defpr)				&
								*INTERPOLATE(               &
									b ,                     &
									vvnd_l(i_rgmp,:,i_ssp),	&
									ngpb ,                  &
									bpinfl )				&
								+ (1-theta + theta*defpr)	&
								*INTERPOLATE(               &
									b ,                     &
									vvdef_l(i_rgmp,:,i_ssp),&
									ngpb ,                  &
									bpinfl )				&	
							)
				END DO
				END DO
				
			vdef_l(i_rgm,i_b,i_ss) 	= ev 					! payments
			vvdef_l(i_rgm,i_b,i_ss) = evv 					! consumption
			
			IF (def(i_rgm,i_b,i_ss).EQ.onedp) THEN
				v_l(i_rgm,i_b,i_ss) 	= vdef_l(i_rgm,i_b,i_ss) 	! payments
				vv_l(i_rgm,i_b,i_ss) 	= vvdef_l(i_rgm,i_b,i_ss) 	! consumption
			ELSE
				v_l(i_rgm,i_b,i_ss) 	= vnd_l(i_rgm,i_b,i_ss) 	! payments
				vv_l(i_rgm,i_b,i_ss) 	= vvnd_l(i_rgm,i_b,i_ss) 	! consumption
			END IF

		END DO
		END DO 
        END DO
        !$OMP END DO
        !$OMP END PARALLEL
    
        !! VALUE FUNCTION CONVERGENCE
        gapl 	= SUM(ABS (v_l - vo_l )) / DBLE(rgm*ngpb*ss) 
        gapll 	= SUM(ABS (vv_l - vvo_l )) / DBLE(rgm*ngpb*ss) 
        
        IF ( ( (gapl<critv_l) .AND. (gapll<critv_l)) .OR. (iter==vlitmax) ) THEN
50          FORMAT ('iteration: ',I5, ' avg errors (v_l): ',F9.4, ' avg errors (vv_l): ',F9.4)         
            WRITE(*,50) iter, gapl, gapll
            EXIT LENDER
        END IF
    END DO LENDER
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! REAL DISCOUNTED PRICE - VERSION 1 !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
	! Initialize real prices
	real_p 			= p			! real bond price schedule
	real_pdef 		= pdef		! real equilibrium default bond price
													
	REAL_PK_LOOP: DO piter=1,pitmax
		
		real_p_old 		= real_p
		real_pdef_old	= real_pdef
		
        DO i_rgm=1,rgm                                  ! regime
		    DO i_ss=1,ss                                ! loop over income states s = (y,pi)
			    DO i_b=1,ngpb  
			    ! loop over incoming assets B
				    ! real equilibrium bond price (no default today)
				    real_psavnd(i_rgm,i_b,i_ss)     = INTERPOLATE(              &
                                                        bprime,                 &
                                                        real_p(i_rgm,i_ss,:),   &
                                                        ngpbprime,              &
                                                        savnd(i_rgm,i_b,i_ss))
                    
				    ! real equilibrium bond price
				    real_psav(i_rgm,i_b,i_ss) 		= INTERPOLATE(              &
                                                        bprime,                 &
                                                        real_p(i_rgm,i_ss,:),   &
                                                        ngpbprime,              &
                                                        sav(i_rgm,i_b,i_ss))
                    
				    ! real equilibrium bond price (no default tomorrow)
				    real_psav_nodef(i_rgm,i_b,i_ss)	= INTERPOLATE(                  &
                                                        bprime,                     &
                                                        real_p_nodef(i_rgm,i_ss,:), &
                                                        ngpbprime,                  &
                                                        sav(i_rgm,i_b,i_ss))
                    
				    ! real equilibrium bond price (default tomorrow)
				    real_psav_def(i_rgm,i_b,i_ss)   = INTERPOLATE(                  &
                                                        bprime,                     &
                                                        real_p_def(i_rgm,i_ss,:),   &
                                                        ngpbprime,                  &
                                                        sav(i_rgm,i_b,i_ss))
			    END DO
            END DO
        END DO
		
		! First, compute the real price of a bond

		!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(real_evp_nodef,real_evp_def,bpinfl,defpr,real_price,real_pricedef)
		!$OMP DO COLLAPSE(3)
        DO i_rgm=1,rgm                                      ! regime
            DO i_ss=1,ss                                    ! loop over income states s = (y,pi)
			    DO i_bp=1,ngpbprime                         ! loop candidate outgoing assets B'
			
				    real_evp_nodef	= zerodp                    ! initialize no default part of q
				    real_evp_def	= zerodp                    ! initialize default part of q

				    
                    DO i_rgmp=1,rgm  
                        DO i_ssp=1,ss                               ! loop over future states s'
					        bpinfl  = bprime_inf(i_ssp,i_bp)        ! inflated debt 
					        defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
							
					        ! Equilibrium price next period under NO DEFAULT
					        real_price	= INTERPOLATE(b, real_psavnd(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
						
					        ! Bond price next period in DEFAULT
					        real_pricedef	= INTERPOLATE(b, real_pdef(i_rgmp,:,i_ssp),ngpb,bpinfl ) 
														
					        ! update borrowing cost (no default next period)
					        real_evp_nodef	= real_evp_nodef +                                      &
                                ptran(i_ss,i_ssp)                                                   &
                                *pr_rgm(i_rgm,i_rgmp)                                               &
                                *sdf(i_ss,i_ssp)	                                                &
						        *(onedp - defpr)*((rc+delta)+(onedp-delta)*real_price)   			&     
						        *(onedp+ex_inflinv(i_ss))/(onedp + pigrid(i_ssp))                                       
							
					        ! update borrowing cost (default next period)
					        real_evp_def	= real_evp_def +                                        &
                                ptran(i_ss,i_ssp)                                                   &
                                *pr_rgm(i_rgm,i_rgmp)                                               &
                                *sdf(i_ss,i_ssp)	                                                &
						        *defpr*real_pricedef                                     			&
						        *(onedp+ex_inflinv(i_ss))/(onedp + pigrid(i_ssp)) 
                        END DO 
                    END DO
				
				    real_p(i_rgm,i_ss,i_bp)	        = real_evp_nodef + real_evp_def	! implied real borrowing cost
				    real_p_nodef(i_rgm,i_ss,i_bp)	= real_evp_nodef			    ! implied real borrowing cost (future no default states)
				    real_p_def(i_rgm,i_ss,i_bp)	    = real_evp_def				    ! implied real borrowing cost (future default states)
			
			    END DO
            END DO
        END DO
		!$OMP END DO
		!$OMP END PARALLEL
		
		! Second, compute the real price of a bond in default
		!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(assets,evp,bpinfl,defpr,real_price,real_pricedef)
		!$OMP DO COLLAPSE(3)
        DO i_rgm=1,rgm                                      ! regime
		    DO i_b=1,ngpb                                   ! loop over incoming assets B
			    DO i_ss=1,ss                                ! loop over income states s = (y,pi)
		
				    assets  = b(i_b)
				    evp	    = zerodp                                ! initialize cost of borrowing q(s,B')                      

				    DO i_rgmp=1,rgm 
                        DO i_ssp=1,ss                                   ! loop over future states s'
					        bpinfl  = lambda*assets/(1+pigrid(i_ssp))   ! inflated & diluted debt
					        defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
					
					        ! Equilibrium price next period under NO default
					        real_price   = INTERPOLATE(b, real_psavnd(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
					
					        ! Bond price next period in DEFAULT
					        real_pricedef = INTERPOLATE(b, real_pdef(i_rgmp,:,i_ssp),ngpb,bpinfl ) 
																				
					        ! update borrowing cost
					        evp = evp +                                                         &
                                ptran(i_ss,i_ssp)                                               &
                                *pr_rgm(i_rgm,i_rgmp)                                           &
                                *sdf(i_ss,i_ssp)                                                &
						        *lambda*(onedp+ex_inflinv(i_ss))/(onedp + pigrid(i_ssp))		&
						        *(                                                          	&
							        theta_mat(i_rgm,i_rgmp)*(onedp-defpr)                       &
                                            *((rc+delta) + (onedp-delta)*real_price)	        &
							        + (onedp-theta_mat(i_rgm,i_rgmp)+theta_mat(i_rgm,i_rgmp)*defpr)*real_pricedef   &
						        )
                        END DO 
                    END DO
				    real_pdef(i_rgm,i_b,i_ss) = evp    ! implied borrowing cost
			    END DO
            END DO
        END DO
		!$OMP END DO
		!$OMP END PARALLEL			
		
		gappavg = (SUM (ABS (real_p_old-real_p))) / DBLE(rgm*ss*ngpbprime)
		gappdefavg = (SUM (ABS (real_pdef_old-real_pdef))) / DBLE(rgm*ngpb*ss)

		IF ((gappavg<critpavg).AND.(gappdefavg<critpavg))  THEN
				PRINT *, "REAL PRICE UPDATING DONE"
			EXIT REAL_PK_LOOP
		ELSE IF ((gappavg>infty).OR.(gappdefavg>infty))  THEN
				PRINT *, "REAL PRICE UPDATING BLEW UP :("
			EXIT REAL_PK_LOOP
		ELSE IF ((ISNAN(gappavg)).OR.(ISNAN(gappdefavg)))  THEN
				PRINT *, "REAL PRICE UPDATING NaN :("
			EXIT REAL_PK_LOOP 
		END IF
	END DO REAL_PK_LOOP	

	PRINT *, ""
	PRINT *, "Number of price iterations:            ", piter
	PRINT *, "avg abs errors (real price schedule)   ", gappavg
	PRINT *, "avg abs errors (real def price)        ", gappdefavg

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! ALTERNATIVE REAL DISCOUNTED PRICE !
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
	! Initialize real prices - v2
	real_p_v2       = p			! real bond price schedule
	real_pdef_v2    = pdef		! real equilibrium default bond price
													
	REAL_PK_LOOP_V2: DO piter=1,pitmax
		
		real_p_old_v2       = real_p_v2
		real_pdef_old_v2    = real_pdef_v2
		
        DO i_rgm=1,rgm                                  ! regime
		    DO i_ss=1,ss                                ! loop over income states s = (y,pi)
			    DO i_b=1,ngpb  
			    ! loop over incoming assets B
				    ! real equilibrium bond price (no default today)
				    real_psavnd_v2(i_rgm,i_b,i_ss)      = INTERPOLATE(                      &
                                                                bprime,                     &
                                                                real_p_v2(i_rgm,i_ss,:),    &
                                                                ngpbprime,                  &
                                                                savnd(i_rgm,i_b,i_ss))
				    
                    ! real equilibrium bond price
				    real_psav_v2(i_rgm,i_b,i_ss) 		= INTERPOLATE(                      &
                                                                bprime,                     &
                                                                real_p_v2(i_rgm,i_ss,:),    &
                                                                ngpbprime,                  &
                                                                sav(i_rgm,i_b,i_ss))
                    
				    ! real equilibrium bond price (no default tomorrow)
				    real_psav_nodef_v2(i_rgm,i_b,i_ss)  = INTERPOLATE(                          &
                                                                bprime,                         &
                                                                real_p_nodef_v2(i_rgm,i_ss,:),  &
                                                                ngpbprime,                      &
                                                                sav(i_rgm,i_b,i_ss))
				    
                    ! real equilibrium bond price (default tomorrow)
				    real_psav_def_v2(i_rgm,i_b,i_ss)    = INTERPOLATE(                          &
                                                                bprime,                         &
                                                                real_p_def_v2(i_rgm,i_ss,:),    &
                                                                ngpbprime,                      &
                                                                sav(i_rgm,i_b,i_ss))
			    END DO
            END DO
        END DO
		
		! First, compute the real price of a bond

		!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(real_evp_nodef,real_evp_def,bpinfl,defpr,real_price,real_pricedef)
		!$OMP DO COLLAPSE(3)
        DO i_rgm=1,rgm                                      ! regime
		    DO i_ss=1,ss                                    ! loop over income states s = (y,pi)
			    DO i_bp=1,ngpbprime                         ! loop candidate outgoing assets B'
			
				    real_evp_nodef	= zerodp                    ! initialize no default part of q
				    real_evp_def	= zerodp                    ! initialize default part of q

				    DO i_rgmp=1,rgm 
				        DO i_ssp=1,ss                               ! loop over future states s'
					        bpinfl  = bprime_inf(i_ssp,i_bp)        ! inflated debt 
					        defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
							
					        ! Equilibrium price next period under NO DEFAULT
					        real_price	= INTERPOLATE(b, real_psavnd_v2(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
						
					        ! Bond price next period in DEFAULT
					        real_pricedef	= INTERPOLATE(b, real_pdef_v2(i_rgmp,:,i_ssp),ngpb,bpinfl ) 
														
					        ! update borrowing cost (no default next period)
					        real_evp_nodef	= real_evp_nodef + ptran(i_ss,i_ssp)                    &
                                *pr_rgm(i_rgm,i_rgmp)                                               &
                                *(sdf(i_ss,i_ssp)/ex_sdf(i_ss))	                                    &
						        *(onedp - defpr)*((rc+delta)+(onedp-delta)*real_price)   			&     
						        *(onedp+ex_inflinv(i_ss))/(onedp + pigrid(i_ssp))                                       
							
					        ! update borrowing cost (default next period)
					        real_evp_def	= real_evp_def +                                        &
                                                ptran(i_ss,i_ssp)                                   &
                                                *pr_rgm(i_rgm,i_rgmp)                               &
                                                *(sdf(i_ss,i_ssp)/ex_sdf(i_ss))	                    &
						                        *defpr*real_pricedef                                &
						                        *(onedp+ex_inflinv(i_ss))/(onedp + pigrid(i_ssp)) 
                        END DO 
                    END DO
				
				    real_p_v2(i_rgm,i_ss,i_bp)	        = real_evp_nodef + real_evp_def	! implied real borrowing cost
				    real_p_nodef_v2(i_rgm,i_ss,i_bp)	= real_evp_nodef			! implied real borrowing cost (future no default states)
				    real_p_def_v2(i_rgm,i_ss,i_bp)	    = real_evp_def				! implied real borrowing cost (future default states)
			
			    END DO
            END DO
        END DO
		!$OMP END DO
		!$OMP END PARALLEL
		
		! Second, compute the real price of a bond in default
		!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(assets,evp,bpinfl,defpr,real_price,real_pricedef)
		!$OMP DO COLLAPSE(3)
        DO i_rgm=1,rgm        
		    DO i_b=1,ngpb                                   ! loop over incoming assets B
			    DO i_ss=1,ss                                ! loop over income states s = (y,pi)
		
				    assets  = b(i_b)
				    evp	    = zerodp                                ! initialize cost of borrowing q(s,B')                      

				    DO i_rgmp=1,rgm 
				        DO i_ssp=1,ss                                   ! loop over future states s'
					        bpinfl  = lambda*assets/(1+pigrid(i_ssp))   ! inflated & diluted debt
					        defpr   = INTERPOLATE( b, def(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
					
					        ! Equilibrium price next period under NO default
					        real_price   = INTERPOLATE(b, real_psavnd_v2(i_rgmp,:,i_ssp), ngpb, bpinfl ) 
					
					        ! Bond price next period in DEFAULT
					        real_pricedef = INTERPOLATE(b, real_pdef_v2(i_rgmp,:,i_ssp),ngpb,bpinfl ) 
																				
					        ! update borrowing cost
					        evp = evp +                                                         &
                                    ptran(i_ss,i_ssp)                                           &
                                    *pr_rgm(i_rgm,i_rgmp)                                       &
                                    *(sdf(i_ss,i_ssp)/ex_sdf(i_ss))                             &
						            *lambda*(onedp+ex_inflinv(i_ss))/(onedp + pigrid(i_ssp))    &
						            *(                                                          &
							            theta_mat(i_rgm,i_rgmp)*(onedp-defpr)                   &
                                                *((rc+delta) + (onedp-delta)*real_price)        &
							            + (onedp-theta_mat(i_rgm,i_rgmp)+theta_mat(i_rgm,i_rgmp)*defpr)*real_pricedef   &
						            )
                        END DO 
                    END DO
                    
				    real_pdef_v2(i_rgm,i_b,i_ss) = evp    ! implied borrowing cost
			    END DO
            END DO
        END DO
		!$OMP END DO
		!$OMP END PARALLEL			
		
		gappavg = (SUM (ABS (real_p_old_v2-real_p_v2))) / DBLE(rgm*ss*ngpbprime)
		gappdefavg = (SUM (ABS (real_pdef_old_v2-real_pdef_v2))) / DBLE(rgm*ngpb*ss)

		IF ((gappavg<critpavg).AND.(gappdefavg<critpavg))  THEN
				PRINT *, "REAL PRICE UPDATING DONE"
			EXIT REAL_PK_LOOP_V2
		ELSE IF ((gappavg>infty).OR.(gappdefavg>infty))  THEN
				PRINT *, "REAL PRICE UPDATING BLEW UP :("
			EXIT REAL_PK_LOOP_V2
		ELSE IF ((ISNAN(gappavg)).OR.(ISNAN(gappdefavg)))  THEN
				PRINT *, "REAL PRICE UPDATING NaN :("
			EXIT REAL_PK_LOOP_V2 
		END IF
    END DO REAL_PK_LOOP_V2	
    
	PRINT *, ""
	PRINT *, "Number of price iterations v2:          ", piter
	PRINT *, "avg abs errors (real price schedule v2) ", gappavg
	PRINT *, "avg abs errors (real def price v2)      ", gappdefavg    
    
    
    
    !! Compute decomposition of real prices
    !
    ! compute corariances for decomposition of spreads
    ! E(xyz) = E(x)E(y)E(z) + E(x)C(y,z) + E(y)C(x,z) + E(z)C(x,y) + E(x_y_z_)
    ! where v_ = E(v - E(v))    
    !
    ! So
    ! E(xyz) ~ E(x)E(y)E(z) + E(x)C(y,z) + E(y)C(x,z) + E(z)C(x,y)
    ! using 
    ! 
    !   x = (1-def)                                             ~ no default
    !
    !        [1+E_t(infl(t+1))]
    !   y = --------------------                                ~ surprise deflation
    !           [1+infl(t+1)]
    !
    !   z = (delta+r+(1-delta)*q)*[sdf(t,t+1)/E_t(sdf(t,t+1))]  ~ sdf  - under no default today
    ! 
    
    DO i_rgm=1,rgm        
        DO i_ss=1,ss                                    ! loop over states s
            DO i_b=1,ngpb                               ! loop over incoming asset level  B
            
                x_m     = zerodp                        ! mean - no def
                y_m     = zerodp                        ! mean - deflation surprise
                z_m     = zerodp                        ! mean - sdf lender
                yz_m    = zerodp                        ! cov
                xz_m    = zerodp                        ! cov
                xy_m    = zerodp                        ! cov
            
                E_RGM: DO i_rgmp=1,rgm 
                    E_SSP: DO i_ssp=1,ss                    ! expectations over future states s'
                                                
                        bpinfl = sav(i_rgm,i_b,i_ss)/(onedp+pigrid(i_ssp))
                
                        x_      = (onedp - INTERPOLATE(b,def(i_rgmp,:,i_ssp),ngpb,bpinfl))
                        y_      = (onedp + ex_inflinv(i_ss))/(onedp+pigrid(i_ssp))
                        z_      = (sdf(i_ss,i_ssp)/ex_sdf(i_ss))                                &
                                    *(delta + rc +                                              & 
                                        (1-delta)*                                              &
                                        INTERPOLATE(b,real_psavnd(i_rgmp,:,i_ssp),ngpb,bpinfl))   

                        !! default price term NOT included in x, y or z: 
                        !! INTERPOLATE(b,def(:,i_ssp),ngpb,bpinfl)*INTERPOLATE(b,pdef(:,i_ssp),ngpb,bpinfl)
                
                        x_m     = x_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*x_
                        y_m     = y_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*y_
                        z_m     = z_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*z_
                
                        yz_m    = yz_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*y_*z_
                        xz_m    = xz_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*x_*z_
                        xy_m    = xy_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*x_*y_

                    END DO E_SSP
                END DO E_RGM
            
                ! compute covariances
                yz_m    = yz_m - y_m*z_m
                xz_m    = xz_m - x_m*z_m
                xy_m    = xy_m - x_m*y_m
                
                ! real prices and components
                ! real_psav(i_b,i_ss)                  = psav(i_b,i_ss)*(onedp+ex_inflinv(i_ss))
                ! real_psavnd(i_b,i_ss)            = INTERPOLATE(bprime,p_nodef(i_ss,:),   &
                                                                    ! ngpbprime,sav(i_b,i_ss))    &
                                                        ! *(onedp+ex_inflinv(i_ss))
                                                        ! !psav_nodef(i_b,i_ss)*(onedp+ex_inflinv(i_ss))
                ! real_psav_def(i_b,i_ss)              = INTERPOLATE(bprime,p_def(i_ss,:),     &
                                                                    ! ngpbprime,sav(i_b,i_ss))    &
                                                        ! *(onedp+ex_inflinv(i_ss))
                                                        ! !psav_def(i_b,i_ss)*(onedp+ex_inflinv(i_ss))
                                                    
                ! decomposition terms - means
                realp_mean_nodef(i_rgm,i_b,i_ss)        = x_m
                realp_mean_sdfl(i_rgm,i_b,i_ss)         = z_m
                realp_mean_defl(i_rgm,i_b,i_ss)         = y_m
                realp_prod_mean(i_rgm,i_b,i_ss)         = x_m*y_m*z_m
                
                ! decomposition terms - mean times cov
                realp_mean_nodef_cov_sdfl_defl(i_rgm,i_b,i_ss)  = x_m*yz_m
                realp_mean_sdfl_cov_nodef_defl(i_rgm,i_b,i_ss)  = z_m*xy_m
                realp_mean_defl_cov_sdfl_nodef(i_rgm,i_b,i_ss)  = y_m*xz_m
            
            END DO
        END DO    
    END DO

    !! Compute alternative decomposition of real prices
    !
    ! compute covariances for alternative decomposition of spreads
    ! E(xy+xz) = E(x)E(y) + C(x,y) + E(x)E(z) + C(x,z) 
    !
    ! 
    ! using 
    !        [1+E_t(infl(t+1))]
    !   x = --------------------                                ~ surprise deflation
    !           [1+infl(t+1)]
    !
    ! 
    !   y = (1-def)*(delta+r+(1-delta)*q)*sdf(s,s')/E_s(sdf(s,s'))      ~ no default*sdfl   - under no default today
    !
    !   z = def*q_d*sdf(s,s')/E_s(sdf(s,s'))           ~ default*qdef*sdfl - under no default today
    ! 
    
    DO i_rgm=1,rgm     
        DO i_ss=1,ss                                    ! loop over states s
            DO i_b=1,ngpb                               ! loop over incoming asset level  B
            
                x_m     = zerodp                        ! mean - deflation surprise
                y_m     = zerodp                        ! mean - sdl lender in repayment
                z_m     = zerodp                        ! mean - qdef*sdfl during default
                xz_m    = zerodp                        ! cov
                xy_m    = zerodp                        ! cov
            
                E_RGM2: DO i_rgmp=1,rgm 
                    E_SSP2: DO i_ssp=1,ss                    ! expectations over future states s'
                                                
                        bpinfl = sav(i_rgm,i_b,i_ss)/(onedp+pigrid(i_ssp))
                
                        x_      = (onedp + ex_inflinv(i_ss))/(onedp+pigrid(i_ssp))
                
                        y_      = (onedp - INTERPOLATE(b,def(i_rgmp,:,i_ssp),ngpb,bpinfl))          &
                                    *(delta + rc +                                                  & 
                                        (1-delta)*                                                  &
                                            INTERPOLATE(b,real_psavnd(i_rgmp,:,i_ssp),ngpb,bpinfl)) &
                                    *(sdf_nodef(i_ss,i_ssp)/ex_sdf_nodef(i_ss))          
                
                        z_      = INTERPOLATE(b,def(i_rgmp,:,i_ssp),ngpb,bpinfl)                    &
                                    *INTERPOLATE(b, pdef(i_rgmp,:,i_ssp),ngpb,bpinfl )              &
                                    *(sdf_def(i_ss,i_ssp)/ex_sdf_def(i_ss))
                                
                        x_m     = x_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*x_
                        y_m     = y_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*y_
                        z_m     = z_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*z_
                
                        xz_m    = xz_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*x_*z_
                        xy_m    = xy_m + ptran(i_ss,i_ssp)*pr_rgm(i_rgm,i_rgmp)*x_*y_

                    END DO E_SSP2
                END DO E_RGM2
            
                ! compute alternative decomposition -- covariances
                xz_m    = xz_m - x_m*z_m
                xy_m    = xy_m - x_m*y_m
                
                ! alternative decomposition terms - means
                realp_prod_mean_nodef_alt(i_rgm,i_b,i_ss)         = x_m*y_m
                realp_prod_mean_indef_alt(i_rgm,i_b,i_ss)         = x_m*z_m
                
                ! alternative decomposition terms - mean times cov
                realp_cov_defl_sdfl_nodef_alt(i_rgm,i_b,i_ss)     = xy_m
                realp_cov_defl_sdfl_indef_alt(i_rgm,i_b,i_ss)     = xz_m
            
            END DO
        END DO       
    END DO
    
    

	IF (simonly == 1) THEN
		! default values when in simulation only mode
		compconv%conv_vf_val    = 0
		compconv%chi_val        = chi
		compconv%adjcost_val    = zerodp
		compconv%ngpb_val       = ngpb
		compconv%ngpbprime_val  = ngpbprime
		compconv%ss_val         = ss
		compconv%i_b_zero_val   = i_b_zero   
	END IF
	
	!PRINT *, " END - Post-iteration processing"
    
END SUBROUTINE POST_PROCESS

SUBROUTINE SAVE_SIMPLE
    ! This routine saves to output files the following
    ! (a) policy and value functionsalong with computation grids
    ! (b) sumary simulation stats across all spells
    ! (c) sumary simulation stats for each spell

    INTEGER                 :: ios          ! io status for system call
    CHARACTER (LEN=7)       :: cfg_str      ! configuration number
    CHARACTER (LEN=80)      :: outdir       ! output folder
    INTEGER                 ::  w
    
       
    ! generate output folder name
    WRITE (UNIT=cfg_str, FMT='(I<sfx>.<sfx>)') config
    outdir = TRIM(folder_out) // TRIM(cfgpfx) // '_' // TRIM(cfg_str) // '/'      
    
    ! Setup folder for ouptut files
    ios     = SYSTEM( 'mkdir -p ' // TRIM(outdir) )

    PRINT *, "Writing Results ..."
    
    !!!!!!!!!!!!!!!!!!!!!!!!
    !!    WRITE OUTPUT    !!
    !!!!!!!!!!!!!!!!!!!!!!!!  
    
    !! Save policy and value functions, prices 
    OPEN (unit=9, file= TRIM(outdir) // "v.out", position="rewind")
    WRITE (9, *) v
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "v_l.out", position="rewind")
    WRITE (9, *) v_l
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vv_l.out", position="rewind")
    WRITE (9, *) vv_l
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vnd.out", position="rewind")
    WRITE (9, *) vnd
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vnd_l.out", position="rewind")
    WRITE (9, *) vnd_l
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vvnd_l.out", position="rewind")
    WRITE (9, *) vvnd_l
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "b.out", position="rewind")
    WRITE (9, *) b
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "p.out", position="rewind")
    WRITE (9, *) p
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vdef.out", position="rewind")
    WRITE (9, *) vdef
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vdef_l.out", position="rewind")
    WRITE (9, *) vdef_l
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "vvdef_l.out", position="rewind")
    WRITE (9, *) vvdef_l
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "sav.out", position="rewind")
    WRITE (9, *) sav
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "savo.out", position="rewind")
    WRITE (9, *) savo
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "diffsav.out", position="rewind")
    WRITE (9, *) ABS (sav-savo)
    CLOSE (unit=9)     
    OPEN (unit=9, file= TRIM(outdir) // "savnd.out", position="rewind")
    WRITE (9, *) savnd
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "psav.out", position="rewind")
    WRITE (9, *) psav
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "psavo.out", position="rewind")
    WRITE (9, *) psavo
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "diffpsav.out", position="rewind")
    WRITE (9, *) ABS(psav-psavo)
    CLOSE (unit=9)      
    OPEN (unit=9, file= TRIM(outdir) // "psavnd.out", position="rewind")
    WRITE (9, *) psavnd
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "def.out", position="rewind")
    WRITE (9, *) def
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "sdf.out", position="rewind")
    WRITE (9, *) sdf
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "cons.out", position="rewind")
    WRITE (9, *) cons
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "consnd.out", position="rewind")
    WRITE (9, *) consnd
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "yy.out", position="rewind")
    WRITE (9, *) yy
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "dprob.out", position="rewind")
    WRITE (9, *) dprob
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "conaut.out", position="rewind")
    WRITE (9, *) conaut
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "pdef.out", position="rewind")
    WRITE (9, *) pdef
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "p_riskfree.out", position="rewind")
    WRITE (9, *) p_riskfree
    CLOSE (unit=9)           
    OPEN (unit=9, file= TRIM(outdir) // "dprobnd.out", position="rewind")
    WRITE (9, *) dprobnd
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "dprobid.out", position="rewind")
    WRITE (9, *) dprobid
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "realp_prod_mean.out", position="rewind")
    WRITE (9, *) realp_prod_mean
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "realp_mean_nodef.out", position="rewind")
    WRITE (9, *) realp_mean_nodef
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "realp_mean_sdfl.out", position="rewind")
    WRITE (9, *) realp_mean_sdfl
    CLOSE (unit=9)    
    OPEN (unit=9, file= TRIM(outdir) // "realp_mean_defl.out", position="rewind")
    WRITE (9, *) realp_mean_defl
    CLOSE (unit=9) 
    OPEN (unit=9, file= TRIM(outdir) // "realp_mean_nodef_cov_sdfl_defl.out", position="rewind")
    WRITE (9, *) realp_mean_nodef_cov_sdfl_defl
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "realp_mean_sdfl_cov_nodef_defl.out", position="rewind")
    WRITE (9, *) realp_mean_sdfl_cov_nodef_defl
    CLOSE (unit=9)    
    OPEN (unit=9, file= TRIM(outdir) // "realp_mean_defl_cov_sdfl_nodef.out", position="rewind")
    WRITE (9, *) realp_mean_defl_cov_sdfl_nodef
    CLOSE (unit=9)        
    OPEN (unit=9, file= TRIM(outdir) // "realp_prod_mean_nodef_alt.out", position="rewind")
    WRITE (9, *) realp_prod_mean_nodef_alt
    CLOSE (unit=9)        
    OPEN (unit=9, file= TRIM(outdir) // "realp_prod_mean_indef_alt.out", position="rewind")
    WRITE (9, *) realp_prod_mean_indef_alt
    CLOSE (unit=9)        
    OPEN (unit=9, file= TRIM(outdir) // "realp_cov_defl_sdfl_nodef_alt.out", position="rewind")
    WRITE (9, *) realp_cov_defl_sdfl_nodef_alt
    CLOSE (unit=9)        
    OPEN (unit=9, file= TRIM(outdir) // "realp_cov_defl_sdfl_indef_alt.out", position="rewind")
    WRITE (9, *) realp_cov_defl_sdfl_indef_alt
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "real_psav.out", position="rewind")
    WRITE (9, *) real_psav
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "erg_sav_shock.out", position="rewind")
    WRITE (9, *) erg_sav_shock
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "erg_sav_shock_nd.out", position="rewind")
    WRITE (9, *) erg_sav_shock_nd
    CLOSE (unit=9)   
    OPEN (unit=9, file= TRIM(outdir) // "erg_sav.out", position="rewind")
    WRITE (9, *) erg_sav
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "erg_sav_nd.out", position="rewind")
    WRITE (9, *) erg_sav_nd
    CLOSE (unit=9)     
    
	! Save grids
	OPEN (unit=9, file= TRIM(outdir) // "b.out", position="rewind")
    WRITE (9, *) b
    CLOSE (unit=9)    
    OPEN (unit=9, file= TRIM(outdir) // "bprime.out", position="rewind")
    WRITE (9, *) bprime
    CLOSE (unit=9)
	OPEN (UNIT=unit_idx, file= TRIM(outdir) // "bprime_inf.out", position="rewind")
    WRITE (unit_idx, *) bprime_inf
    CLOSE (UNIT=unit_idx)    
    OPEN (UNIT=unit_idx, file= TRIM(outdir) // 'ptran.out', POSITION = "REWIND")
    WRITE (unit_idx, *) ptran
    CLOSE (UNIT=unit_idx)     
    OPEN (UNIT=unit_idx, file= TRIM(outdir) // 'pr_rgm.out', POSITION = "REWIND")
    WRITE (unit_idx, *) pr_rgm
    CLOSE (UNIT=unit_idx) 
    OPEN (UNIT=unit_idx, file= TRIM(outdir) // 'theta_mat.out', POSITION = "REWIND")
    WRITE (unit_idx, *) theta_mat
    CLOSE (UNIT=unit_idx) 
    OPEN (UNIT=unit_idx, file= TRIM(outdir) // 'ygrid.out', POSITION = "REWIND")
    WRITE (unit_idx, *) ygrid
    CLOSE (UNIT=unit_idx)    
    OPEN (UNIT=unit_idx, file= TRIM(outdir) // 'pigrid.out', POSITION = "REWIND")
    WRITE (unit_idx, *) pigrid
    CLOSE (UNIT=unit_idx)
    OPEN (unit=9, file= TRIM(outdir) // "chigrid.out", position="rewind")
    WRITE (9, *) chigrid
    CLOSE (unit=9)
    OPEN (unit=9, file= TRIM(outdir) // "chi.out", position="rewind")
    WRITE (9, *) chi
    CLOSE (unit=9)
    
    !!!!!!!!!!!!!!!!!!!!
    ! Model parameters !
    !!!!!!!!!!!!!!!!!!!!    
    
100 FORMAT ('#' ,                   &
            '     ngpb',    T10,    &
            '    ngpbp',    T20,    &
            '       ss',    T30,    &
            ' i_b_zero',    T40,    &
            '     bita',    T50,    &
            '   bita_l',    T60,    &
            '    sigma',    T70,    &
            '  sigma_l',    T80,    &
            '    ies_l',    T90,    &
            '    theta',    T100,   &
            '   lambda',    T110,   &
            '    delta',    T120,   &
            '       d0',    T130,   &
            '     d1_h',    T140,   &
            '     d1_l',    T150,   &
            '  cov_piy',    T160,   &
            ' chi@conv',    T170,   &
            '  adjcost',    T180,   &
            ' conv_ind',    T190    &
                )
   
102 FORMAT (  1X,                   &
            I7,             T10,    &
            I9,             T20,    &
            I9,             T30,    &
            I9,             T40,    &
            F9.5,           T50,    &
            F9.5,           T60,    &
            F9.5,           T70,    &
            F9.5,           T80,    &
            F9.5,           T90,    &
            F9.5,           T100,   &
            F9.5,           T110,   &
            F9.5,           T120,   &
            F9.5,           T130,   &
            F9.5,           T140,   &
            F9.5,           T150,   &
            F11.7,          T160,   &
            F9.5,           T170,   &
            F9.5,           T180,   &
            I1,             T190    &
                )  
    
    OPEN (unit=11, file= TRIM(outdir) // "params_matlab.out", position="rewind")
    WRITE (11,100)
    WRITE (11,102)      compconv%ngpb_val,          &
                        compconv%ngpbprime_val,     &
                        compconv%ss_val,            &
                        compconv%i_b_zero_val,      &
                        params_vec%bita_val,        &
                        params_vec%bita_l_val,      &
	                    params_vec%sigma_val,       &
	                    params_vec%sigma_l_val,     &
	                    params_vec%ies_l_val,       &
                        params_vec%theta_val,       &
                        params_vec%lambda_val,      &
                        params_vec%delta_val,       &
	                    params_vec%d0_val,          &
	                    params_vec%d1_h_val,        &
	                    params_vec%d1_l_val,        &
                        params_vec%cov_piy_val,     &
                        compconv%chi_val,           &
                        compconv%adjcost_val,       &
                        compconv%conv_vf_val
    
    CLOSE (unit=11)
    
    !!!!!!!!!!!!!!!!!!!!!!
    ! Simulation results !
    !!!!!!!!!!!!!!!!!!!!!!

200 FORMAT ('#',                    &
            ' debt_in',     T12,    &
            'debt_out',     T24,    &
            'prdef   ',     T36,    &
            'spread1 ',     T48,    &
            'rfree   ',     T60,    &
            'spread_ ',     T72,    &
            'sdvsprd1',     T84,    &
            'sdvsprd2',     T96,    &
            'sdvsprd_',     T108,   &
            'realp   ',     T120,   &
            'sdvrealp',     T132,   &
            'm_realp ',     T144,   &
            'prodmean',     T156,   &
            'm_nodef ',     T168,   &
            'm_sdfl  ',     T180,   &
            'm_defl  ',     T192,   &  
            'mnod_csd',     T204,   &
            'msdf_cnd',     T216,   &
            'mdeflcsn',     T228,   &    
            'chi@conv',     T240,   &
            ' adjcost',     T252,   &
            'rfree_g',      T264,   &
            'rfree_b',      T276,   &
            'spread_g',     T288,   &
            'spread_b',     T300,   &
            'sprdv2_g',     T312,   &
            'sprdv2_b',     T324,   &
            'dprob_g ',     T336,   & 
            'dprob_b ',     T348,   &
            '    conv',     T360,   &
            'prd_nodf',     T372,   &
            'prd_indf',     T384,   &
            'cov_nodf',     T396,   &
            'cov_indf',     T408    & 
                )
    
202 FORMAT (  1X,                   &
            F9.3,           T12,    &
            F9.3,           T24,    &
            F9.3,           T36,    &
            F9.3,           T48,    &
            F9.3,           T60,    &
            F9.3,           T72,    &
            F9.3,           T84,    &
            F9.3,           T96,    &
            F9.3,           T108,   &
            F9.3,           T120,   &
            F9.5,           T132,   &
            F9.5,           T144,   &
            F9.5,           T156,   &
            F9.5,           T168,   &
            F9.5,           T180,   &
            F9.5,           T192,   &
            F9.5,           T204,   &
            F9.5,           T216,   &
            F9.5,           T228,   &
            F9.3,           T240,   &
            F9.3,           T252,   &
            F9.3,           T264,   &
            F9.3,           T276,   &
            F9.3,           T288,   &
            F9.3,           T300,   &
            F9.3,           T312,   &
            F9.3,           T324,   &
            F9.3,           T336,   &
            F9.3,           T348,   &
            I1,             T360,   & 
            F9.5,           T372,   &
            F9.5,           T384,   &
            F9.5,           T396,   &
            F9.5,           T408    &
                )  
    
    OPEN (unit=21, file= TRIM(outdir) // "simuls_simple.out", position="rewind")
    WRITE (21,200)    
    WRITE (21,202)      results_simple%DEBT_IN_SIMUL_VAL,       &
                        results_simple%DEBT_OUT_SIMUL_VAL,      &
                        results_simple%DPROB_SIMUL_VAL,         &
                        results_simple%SPREAD1_SIMUL_VAL,       &
                        results_simple%RFREE_SIMUL_VAL,         &
                        results_simple%SPREAD__SIMUL_VAL,       &
                        results_simple%SDVSPREAD1_SIMUL_VAL,    &
                        results_simple%SDVSPREAD2_SIMUL_VAL,    & 
                        results_simple%SDVSPREAD__SIMUL_VAL,    &     
                        results_simple%REALPRICE_SIMUL_VAL,     &
                        results_simple%SDVREALP_SIMUL_VAL,      &
                        results_simple%MEAN_REAL_PSAV_SIMUL_VAL,	&
                        results_simple%PROD_MEAN_SIMUL_VAL,     &
                        results_simple%MEAN_NODEF_SIMUL_VAL,    &
                        results_simple%MEAN_SDFL_SIMUL_VAL,     &
                        results_simple%MEAN_DEFL_SIMUL_VAL,     &
                        results_simple%MEAN_NODEF_CSDFLDEFL_SIMUL_VAL,	&
                        results_simple%MEAN_SDFL_CNODEFDEFL_SIMUL_VAL,  &
                        results_simple%MEAN_DEFL_CSDFLNODEF_SIMUL_VAL,  & 
                        compconv%chi_val,                       &
                        compconv%adjcost_val,                   &
                        results_simple%RFREE_GOOD_SIMUL_VAL,	&
                        results_simple%RFREE_BAD_SIMUL_VAL,	    &
                        results_simple%SPREAD_GOOD_SIMUL_VAL,	&
                        results_simple%SPREAD_BAD_SIMUL_VAL,	&
                        results_simple%SPRDV2_GOOD_SIMUL_VAL,	&
                        results_simple%SPRDV2_BAD_SIMUL_VAL,	&
                        results_simple%DPROB_GOOD_SIMUL_VAL,	&
                        results_simple%DPROB_BAD_SIMUL_VAL,     &
                        compconv%conv_vf_val,                   &
                        results_simple%PROD_MEAN_NODEF_SIMUL_VAL,       &
                        results_simple%PROD_MEAN_INDEF_SIMUL_VAL,       &
                        results_simple%COV_DEFL_SDFL_NODEF_SIMUL_VAL,   &
                        results_simple%COV_DEFL_SDFL_INDEF_SIMUL_VAL
    CLOSE (unit=21)
        
    !!!!!!!!!!!!!!!!!!!!!!
    ! Simulation results !
    ! Good vs Bad times  !
    !!!!!!!!!!!!!!!!!!!!!!

300 FORMAT ('# ',                   &
            'spread_g  ',   T012,   &
            'dprob_g   ',   T024,   &
            'realp_g   ',   T036,   &
            'm_realp_g ',   T048,   &
            'prodmean_g',   T060,   &
            'm_nodef_g ',   T072,   &
            'm_sdfl_g  ',   T084,   &
            'm_defl_g  ',   T096,   &  
            'mnod_csd_g',   T108,   &
            'msdf_cnd_g',   T120,   &
            'mdeflcsn_g',   T132,   & 
            'prd_nodf_g',   T144,   &
            'prd_indf_g',   T156,   &
            'cov_nodf_g',   T168,   &
            'cov_indf_g',   T180,   &
            'spread_b  ',   T192,   & 
            'dprob_b   ',   T204,   &
            'realp_b   ',   T216,   &
            'm_realp_b ',   T228,   &
            'prodmean_b',   T240,   &
            'm_nodef_b ',   T252,   &
            'm_sdfl_b  ',   T264,   &
            'm_defl_b  ',   T276,   &  
            'mnod_csd_b',   T288,   &
            'msdf_cnd_b',   T300,   &
            'mdeflcsn_b',   T312,   & 
            'prd_nodf_b',   T324,   &
            'prd_indf_b',   T336,   &
            'cov_nodf_b',   T348,   &
            'cov_indf_b',   T360    &     
            'chi@conv  ',   T372,   &
            ' adjcost  ',   T384,   &
            '    conv  ',   T396    &
                )
    
302 FORMAT (  1X,                   &
            F9.3,           T12,    &
            F9.3,           T24,    &
            F9.5,           T36,    &
            F9.5,           T48,    &
            F9.5,           T60,    &
            F9.5,           T72,    &
            F9.5,           T84,    &
            F9.5,           T96,    &
            F9.5,           T108,   &
            F9.5,           T120,   &
            F9.5,           T132,   &
            F9.5,           T144,   &
            F9.5,           T156,   &
            F9.5,           T168,   &
            F9.5,           T180,   &
            F9.3,           T192,   &
            F9.3,           T204,   &
            F9.5,           T216,   &
            F9.5,           T228,   &
            F9.5,           T240,   &
            F9.5,           T252,   &
            F9.5,           T264,   &
            F9.5,           T276,   &
            F9.5,           T288    & 
            F9.5,           T300,   &
            F9.5,           T312,   &
            F9.5,           T324,   &
            F9.5,           T336,   &
            F9.5,           T348,   & 
            F9.5,           T360,   &
            F9.5,           T372,   &
            F9.5,           T384,   &
            I1,             T396    &
                )  
    
    OPEN (unit=31, file= TRIM(outdir) // "simuls_gd_bd.out", position="rewind")
    WRITE (31,300)    
    WRITE (31,302)      results_simple%SPREAD_GOOD_SIMUL_VAL,	    &
                        results_simple%DPROB_GOOD_SIMUL_VAL,	    &    
                        results_simple%REALPRICE_GD_SIMUL_VAL,      &
                        results_simple%MEAN_REAL_PSAV_GD_SIMUL_VAL,	&
                        results_simple%PROD_MEAN_GD_SIMUL_VAL,      &    
                        results_simple%MEAN_NODEF_GD_SIMUL_VAL,     &
                        results_simple%MEAN_SDFL_GD_SIMUL_VAL,      &
                        results_simple%MEAN_DEFL_GD_SIMUL_VAL,      &
                        results_simple%MEAN_NODEF_CSDFLDEFL_GD_SIMUL_VAL,	&
                        results_simple%MEAN_SDFL_CNODEFDEFL_GD_SIMUL_VAL,   &
                        results_simple%MEAN_DEFL_CSDFLNODEF_GD_SIMUL_VAL,   & 
                        results_simple%PROD_MEAN_NODEF_GD_SIMUL_VAL,        &
                        results_simple%PROD_MEAN_INDEF_GD_SIMUL_VAL,        &
                        results_simple%COV_DEFL_SDFL_NODEF_GD_SIMUL_VAL,    &
                        results_simple%COV_DEFL_SDFL_INDEF_GD_SIMUL_VAL,    &
                        results_simple%SPREAD_BAD_SIMUL_VAL,	    &
                        results_simple%DPROB_BAD_SIMUL_VAL,         &
                        results_simple%REALPRICE_BD_SIMUL_VAL,      &
                        results_simple%MEAN_REAL_PSAV_BD_SIMUL_VAL,  &
                        results_simple%PROD_MEAN_BD_SIMUL_VAL,      &
                        results_simple%MEAN_NODEF_BD_SIMUL_VAL,     &
                        results_simple%MEAN_SDFL_BD_SIMUL_VAL,      &
                        results_simple%MEAN_DEFL_BD_SIMUL_VAL,      &
                        results_simple%MEAN_NODEF_CSDFLDEFL_BD_SIMUL_VAL,   &
                        results_simple%MEAN_SDFL_CNODEFDEFL_BD_SIMUL_VAL,   &
                        results_simple%MEAN_DEFL_CSDFLNODEF_BD_SIMUL_VAL,   & 
                        results_simple%PROD_MEAN_NODEF_BD_SIMUL_VAL,        &
                        results_simple%PROD_MEAN_INDEF_BD_SIMUL_VAL,        &
                        results_simple%COV_DEFL_SDFL_NODEF_BD_SIMUL_VAL,    &
                        results_simple%COV_DEFL_SDFL_INDEF_BD_SIMUL_VAL,    &
                        compconv%chi_val,                           &
                        compconv%adjcost_val,                       &
                        compconv%conv_vf_val                    
    CLOSE (unit=31)

    !!!!!!!!!!!!!!!!!!!!!!
    ! Simulation results !
    ! regimes            !
    !!!!!!!!!!!!!!!!!!!!!!

400 FORMAT ('#',                    &
            ' b_in   ',     T12,    &
            ' b_in_h ',     T24,    &
            ' b_in_l ',     T36,    &
            ' b_out  ',     T48,    &
            ' b_out_h',     T60,    &
            ' b_out_l',     T72,    &
            'prdef   ',     T84,    &
            'prdef_h ',     T96,    &
            'prdef_l ',     T108,   &
            'sprd    ',     T120,   &
            'sprd1_h ',     T132,   &
            'sprd1_l ',     T144,   &
            'rfree   ',     T156,   &
            'rfree_h ',     T168,   &
            'rfree_l ',     T180,   &
            'realp   ',     T192,   &
            'realp_h ',     T204,   &
            'realp_l ',     T216,   &
            'rf_g    ',     T228,   &
            'rf_g_h  ',     T240,   &
            'rf_g_l  ',     T252,   &
            'rf_b    ',     T264,   &
            'rf_b_h  ',     T276,   &
            'rf_b_l  ',     T288,   &
            'spr1_g  ',     T300,   &
            'spr1_g_h',     T312,   &
            'spr1_g_l',     T324,   &
            'spr1_b  ',     T336,   &
            'spr1_b_h',     T348,   &
            'spr1_b_l',     T360,   &
            'dprb_g  ',     T372,   &
            'dprb_g_h',     T384,   &
            'dprb_g_l',     T396,   & 
            'dprb_b  ',     T408,   & 
            'dprb_b_h',     T420,   & 
            'dprb_b_l',     T432,   &
            '    conv',     T444   &
                )
    
402 FORMAT (  1X,                   &
            F9.3,           T12,    &
            F9.3,           T24,    &
            F9.3,           T36,    &
            F9.3,           T48,    &
            F9.3,           T60,    &
            F9.3,           T72,    &
            F9.3,           T84,    &
            F9.3,           T96,    &
            F9.3,           T108,   &
            F9.3,           T120,   &
            F9.3,           T132,   &
            F9.3,           T144,   &
            F9.3,           T156,   &
            F9.3,           T168,   &
            F9.3,           T180,   &
            F9.3,           T192,   &
            F9.3,           T204,   &
            F9.3,           T216,   &
            F9.3,           T228,   &
            F9.3,           T240,   &
            F9.3,           T252,   &
            F9.3,           T264,   &
            F9.3,           T276,   &
            F9.3,           T288,   &
            F9.3,           T300,   &
            F9.3,           T312,   &
            F9.3,           T324,   &
            F9.3,           T336,   &
            F9.3,           T348,   &
            F9.3,           T360,   & 
            F9.3,           T372,   &
            F9.3,           T384,   &
            F9.3,           T396,   &
            F9.3,           T408,   &
            F9.3,           T420,   &
            F9.3,           T432,   &
            I1  ,           T444    &
                )  
    
    OPEN (unit=41, file= TRIM(outdir) // "simuls_regime.out", position="rewind")
    WRITE (41,400)    
    WRITE (41,402)      results_simple%DEBT_IN_SIMUL_VAL,       &
                        results_simple%DEBT_IN_H_SIMUL_VAL,     &
                        results_simple%DEBT_IN_L_SIMUL_VAL,     &
                        results_simple%DEBT_OUT_SIMUL_VAL,      &
                        results_simple%DEBT_OUT_H_SIMUL_VAL,    &
                        results_simple%DEBT_OUT_L_SIMUL_VAL,    &  
                        results_simple%DPROB_SIMUL_VAL,         &
                        results_simple%DPROB_H_SIMUL_VAL,       &
                        results_simple%DPROB_L_SIMUL_VAL,       &
                        results_simple%SPREAD1_SIMUL_VAL,       &
                        results_simple%SPREAD1_H_SIMUL_VAL,     &
                        results_simple%SPREAD1_L_SIMUL_VAL,     &
                        results_simple%RFREE_SIMUL_VAL,         &
                        results_simple%RFREE_H_SIMUL_VAL,       &
                        results_simple%RFREE_L_SIMUL_VAL,       &   
                        results_simple%REALPRICE_SIMUL_VAL,     &     
                        results_simple%REALPRICE_H_SIMUL_VAL,   &     
                        results_simple%REALPRICE_L_SIMUL_VAL,   &
                        results_simple%RFREE_GOOD_SIMUL_VAL,	&
                        results_simple%RFREE_GOOD_H_SIMUL_VAL,	&
                        results_simple%RFREE_GOOD_L_SIMUL_VAL,	&
                        results_simple%RFREE_BAD_SIMUL_VAL,	    &
                        results_simple%RFREE_BAD_H_SIMUL_VAL,   &
                        results_simple%RFREE_BAD_L_SIMUL_VAL,   &
                        results_simple%SPREAD_GOOD_SIMUL_VAL,	&
                        results_simple%SPREAD_GOOD_H_SIMUL_VAL,	&
                        results_simple%SPREAD_GOOD_L_SIMUL_VAL,	&
                        results_simple%SPREAD_BAD_SIMUL_VAL,	&
                        results_simple%SPREAD_BAD_H_SIMUL_VAL,	&
                        results_simple%SPREAD_BAD_L_SIMUL_VAL,	&
                        results_simple%DPROB_GOOD_SIMUL_VAL,	&
                        results_simple%DPROB_GOOD_H_SIMUL_VAL,	&
                        results_simple%DPROB_GOOD_L_SIMUL_VAL,	&
                        results_simple%DPROB_BAD_SIMUL_VAL,     &
                        results_simple%DPROB_BAD_H_SIMUL_VAL,   &
                        results_simple%DPROB_BAD_L_SIMUL_VAL,   &
                        compconv%conv_vf_val
    CLOSE (unit=41)


    !!!!!!!!!!!!!!!!!!!!!!
    ! Simulation results !
    ! value fcns & rgms  !
    !!!!!!!!!!!!!!!!!!!!!!

500 FORMAT ('#',                    &
            ' vlen  ',      T12,    &
            ' vlen_h ',     T24,    &
            ' vlen_l ',     T36,    &
            ' vlen_g ',     T48,    &
            'vlen_g_h',     T60,    &
            'vlen_g_l',     T72,    &
            ' vlen_b ',     T84,    &
            'vlen_b_h',     T96,    &
            'vlen_b_l',     T108,   &
            ' wlen  ',      T120,   &
            ' wlen_h ',     T132,   &
            ' wlen_l ',     T144,   &
            ' wlen_g ',     T156,   &
            'wlen_g_h',     T168,   &
            'wlen_g_l',     T180,   &
            ' wlen_b ',     T192,   &
            'wlen_b_h',     T204,   &
            'wlen_b_l',     T216,   &
            ' vbor  ',      T228,   &
            ' vbor_h ',     T240,   &
            ' vbor_l ',     T252,   &
            ' vbor_g ',     T264,   &
            'vbor_g_h',     T276,   &
            'vbor_g_l',     T288,   &
            ' vbor_b ',     T300,   &
            'vbor_b_h',     T312,   &
            'vbor_b_l',     T324,   &
            'vbor_bb ',     T336,   &
            'vbor_bbh',     T348,   &
            'vbor_bbl',     T360,   &
            'wlen_bb ',     T372,   &
            'wlen_bbh',     T384,   &
            'wlen_bbl',     T396    &
                )
    
502 FORMAT (  1X,                   &
            F9.3,           T12,    &
            F9.3,           T24,    &
            F9.3,           T36,    &
            F9.3,           T48,    &
            F9.3,           T60,    &
            F9.3,           T72,    &
            F9.3,           T84,    &
            F9.3,           T96,    &
            F9.3,           T108,   &
            F9.3,           T120,   &
            F9.3,           T132,   &
            F9.3,           T144,   &
            F9.3,           T156,   &
            F9.3,           T168,   &
            F9.3,           T180,   &
            F9.3,           T192,   &
            F9.3,           T204,   &
            F9.3,           T216,   &
            F9.3,           T228,   &
            F9.3,           T240,   &
            F9.3,           T252,   &
            F9.3,           T264,   &
            F9.3,           T276,   &
            F9.3,           T288,   &
            F9.3,           T300,   &
            F9.3,           T312,   &
            F9.3,           T324,   &
            F9.3,           T336,   &
            F9.3,           T348,   &
            F9.3,           T360,   &
            F9.3,           T372,   &
            F9.3,           T384,   &
            F9.3,           T396    &
                )  
    
    OPEN (unit=51, file= TRIM(outdir) // "simuls_vfcns.out", position="rewind")
    WRITE (51,500)    
    WRITE (51,502)          results_simple%VLEND_SIMUL_VAL, 				&
                            results_simple%VLEND_H_SIMUL_VAL, 				&
                            results_simple%VLEND_L_SIMUL_VAL, 				&
                            results_simple%VLEND_GOOD_SIMUL_VAL, 			&
                            results_simple%VLEND_GOOD_H_SIMUL_VAL, 			&
                            results_simple%VLEND_GOOD_L_SIMUL_VAL, 			&
                            results_simple%VLEND_BAD_SIMUL_VAL, 			&
                            results_simple%VLEND_BAD_H_SIMUL_VAL, 			&
                            results_simple%VLEND_BAD_L_SIMUL_VAL, 			&
                            results_simple%VVLEND_SIMUL_VAL, 				&
                            results_simple%VVLEND_H_SIMUL_VAL, 				&
                            results_simple%VVLEND_L_SIMUL_VAL, 				&
                            results_simple%VVLEND_GOOD_SIMUL_VAL, 			&
                            results_simple%VVLEND_GOOD_H_SIMUL_VAL, 		&
                            results_simple%VVLEND_GOOD_L_SIMUL_VAL, 		&
                            results_simple%VVLEND_BAD_SIMUL_VAL, 			&
                            results_simple%VVLEND_BAD_H_SIMUL_VAL, 			&
                            results_simple%VVLEND_BAD_L_SIMUL_VAL, 			&
                            results_simple%VBORR_SIMUL_VAL, 				&
                            results_simple%VBORR_H_SIMUL_VAL, 				&
                            results_simple%VBORR_L_SIMUL_VAL, 				&
                            results_simple%VBORR_GOOD_SIMUL_VAL, 			&
                            results_simple%VBORR_GOOD_H_SIMUL_VAL, 			&
                            results_simple%VBORR_GOOD_L_SIMUL_VAL, 			&
                            results_simple%VBORR_BAD_SIMUL_VAL, 			&
                            results_simple%VBORR_BAD_H_SIMUL_VAL, 			&
                            results_simple%VBORR_BAD_L_SIMUL_VAL, 			&
                            results_simple%VBORR_BB_SIMUL_VAL, 			    &
                            results_simple%VBORR_BB_H_SIMUL_VAL, 			&
                            results_simple%VBORR_BB_L_SIMUL_VAL, 			&
                            results_simple%VVLEND_BB_SIMUL_VAL,             &
                            results_simple%VVLEND_BB_H_SIMUL_VAL, 			&
                            results_simple%VVLEND_BB_L_SIMUL_VAL
    
    CLOSE (unit=51)
    
    PRINT *, "Writing Results ... - END"    
    
END SUBROUTINE SAVE_SIMPLE

SUBROUTINE SIMULATE_SIMPLE(showdetails)

    IMPLICIT NONE
    INTEGER, INTENT(IN), OPTIONAL :: showdetails
    
    LOGICAL     ::  showdetails_prm
    INTEGER     ::  i, j, ind_simul
    
    REAL (DP)   ::  dprob_simul, dprob_h_simul, dprob_l_simul
    REAL (DP)   ::  debt_in_simul, debt_in_h_simul, debt_in_l_simul
    REAL (DP)   ::  debt_out_simul, debt_out_h_simul, debt_out_l_simul
    
    REAL (DP)   ::  spread1_avg_simul, spread1_avg_h_simul, spread1_avg_l_simul
    REAL (DP)   ::  spread__avg_simul   !, spread__avg_h_simul, spread__avg_l_simul
    REAL (DP)   ::  spread2_avg_simul   !, spread2_avg_h_simul, spread2_avg_l_simul
    
    REAL (DP)   ::  spreadg_avg_simul, spreadg_avg_h_simul, spreadg_avg_l_simul
    REAL (DP)   ::  spreadb_avg_simul, spreadb_avg_h_simul, spreadb_avg_l_simul
    
    REAL (DP)   ::  sprdv2g_avg_simul   !, sprdv2g_avg_h_simul, sprdv2g_avg_l_simul
    REAL (DP)   ::  sprdv2b_avg_simul   !, sprdv2b_avg_h_simul, sprdv2b_avg_l_simul
    
    REAL (DP)   ::  rfree_avg_simul, rfree_avg_h_simul, rfree_avg_l_simul
    REAL (DP)   ::  rfreeg_avg_simul, rfreeg_avg_h_simul, rfreeg_avg_l_simul
    REAL (DP)   ::  rfreeb_avg_simul, rfreeb_avg_h_simul, rfreeb_avg_l_simul
    
    REAL (DP)   ::  dprobg_avg_simul, dprobg_avg_h_simul, dprobg_avg_l_simul
    REAL (DP)   ::  dprobb_avg_simul, dprobb_avg_h_simul, dprobb_avg_l_simul
    
    REAL (DP)   ::  spread1_sdv_simul
    REAL (DP)   ::  spread__sdv_simul
    REAL (DP)   ::  spread2_sdv_simul
    
    REAL (DP)   ::  realprice_avg_simul, realprice_avg_h_simul, realprice_avg_l_simul
    REAL (DP)   ::  realprice_sdv_simul
    
    REAL (DP)   ::  realprice_avg_gd_simul, realprice_avg_gd_h_simul, realprice_avg_gd_l_simul
    REAL (DP)   ::  realprice_sdv_gd_simul
    
    REAL (DP)   ::  realprice_avg_bd_simul, realprice_avg_bd_h_simul, realprice_avg_bd_l_simul
    REAL (DP)   ::  realprice_sdv_bd_simul    
    
    REAL (DP)	::  vlend_avg_simul, vlend_avg_h_simul, vlend_avg_l_simul,            &
				    vlendg_avg_simul, vlendg_avg_h_simul, vlendg_avg_l_simul,         &
				    vlendb_avg_simul, vlendb_avg_h_simul, vlendb_avg_l_simul,         &
				    vlendbb_avg_simul, vlendbb_avg_h_simul, vlendbb_avg_l_simul

    REAL (DP)	::  vvlend_avg_simul, vvlend_avg_h_simul, vvlend_avg_l_simul,         &
				    vvlendg_avg_simul, vvlendg_avg_h_simul, vvlendg_avg_l_simul,      &
				    vvlendb_avg_simul, vvlendb_avg_h_simul, vvlendb_avg_l_simul,      &
				    vvlendbb_avg_simul, vvlendbb_avg_h_simul, vvlendbb_avg_l_simul

    REAL (DP) 	::  vborr_avg_simul, vborr_avg_h_simul, vborr_avg_l_simul,            &
				    vborrg_avg_simul, vborrg_avg_h_simul, vborrg_avg_l_simul,         &
				    vborrb_avg_simul, vborrb_avg_h_simul, vborrb_avg_l_simul,         &
				    vborrbb_avg_simul, vborrbb_avg_h_simul, vborrbb_avg_l_simul  
    
    REAL (DP)   ::  decompose_mean_real_psav_simul, &
                    dcp_mean_real_psav_gd_simul, dcp_mean_real_psav_bd_simul
    REAL (DP)   ::  decompose_mean_real_psav_nodef_simul, &
                    dcp_mean_real_psav_nodef_gd_simul, dcp_mean_real_psav_nodef_bd_simul
    REAL (DP)   ::  decompose_mean_real_psav_def_simul, &
                    dcp_mean_real_psav_def_gd_simul, dcp_mean_real_psav_def_bd_simul
    REAL (DP)   ::  decompose_prod_mean_simul, &
                    dcp_prod_mean_gd_simul, dcp_prod_mean_bd_simul
    REAL (DP)   ::  decompose_mean_nodef_simul, &
                    dcp_mean_nodef_gd_simul, dcp_mean_nodef_bd_simul
    REAL (DP)   ::  decompose_mean_sdfl_simul, &
                    dcp_mean_sdfl_gd_simul, dcp_mean_sdfl_bd_simul
    REAL (DP)   ::  decompose_mean_defl_simul, &
                    dcp_mean_defl_gd_simul, dcp_mean_defl_bd_simul
    REAL (DP)   ::  decompose_mean_nodef_cov_sdfl_defl_simul, &
                    dcp_mean_nodef_cov_sdfl_defl_gd_simul, dcp_mean_nodef_cov_sdfl_defl_bd_simul
    REAL (DP)   ::  decompose_mean_sdfl_cov_nodef_defl_simul, &
                    dcp_mean_sdfl_cov_nodef_defl_gd_simul, dcp_mean_sdfl_cov_nodef_defl_bd_simul
    REAL (DP)   ::  decompose_mean_defl_cov_sdfl_nodef_simul, &
                    dcp_mean_defl_cov_sdfl_nodef_gd_simul, dcp_mean_defl_cov_sdfl_nodef_bd_simul
    
    REAL (DP)   ::  decompose_prod_mean_nodef_alt_simul, &
                    dcp_prod_mean_nodef_alt_gd_simul, dcp_prod_mean_nodef_alt_bd_simul
    REAL (DP)   ::  decompose_prod_mean_indef_alt_simul, &
                    dcp_prod_mean_indef_alt_gd_simul, dcp_prod_mean_indef_alt_bd_simul
    REAL (DP)   ::  decompose_cov_defl_sdfl_nodef_alt_simul, &
                    dcp_cov_defl_sdfl_nodef_alt_gd_simul, dcp_cov_defl_sdfl_nodef_alt_bd_simul
    REAL (DP)   ::  decompose_cov_defl_sdfl_indef_alt_simul, &
                    dcp_cov_defl_sdfl_indef_alt_gd_simul, dcp_cov_defl_sdfl_indef_alt_bd_simul
    
    
    REAL (DP)   ::  tmp
    
    IF (PRESENT(showdetails)) THEN
        showdetails_prm  = (showdetails .EQ. 1)
    ELSE
        showdetails_prm  = .FALSE.
    END IF
    
    PRINT *, " -------------------- SIMULATION ------------------------ "
      
    
    ! set empirical distribution to zero - then sum across simulation
    erg_sav_shock   = zerodp
    erg_sav_shock_nd= zerodp
    
    ! Run simulations and store for each simulation relevant non-crisis stats
    DO ind_simul = 1,N_S_simple
        CALL RUNSIMUL_SIMPLE(ind_simul)
    END DO
    
    ! compute empirical distribution by averaging long-run distribution
    tmp             = SUM(erg_sav_shock)
    erg_sav_shock   = erg_sav_shock/tmp
    
    ! compute empirical distribution by averaging long-run distribution
    tmp             = SUM(erg_sav)
    erg_sav         = erg_sav/tmp

    ! compute empirical no default distribution by averaging long-run distribution
    tmp             = SUM(erg_sav_shock_nd)
    erg_sav_shock_nd= erg_sav_shock_nd/tmp

    ! compute empirical no default distribution by averaging long-run distribution
    tmp             = SUM(erg_sav_nd)
    erg_sav_nd      = erg_sav_nd/tmp    
    
    ! Default probability
    dprob_simul         = AVG(dprob_avg  ,N_S_simple)*100_dp
    dprob_h_simul       = AVG(dprob_avg_h,N_S_simple)*100_dp
    dprob_l_simul       = AVG(dprob_avg_l,N_S_simple)*100_dp 
    
    ! Debt level
    debt_in_simul       = AVG(debt_in_avg  ,N_S_simple)*100_dp
    debt_in_h_simul     = AVG(debt_in_avg_h,N_S_simple)*100_dp
    debt_in_l_simul     = AVG(debt_in_avg_l,N_S_simple)*100_dp
    
    debt_out_simul      = AVG(debt_out_avg  ,N_S_simple)*100_dp
    debt_out_h_simul    = AVG(debt_out_avg_h,N_S_simple)*100_dp
    debt_out_l_simul    = AVG(debt_out_avg_l,N_S_simple)*100_dp
    
    ! Non-crisis spread
    spread1_avg_simul   = AVG(spread1_avg  ,N_S_simple)*100_dp
    spread1_avg_h_simul = AVG(spread1_avg_h,N_S_simple)*100_dp
    spread1_avg_l_simul = AVG(spread1_avg_l,N_S_simple)*100_dp
    
    spread__avg_simul   = AVG(spread__avg,N_S_simple)*100_dp
 
    spread2_avg_simul   = AVG(spread2_avg,N_S_simple)*100_dp
	
    spreadg_avg_simul   = AVG(spreadg_avg  ,N_S_simple)*100_dp
    spreadg_avg_h_simul = AVG(spreadg_avg_h,N_S_simple)*100_dp
    spreadg_avg_l_simul = AVG(spreadg_avg_l,N_S_simple)*100_dp
    
    spreadb_avg_simul   = AVG(spreadb_avg  ,N_S_simple)*100_dp
    spreadb_avg_h_simul = AVG(spreadb_avg_h,N_S_simple)*100_dp
    spreadb_avg_l_simul = AVG(spreadb_avg_l,N_S_simple)*100_dp
    
    sprdv2g_avg_simul   = AVG(sprdv2g_avg,N_S_simple)*100_dp
    
    sprdv2b_avg_simul   = AVG(sprdv2b_avg,N_S_simple)*100_dp
    
    rfree_avg_simul     = AVG(rfree_avg  ,N_S_simple)*100_dp
    rfree_avg_h_simul   = AVG(rfree_avg_h,N_S_simple)*100_dp
    rfree_avg_l_simul   = AVG(rfree_avg_l,N_S_simple)*100_dp
    
    rfreeg_avg_simul    = AVG(rfreeg_avg  ,N_S_simple)*100_dp
    rfreeg_avg_h_simul  = AVG(rfreeg_avg_h,N_S_simple)*100_dp
    rfreeg_avg_l_simul  = AVG(rfreeg_avg_l,N_S_simple)*100_dp
    
    rfreeb_avg_simul    = AVG(rfreeb_avg  ,N_S_simple)*100_dp
    rfreeb_avg_h_simul  = AVG(rfreeb_avg_h,N_S_simple)*100_dp
    rfreeb_avg_l_simul  = AVG(rfreeb_avg_l,N_S_simple)*100_dp
    
    dprobg_avg_simul    = AVG(dprobg_avg  ,N_S_simple)*100_dp
    dprobg_avg_h_simul  = AVG(dprobg_avg_h,N_S_simple)*100_dp
    dprobg_avg_l_simul  = AVG(dprobg_avg_l,N_S_simple)*100_dp
    
    dprobb_avg_simul    = AVG(dprobb_avg  ,N_S_simple)*100_dp
    dprobb_avg_h_simul  = AVG(dprobb_avg_h,N_S_simple)*100_dp
    dprobb_avg_l_simul  = AVG(dprobb_avg_l,N_S_simple)*100_dp
    
    spread1_sdv_simul   = AVG(spread1_sdv,N_S_simple)*100_dp
    
    spread__sdv_simul   = AVG(spread__sdv,N_S_simple)*100_dp
    
    spread2_sdv_simul   = AVG(spread2_sdv,N_S_simple)*100_dp
    
    ! Real price
    realprice_avg_simul     = AVG(realprice_avg  ,N_S_simple)*100_dp
    realprice_avg_h_simul   = AVG(realprice_avg_h,N_S_simple)*100_dp
    realprice_avg_l_simul   = AVG(realprice_avg_l,N_S_simple)*100_dp
    
    realprice_avg_gd_simul  = AVG(realprice_avg_gd  ,N_S_simple)*100_dp
    realprice_avg_gd_h_simul= AVG(realprice_avg_gd_h,N_S_simple)*100_dp
    realprice_avg_gd_l_simul= AVG(realprice_avg_gd_l,N_S_simple)*100_dp
    
    realprice_avg_bd_simul  = AVG(realprice_avg_bd  ,N_S_simple)*100_dp
    realprice_avg_bd_h_simul= AVG(realprice_avg_bd_h,N_S_simple)*100_dp
    realprice_avg_bd_l_simul= AVG(realprice_avg_bd_l,N_S_simple)*100_dp
    
    realprice_sdv_simul     = AVG(realprice_sdv,N_S_simple)*100_dp
       
    realprice_sdv_gd_simul  = AVG(realprice_sdv_gd,N_S_simple)*100_dp
       
    realprice_sdv_bd_simul  = AVG(realprice_sdv_bd,N_S_simple)*100_dp

    ! Decomposition    
    decompose_mean_real_psav_simul               = AVG(decompose_mean_real_psav,N_S_simple)*100_dp
    decompose_mean_real_psav_nodef_simul         = AVG(decompose_mean_real_psav_nodef,N_S_simple)*100_dp
    decompose_mean_real_psav_def_simul           = AVG(decompose_mean_real_psav_def,N_S_simple)*100_dp
    
    decompose_prod_mean_simul                   = AVG(decompose_prod_mean,N_S_simple)*100_dp
    decompose_mean_nodef_simul                  = AVG(decompose_mean_nodef,N_S_simple)*100_dp
    decompose_mean_sdfl_simul                   = AVG(decompose_mean_sdfl,N_S_simple)*100_dp
    decompose_mean_defl_simul                   = AVG(decompose_mean_defl,N_S_simple)*100_dp
    
    decompose_mean_nodef_cov_sdfl_defl_simul    = AVG(decompose_mean_nodef_cov_sdfl_defl,N_S_simple)*100_dp
    decompose_mean_sdfl_cov_nodef_defl_simul    = AVG(decompose_mean_sdfl_cov_nodef_defl,N_S_simple)*100_dp
    decompose_mean_defl_cov_sdfl_nodef_simul    = AVG(decompose_mean_defl_cov_sdfl_nodef,N_S_simple)*100_dp

    ! Decomposition  -- good times  
    dcp_mean_real_psav_gd_simul               = AVG(dcp_mean_real_psav_gd,N_S_simple)*100_dp
    dcp_mean_real_psav_nodef_gd_simul         = AVG(dcp_mean_real_psav_nodef_gd,N_S_simple)*100_dp
    dcp_mean_real_psav_def_gd_simul           = AVG(dcp_mean_real_psav_def_gd,N_S_simple)*100_dp
    
    dcp_prod_mean_gd_simul                   = AVG(dcp_prod_mean_gd,N_S_simple)*100_dp
    dcp_mean_nodef_gd_simul                  = AVG(dcp_mean_nodef_gd,N_S_simple)*100_dp
    dcp_mean_sdfl_gd_simul                   = AVG(dcp_mean_sdfl_gd,N_S_simple)*100_dp
    dcp_mean_defl_gd_simul                   = AVG(dcp_mean_defl_gd,N_S_simple)*100_dp
    
    dcp_mean_nodef_cov_sdfl_defl_gd_simul    = AVG(dcp_mean_nodef_cov_sdfl_defl_gd,N_S_simple)*100_dp
    dcp_mean_sdfl_cov_nodef_defl_gd_simul    = AVG(dcp_mean_sdfl_cov_nodef_defl_gd,N_S_simple)*100_dp
    dcp_mean_defl_cov_sdfl_nodef_gd_simul    = AVG(dcp_mean_defl_cov_sdfl_nodef_gd,N_S_simple)*100_dp

    ! Decomposition  -- bad times  
    dcp_mean_real_psav_bd_simul               = AVG(dcp_mean_real_psav_bd,N_S_simple)*100_dp
    dcp_mean_real_psav_nodef_bd_simul         = AVG(dcp_mean_real_psav_nodef_bd,N_S_simple)*100_dp
    dcp_mean_real_psav_def_bd_simul           = AVG(dcp_mean_real_psav_def_bd,N_S_simple)*100_dp
    
    dcp_prod_mean_bd_simul                   = AVG(dcp_prod_mean_bd,N_S_simple)*100_dp
    dcp_mean_nodef_bd_simul                  = AVG(dcp_mean_nodef_bd,N_S_simple)*100_dp
    dcp_mean_sdfl_bd_simul                   = AVG(dcp_mean_sdfl_bd,N_S_simple)*100_dp
    dcp_mean_defl_bd_simul                   = AVG(dcp_mean_defl_bd,N_S_simple)*100_dp
    
    dcp_mean_nodef_cov_sdfl_defl_bd_simul    = AVG(dcp_mean_nodef_cov_sdfl_defl_bd,N_S_simple)*100_dp
    dcp_mean_sdfl_cov_nodef_defl_bd_simul    = AVG(dcp_mean_sdfl_cov_nodef_defl_bd,N_S_simple)*100_dp
    dcp_mean_defl_cov_sdfl_nodef_bd_simul    = AVG(dcp_mean_defl_cov_sdfl_nodef_bd,N_S_simple)*100_dp

    ! Alternative decomposition  
    decompose_prod_mean_nodef_alt_simul         = AVG(decompose_prod_mean_nodef_alt,N_S_simple)*100_dp
    decompose_prod_mean_indef_alt_simul         = AVG(decompose_prod_mean_indef_alt,N_S_simple)*100_dp
    decompose_cov_defl_sdfl_nodef_alt_simul     = AVG(decompose_cov_defl_sdfl_nodef_alt,N_S_simple)*100_dp
    decompose_cov_defl_sdfl_indef_alt_simul     = AVG(decompose_cov_defl_sdfl_indef_alt,N_S_simple)*100_dp

    ! Alternative decomposition  -- good times
    dcp_prod_mean_nodef_alt_gd_simul         = AVG(dcp_prod_mean_nodef_alt_gd,N_S_simple)*100_dp
    dcp_prod_mean_indef_alt_gd_simul         = AVG(dcp_prod_mean_indef_alt_gd,N_S_simple)*100_dp
    dcp_cov_defl_sdfl_nodef_alt_gd_simul     = AVG(dcp_cov_defl_sdfl_nodef_alt_gd,N_S_simple)*100_dp
    dcp_cov_defl_sdfl_indef_alt_gd_simul     = AVG(dcp_cov_defl_sdfl_indef_alt_gd,N_S_simple)*100_dp
    
    ! Alternative decomposition  -- bad times
    dcp_prod_mean_nodef_alt_bd_simul         = AVG(dcp_prod_mean_nodef_alt_bd,N_S_simple)*100_dp
    dcp_prod_mean_indef_alt_bd_simul         = AVG(dcp_prod_mean_indef_alt_bd,N_S_simple)*100_dp
    dcp_cov_defl_sdfl_nodef_alt_bd_simul     = AVG(dcp_cov_defl_sdfl_nodef_alt_bd,N_S_simple)*100_dp
    dcp_cov_defl_sdfl_indef_alt_bd_simul     = AVG(dcp_cov_defl_sdfl_indef_alt_bd,N_S_simple)*100_dp
    
    ! Value Functions -- lender
    vlend_avg_simul     	= AVG(vlend_avg  ,N_S_simple)
    vlend_avg_h_simul   	= AVG(vlend_avg_h,N_S_simple)
    vlend_avg_l_simul   	= AVG(vlend_avg_l,N_S_simple)
    
    vlendg_avg_simul  		= AVG(vlendg_avg  ,N_S_simple)
    vlendg_avg_h_simul		= AVG(vlendg_avg_h,N_S_simple)
    vlendg_avg_l_simul		= AVG(vlendg_avg_l,N_S_simple)
    
    vlendb_avg_simul  		= AVG(vlendb_avg  ,N_S_simple)
    vlendb_avg_h_simul		= AVG(vlendb_avg_h,N_S_simple)
    vlendb_avg_l_simul		= AVG(vlendb_avg_l,N_S_simple)
    
    vlendbb_avg_simul  		= AVG(vlendb_avg  ,N_S_simple)
    vlendbb_avg_h_simul		= AVG(vlendbb_avg_h,N_S_simple)
    vlendbb_avg_l_simul		= AVG(vlendbb_avg_l,N_S_simple)
	
    ! Value Functions -- lender	
    vvlend_avg_simul     	= AVG(vvlend_avg  ,N_S_simple)
    vvlend_avg_h_simul   	= AVG(vvlend_avg_h,N_S_simple)
    vvlend_avg_l_simul   	= AVG(vvlend_avg_l,N_S_simple)
    
    vvlendg_avg_simul  		= AVG(vvlendg_avg  ,N_S_simple)
    vvlendg_avg_h_simul		= AVG(vvlendg_avg_h,N_S_simple)
    vvlendg_avg_l_simul		= AVG(vvlendg_avg_l,N_S_simple)
    
    vvlendb_avg_simul  		= AVG(vvlendb_avg  ,N_S_simple)
    vvlendb_avg_h_simul		= AVG(vvlendb_avg_h,N_S_simple)
    vvlendb_avg_l_simul		= AVG(vvlendb_avg_l,N_S_simple)
    
    vvlendbb_avg_simul      = AVG(vvlendbb_avg  ,N_S_simple)
    vvlendbb_avg_h_simul    = AVG(vvlendbb_avg_h,N_S_simple)
    vvlendbb_avg_l_simul    = AVG(vvlendbb_avg_l,N_S_simple)
	
    ! Value Functions -- borrower
    vborr_avg_simul     	= AVG(vborr_avg  ,N_S_simple)
    vborr_avg_h_simul   	= AVG(vborr_avg_h,N_S_simple)
    vborr_avg_l_simul   	= AVG(vborr_avg_l,N_S_simple)
    
    vborrg_avg_simul  		= AVG(vborrg_avg  ,N_S_simple)
    vborrg_avg_h_simul		= AVG(vborrg_avg_h,N_S_simple)
    vborrg_avg_l_simul		= AVG(vborrg_avg_l,N_S_simple)
    
    vborrb_avg_simul  		= AVG(vborrb_avg  ,N_S_simple)
    vborrb_avg_h_simul		= AVG(vborrb_avg_h,N_S_simple)
    vborrb_avg_l_simul		= AVG(vborrb_avg_l,N_S_simple)	
    
    vborrbb_avg_simul  		= AVG(vborrbb_avg  ,N_S_simple)
    vborrbb_avg_h_simul		= AVG(vborrbb_avg_h,N_S_simple)
    vborrbb_avg_l_simul		= AVG(vborrbb_avg_l,N_S_simple)	
    
    ! Print outs
    IF (showdetails_prm) THEN
        PRINT *, ' '
        PRINT *, ' SIMULATED MOMENTS'
        PRINT *, ' '
        PRINT *, 'Avg. debt in (pct borrower income):   ', debt_in_simul
        PRINT *, 'Avg. debt out (pct borrower income):  ', debt_out_simul
        PRINT *, 'Default probability (pct, annual):    ', dprob_simul
        PRINT *, 'Avg. interest rate spread (pct):      ', spread1_avg_simul
        PRINT *, 'Avg. interest rate spread (pct) v2:   ', spread2_avg_simul
        PRINT *, 'Avg. interest rate spread (pct) __:   ', spread__avg_simul
        PRINT *, 'Good times spread (percent):          ', spreadg_avg_simul
        PRINT *, 'Bad times spread (percent):           ', spreadb_avg_simul
        PRINT *, 'Good times spread (percent) - v2:     ', sprdv2g_avg_simul
        PRINT *, 'Bad times spread (percent)  - v2:     ', sprdv2b_avg_simul
        PRINT *, 'Std. dev interest rate spread:        ', spread1_sdv_simul
        PRINT *, 'Std. dev interest rate spread v2:     ', spread2_sdv_simul
        PRINT *, 'Std. dev interest rate spread __:     ', spread__sdv_simul
        PRINT *, 'Good times def prob (percent):        ', dprobg_avg_simul
        PRINT *, 'Bad times def prob (percent):         ', dprobb_avg_simul
        PRINT *, 'Average real price (percent)          ', realprice_avg_simul
        PRINT *, 'Std dev real price', realprice_sdv_simul
        
        PRINT *, ' '
        PRINT *, ' '
        
        PRINT *, 'contribution - mean_real_psav         ', decompose_mean_real_psav_simul
        PRINT *, 'contribution - mean_real_psav_nodef   ', decompose_mean_real_psav_nodef_simul
        PRINT *, 'contribution - mean_real_psav_def     ', decompose_mean_real_psav_def_simul
        PRINT *, 'contribution - prod_mean              ', decompose_prod_mean_simul
        PRINT *, 'contribution - mean_nodef             ', decompose_mean_nodef_simul
        PRINT *, 'contribution - mean_sdfl              ', decompose_mean_sdfl_simul
        PRINT *, 'contribution - mean_defl              ', decompose_mean_defl_simul    
        PRINT *, 'contribution - mean_nodef_csdfldefl   ', decompose_mean_nodef_cov_sdfl_defl_simul
        PRINT *, 'contribution - mean_sdfl_cnodefdefl   ', decompose_mean_sdfl_cov_nodef_defl_simul
        PRINT *, 'contribution - mean_defl_csdflnodef   ', decompose_mean_defl_cov_sdfl_nodef_simul   
        
        PRINT *, ' '
        PRINT *, ' '
        
        PRINT *, 'contribution - mean_nodef_alt         ', decompose_prod_mean_nodef_alt_simul    
        PRINT *, 'contribution - mean_indef_alt         ', decompose_prod_mean_indef_alt_simul
        PRINT *, 'contribution - cov_defl_sdfl_nodef    ', decompose_cov_defl_sdfl_nodef_alt_simul
        PRINT *, 'contribution - cov_defl_sdfl_indef    ', decompose_cov_defl_sdfl_indef_alt_simul   
        
        
    END IF
    
    ! Store simulation stats
    results_simple%DEBT_IN_SIMUL_VAL        = debt_in_simul
    results_simple%DEBT_IN_H_SIMUL_VAL      = debt_in_h_simul
    results_simple%DEBT_IN_L_SIMUL_VAL      = debt_in_l_simul
    
    results_simple%DEBT_OUT_SIMUL_VAL       = debt_out_simul
    results_simple%DEBT_OUT_H_SIMUL_VAL     = debt_out_h_simul
    results_simple%DEBT_OUT_L_SIMUL_VAL     = debt_out_l_simul
    
    results_simple%SPREAD1_SIMUL_VAL        = spread1_avg_simul
    results_simple%SPREAD1_H_SIMUL_VAL      = spread1_avg_h_simul
    results_simple%SPREAD1_L_SIMUL_VAL      = spread1_avg_l_simul
    
    results_simple%SPREAD__SIMUL_VAL        = spread__avg_simul
    
    results_simple%SPREAD2_SIMUL_VAL        = spread2_avg_simul
    
    results_simple%RFREE_SIMUL_VAL          = rfree_avg_simul
    results_simple%RFREE_H_SIMUL_VAL        = rfree_avg_h_simul
    results_simple%RFREE_L_SIMUL_VAL        = rfree_avg_l_simul
    
    results_simple%SPREAD_GOOD_SIMUL_VAL	= spreadg_avg_simul
    results_simple%SPREAD_GOOD_H_SIMUL_VAL	= spreadg_avg_h_simul
    results_simple%SPREAD_GOOD_L_SIMUL_VAL	= spreadg_avg_l_simul
    
    results_simple%SPREAD_BAD_SIMUL_VAL		= spreadb_avg_simul
    results_simple%SPREAD_BAD_H_SIMUL_VAL   = spreadb_avg_h_simul
    results_simple%SPREAD_BAD_L_SIMUL_VAL   = spreadb_avg_l_simul
    
    results_simple%SPRDV2_GOOD_SIMUL_VAL	= sprdv2g_avg_simul
    
    results_simple%SPRDV2_BAD_SIMUL_VAL		= sprdv2b_avg_simul
    
    results_simple%RFREE_GOOD_SIMUL_VAL	    = rfreeg_avg_simul
    results_simple%RFREE_GOOD_H_SIMUL_VAL   = rfreeg_avg_h_simul
    results_simple%RFREE_GOOD_L_SIMUL_VAL   = rfreeg_avg_l_simul
    
    results_simple%RFREE_BAD_SIMUL_VAL		= rfreeb_avg_simul
    results_simple%RFREE_BAD_H_SIMUL_VAL    = rfreeb_avg_h_simul
    results_simple%RFREE_BAD_L_SIMUL_VAL    = rfreeb_avg_l_simul
    
    results_simple%DPROB_GOOD_SIMUL_VAL     = dprobg_avg_simul
    results_simple%DPROB_GOOD_H_SIMUL_VAL   = dprobg_avg_h_simul
    results_simple%DPROB_GOOD_L_SIMUL_VAL   = dprobg_avg_l_simul
    
    results_simple%DPROB_BAD_SIMUL_VAL      = dprobb_avg_simul
    results_simple%DPROB_BAD_H_SIMUL_VAL    = dprobb_avg_h_simul
    results_simple%DPROB_BAD_L_SIMUL_VAL    = dprobb_avg_l_simul
    
    results_simple%DPROB_SIMUL_VAL          = dprob_simul
    results_simple%DPROB_H_SIMUL_VAL        = dprob_h_simul
    results_simple%DPROB_L_SIMUL_VAL        = dprob_l_simul
    
    results_simple%SDVSPREAD1_SIMUL_VAL     = spread1_sdv_simul
    
    results_simple%SDVSPREAD__SIMUL_VAL     = spread__sdv_simul
    
    results_simple%SDVSPREAD2_SIMUL_VAL     = spread2_sdv_simul
    
    results_simple%REALPRICE_SIMUL_VAL      = realprice_avg_simul
    results_simple%REALPRICE_H_SIMUL_VAL    = realprice_avg_h_simul
    results_simple%REALPRICE_L_SIMUL_VAL    = realprice_avg_l_simul
    
    results_simple%REALPRICE_GD_SIMUL_VAL   = realprice_avg_gd_simul
    results_simple%REALPRICE_GD_H_SIMUL_VAL = realprice_avg_gd_h_simul
    results_simple%REALPRICE_GD_L_SIMUL_VAL = realprice_avg_gd_l_simul
    
    results_simple%REALPRICE_BD_SIMUL_VAL   = realprice_avg_bd_simul
    results_simple%REALPRICE_BD_H_SIMUL_VAL = realprice_avg_bd_h_simul
    results_simple%REALPRICE_BD_L_SIMUL_VAL = realprice_avg_bd_l_simul
    
    results_simple%SDVREALP_SIMUL_VAL       = realprice_sdv_simul
    
    results_simple%SDVREALP_GD_SIMUL_VAL    = realprice_sdv_gd_simul
    
    results_simple%SDVREALP_BD_SIMUL_VAL    = realprice_sdv_bd_simul
    
    results_simple%MEAN_REAL_PSAV_SIMUL_VAL         = decompose_mean_real_psav_simul
    results_simple%MEAN_REAL_PSAV_NODEF_SIMUL_VAL   = decompose_mean_real_psav_nodef_simul
    results_simple%MEAN_REAL_PSAV_DEF_SIMUL_VAL     = decompose_mean_real_psav_def_simul
    results_simple%PROD_MEAN_SIMUL_VAL              = decompose_prod_mean_simul
    results_simple%MEAN_NODEF_SIMUL_VAL             = decompose_mean_nodef_simul
    results_simple%MEAN_SDFL_SIMUL_VAL              = decompose_mean_sdfl_simul
    results_simple%MEAN_DEFL_SIMUL_VAL              = decompose_mean_defl_simul
    results_simple%MEAN_NODEF_CSDFLDEFL_SIMUL_VAL   = decompose_mean_nodef_cov_sdfl_defl_simul
    results_simple%MEAN_SDFL_CNODEFDEFL_SIMUL_VAL   = decompose_mean_sdfl_cov_nodef_defl_simul
    results_simple%MEAN_DEFL_CSDFLNODEF_SIMUL_VAL   = decompose_mean_defl_cov_sdfl_nodef_simul
    
    results_simple%MEAN_REAL_PSAV_GD_SIMUL_VAL          = dcp_mean_real_psav_gd_simul
    results_simple%MEAN_REAL_PSAV_NODEF_GD_SIMUL_VAL    = dcp_mean_real_psav_nodef_gd_simul
    results_simple%MEAN_REAL_PSAV_DEF_GD_SIMUL_VAL      = dcp_mean_real_psav_def_gd_simul
    results_simple%PROD_MEAN_GD_SIMUL_VAL              = dcp_prod_mean_gd_simul
    results_simple%MEAN_NODEF_GD_SIMUL_VAL             = dcp_mean_nodef_gd_simul
    results_simple%MEAN_SDFL_GD_SIMUL_VAL              = dcp_mean_sdfl_gd_simul
    results_simple%MEAN_DEFL_GD_SIMUL_VAL              = dcp_mean_defl_gd_simul
    results_simple%MEAN_NODEF_CSDFLDEFL_GD_SIMUL_VAL   = dcp_mean_nodef_cov_sdfl_defl_gd_simul
    results_simple%MEAN_SDFL_CNODEFDEFL_GD_SIMUL_VAL   = dcp_mean_sdfl_cov_nodef_defl_gd_simul
    results_simple%MEAN_DEFL_CSDFLNODEF_GD_SIMUL_VAL   = dcp_mean_defl_cov_sdfl_nodef_gd_simul
        
    results_simple%MEAN_REAL_PSAV_BD_SIMUL_VAL          = dcp_mean_real_psav_bd_simul
    results_simple%MEAN_REAL_PSAV_NODEF_BD_SIMUL_VAL    = dcp_mean_real_psav_nodef_bd_simul
    results_simple%MEAN_REAL_PSAV_DEF_BD_SIMUL_VAL      = dcp_mean_real_psav_def_bd_simul
    results_simple%PROD_MEAN_BD_SIMUL_VAL              = dcp_prod_mean_bd_simul
    results_simple%MEAN_NODEF_BD_SIMUL_VAL             = dcp_mean_nodef_bd_simul
    results_simple%MEAN_SDFL_BD_SIMUL_VAL              = dcp_mean_sdfl_bd_simul
    results_simple%MEAN_DEFL_BD_SIMUL_VAL              = dcp_mean_defl_bd_simul
    results_simple%MEAN_NODEF_CSDFLDEFL_BD_SIMUL_VAL   = dcp_mean_nodef_cov_sdfl_defl_bd_simul
    results_simple%MEAN_SDFL_CNODEFDEFL_BD_SIMUL_VAL   = dcp_mean_sdfl_cov_nodef_defl_bd_simul
    results_simple%MEAN_DEFL_CSDFLNODEF_BD_SIMUL_VAL   = dcp_mean_defl_cov_sdfl_nodef_bd_simul
    
    
    results_simple%PROD_MEAN_NODEF_SIMUL_VAL        = decompose_prod_mean_nodef_alt_simul
    results_simple%PROD_MEAN_INDEF_SIMUL_VAL        = decompose_prod_mean_indef_alt_simul
    results_simple%COV_DEFL_SDFL_NODEF_SIMUL_VAL    = decompose_cov_defl_sdfl_nodef_alt_simul
    results_simple%COV_DEFL_SDFL_INDEF_SIMUL_VAL    = decompose_cov_defl_sdfl_indef_alt_simul
        
    results_simple%PROD_MEAN_NODEF_GD_SIMUL_VAL        = dcp_prod_mean_nodef_alt_gd_simul
    results_simple%PROD_MEAN_INDEF_GD_SIMUL_VAL        = dcp_prod_mean_indef_alt_gd_simul
    results_simple%COV_DEFL_SDFL_NODEF_GD_SIMUL_VAL    = dcp_cov_defl_sdfl_nodef_alt_gd_simul
    results_simple%COV_DEFL_SDFL_INDEF_GD_SIMUL_VAL    = dcp_cov_defl_sdfl_indef_alt_gd_simul
        
    results_simple%PROD_MEAN_NODEF_BD_SIMUL_VAL        = dcp_prod_mean_nodef_alt_bd_simul
    results_simple%PROD_MEAN_INDEF_BD_SIMUL_VAL        = dcp_prod_mean_indef_alt_bd_simul
    results_simple%COV_DEFL_SDFL_NODEF_BD_SIMUL_VAL    = dcp_cov_defl_sdfl_nodef_alt_bd_simul
    results_simple%COV_DEFL_SDFL_INDEF_BD_SIMUL_VAL    = dcp_cov_defl_sdfl_indef_alt_bd_simul
    
    
    results_simple%VLEND_SIMUL_VAL  				= vlend_avg_simul
    results_simple%VLEND_GOOD_SIMUL_VAL  			= vlendg_avg_simul
    results_simple%VLEND_BAD_SIMUL_VAL  			= vlendb_avg_simul
    results_simple%VLEND_BB_SIMUL_VAL  			    = vlendbb_avg_simul
    results_simple%VLEND_H_SIMUL_VAL  				= vlend_avg_h_simul
    results_simple%VLEND_L_SIMUL_VAL  				= vlend_avg_l_simul
    results_simple%VLEND_GOOD_H_SIMUL_VAL  			= vlendg_avg_h_simul
    results_simple%VLEND_GOOD_L_SIMUL_VAL  			= vlendg_avg_l_simul
    results_simple%VLEND_BAD_H_SIMUL_VAL  			= vlendb_avg_h_simul
    results_simple%VLEND_BAD_L_SIMUL_VAL  			= vlendb_avg_l_simul
    results_simple%VLEND_BB_H_SIMUL_VAL  			= vlendbb_avg_h_simul
    results_simple%VLEND_BB_L_SIMUL_VAL  			= vlendbb_avg_l_simul
    
    results_simple%VVLEND_SIMUL_VAL  				= vvlend_avg_simul
    results_simple%VVLEND_GOOD_SIMUL_VAL  			= vvlendg_avg_simul
    results_simple%VVLEND_BAD_SIMUL_VAL  			= vvlendb_avg_simul
    results_simple%VVLEND_BB_SIMUL_VAL  			= vvlendbb_avg_simul
    results_simple%VVLEND_H_SIMUL_VAL  				= vvlend_avg_h_simul
    results_simple%VVLEND_L_SIMUL_VAL  				= vvlend_avg_l_simul
    results_simple%VVLEND_GOOD_H_SIMUL_VAL  		= vvlendg_avg_h_simul
    results_simple%VVLEND_GOOD_L_SIMUL_VAL  		= vvlendg_avg_l_simul
    results_simple%VVLEND_BAD_H_SIMUL_VAL  			= vvlendb_avg_h_simul
    results_simple%VVLEND_BAD_L_SIMUL_VAL  			= vvlendb_avg_l_simul
    results_simple%VVLEND_BB_H_SIMUL_VAL  			= vvlendbb_avg_h_simul
    results_simple%VVLEND_BB_L_SIMUL_VAL  			= vvlendbb_avg_l_simul

    
    results_simple%VBORR_SIMUL_VAL  				= vborr_avg_simul
    results_simple%VBORR_GOOD_SIMUL_VAL  			= vborrg_avg_simul
    results_simple%VBORR_BAD_SIMUL_VAL  			= vborrb_avg_simul
    results_simple%VBORR_BB_SIMUL_VAL  			    = vborrbb_avg_simul
    results_simple%VBORR_H_SIMUL_VAL  				= vborr_avg_h_simul
    results_simple%VBORR_L_SIMUL_VAL  				= vborr_avg_l_simul
    results_simple%VBORR_GOOD_H_SIMUL_VAL  			= vborrg_avg_h_simul
    results_simple%VBORR_GOOD_L_SIMUL_VAL  			= vborrg_avg_l_simul
    results_simple%VBORR_BAD_H_SIMUL_VAL  			= vborrb_avg_h_simul
    results_simple%VBORR_BAD_L_SIMUL_VAL  			= vborrb_avg_l_simul
    results_simple%VBORR_BB_H_SIMUL_VAL  			= vborrbb_avg_h_simul
    results_simple%VBORR_BB_L_SIMUL_VAL  			= vborrbb_avg_l_simul	
                
    PRINT *, " ------------------ END SIMULATION ---------------------- "
  
END SUBROUTINE SIMULATE_SIMPLE

SUBROUTINE RUNSIMUL_SIMPLE(ind_simul)

    IMPLICIT NONE
    INTEGER, INTENT(IN)             ::  ind_simul
    INTEGER                         ::  k, i, j, t, m
    REAL (DP)                       ::  rand, b_infl, defpr
    REAL (DP), DIMENSION(T_simul)   ::  inflss      ! inflation
    REAL (DP), DIMENSION(T_simul)   ::  bss         ! assets
    REAL (DP), DIMENSION(T_simul)   ::  css         ! consumption
    REAL (DP), DIMENSION(T_simul)   ::  defss       ! default state
    REAL (DP), DIMENSION(T_simul)   ::  pss         ! equilibrium bond price
    REAL (DP), DIMENSION(T_simul)   ::  qfss        ! synthetic risk free price for (valid in no default times)
    REAL (DP), DIMENSION(T_simul)   ::  realpss     ! equilibrium bond price adjusted by expected inflation
    REAL (DP), DIMENSION(T_simul)   ::  rlpssv2     ! equilibrium bond price adjusted by expected inflation
    REAL (DP), DIMENSION(T_simul)   ::  rfree       ! synthetic risk free rate for (valid in no default times)
    REAL (DP), DIMENSION(T_simul)   ::  defpss      ! equilibrium bond price in default
    REAL (DP), DIMENSION(T_simul)   ::  yss         ! output shocks
    REAL (DP), DIMENSION(T_simul)   ::  countdef    ! default indicator
    REAL (DP), DIMENSION(T_simul)   ::  asset       ! incoming assets
    REAL (DP), DIMENSION(T_simul)   ::  defprob     ! Expected default probability
    REAL (DP), DIMENSION(T_simul)   ::  spreadss    ! spreads
    REAL (DP), DIMENSION(T_simul)   ::  spreadv2    ! spreads
    REAL (DP), DIMENSION(T_simul)   ::  spread__    ! spreads relative to 1/bita_l
    REAL (DP), DIMENSION(T_simul)   ::  inflss_res  ! inflation innovation
    REAL (DP), DIMENSION(T_simul)   ::  logyss_res  ! log output innovation
    REAL (DP), DIMENSION(2*T_simul) ::  seq_simul   ! random shocks [0,1]
    REAL (DP), DIMENSION(T_simul)   ::  rand_simul  ! random shocks [0,1]
    REAL (DP), DIMENSION(T_simul)   ::  rnd__simul  ! random shocks [0,1]
    INTEGER, DIMENSION(T_simul)     ::  shoc_simul  ! (y,pi) shocks
    INTEGER, DIMENSION(T_simul)     ::  rgm_simul   ! regime shocks
    REAL (DP), DIMENSION(T_simul)   ::  excl_simul  ! exclusion shocks
    
    REAL (DP), DIMENSION(T_simul)   ::  vlend_simul     ! lender value function -- payments
    REAL (DP), DIMENSION(T_simul)   ::  vvlend_simul    ! lender value function -- consumption
    REAL (DP), DIMENSION(T_simul)   ::  vborr_simul     ! borrower value function
    
    LOGICAL, DIMENSION(T_simul)     ::  tranquil    ! tag valid pre-crisis windows 
    LOGICAL, DIMENSION(T_simul)     ::  goodtimes	! tag valid good times
    LOGICAL, DIMENSION(T_simul)     ::  badtimes	! tag valid bad times
    LOGICAL, DIMENSION(T_simul)     ::  bbtimes	    ! tag valid bad times x high def prob
    LOGICAL, DIMENSION(T_simul)     ::  safertimes	! tag valid safe times -- higher default costs
    LOGICAL, DIMENSION(T_simul)     ::  riskytimes	! tag valid risky times -- lower default costs
    
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s     ! empirical LR distribution of (k,B,s)
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s_nd  ! empirical LR distribution of (k,B,s) - no default
    REAL (DP), DIMENSION (ngpb)         ::  frq_B       ! empirical LR distribution of (B)
    REAL (DP), DIMENSION (ngpb)         ::  frq_B_nd    ! empirical LR distribution of (B) - no default
    
    
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s_gd      ! empirical LR distribution of (k,B,s) -- good times
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s_nd_gd   ! empirical LR distribution of (k,B,s) - no default  -- good times
    REAL (DP), DIMENSION (ngpb)         ::  frq_B_gd        ! empirical LR distribution of (B) -- good times
    REAL (DP), DIMENSION (ngpb)         ::  frq_B_nd_gd     ! empirical LR distribution of (B) - no default -- good times
    
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s_bd      ! empirical LR distribution of (k,B,s) -- bad times
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s_bb      ! empirical LR distribution of (k,B,s) -- bad times x high def prob
    REAL (DP), DIMENSION (rgm,ngpb,ss)  ::  frq_B_s_nd_bd   ! empirical LR distribution of (k,B,s) - no default  -- bad times
    REAL (DP), DIMENSION (ngpb)         ::  frq_B_bd        ! empirical LR distribution of (B) -- bad times
    REAL (DP), DIMENSION (ngpb)         ::  frq_B_nd_bd     ! empirical LR distribution of (B) - no default -- bad times
    
    REAL (DP)                           ::  tmp
    
    INTEGER                 :: ios          ! io status for system call
    CHARACTER (LEN=7)       :: cfg_str      ! configuration number
    CHARACTER (LEN=80)      :: outdir       ! output folder
    INTEGER                 ::  w
    
    ! generate output folder name
    WRITE (UNIT=cfg_str, FMT='(I<sfx>.<sfx>)') config
    outdir = TRIM(folder_out) // TRIM(cfgpfx) // '_' // TRIM(cfg_str) // '/' 
        
    ! Setup folder for ouptut files
    ios     = SYSTEM( 'mkdir -p ' // TRIM(outdir) )
    
    ! Draw random sequence
    !CALL RANDOM_SEED
    !CALL RANDOM_NUMBER(rand_simul)
    !CALL SLEEPQQ(1000)
    !CALL RANDOM_SEED
    !CALL RANDOM_NUMBER(rnd__simul)
    
    
    CALL RANDOM_SEED
    CALL RANDOM_NUMBER(seq_simul)
    rnd__simul      = seq_simul(1:T_simul)
    rand_simul      = seq_simul(T_simul+1:2*T_simul)
    
    ! Initialize shocks
    i               = MAXVAL( MINLOC( ABS(pigrid) + ABS(LOG(ygrid)) ) )
    shoc_simul(1)   = i         ! start with zero inflation and mean output
    
    k               = 1
    rgm_simul(1)    = k         ! start in (safer) high default cost regime
    
    ! Create sequence ouf inflation & output indices
    DO t=2,T_simul
        
        ! output/inflation
        j = 1
        DO WHILE ( rand_simul(t) > cum_ptran(i,j) )
            j = j + 1
        END DO
        shoc_simul(t) = j
        i = j
        
        ! regime
        IF ( rnd__simul(t) > pr_rgm(k,k) ) THEN
            k = 3 - k       ! maps (1,2) into (2,1)
        END IF
        rgm_simul(t) = k    ! new regime     
        
    END DO
    
    ! initial exclusion chocks to zero (set to random draw post-default)
    excl_simul  = zerodp
    
    ! set frequencies to zero (update for t > T_erg)
    frq_B_s     = zerodp
    frq_B_s_nd  = zerodp
    frq_B       = zerodp
    frq_B_nd    = zerodp
    
    frq_B_s_gd      = zerodp
    frq_B_s_nd_gd   = zerodp
    frq_B_gd        = zerodp
    frq_B_nd_gd     = zerodp
    
    frq_B_s_bd      = zerodp
    frq_B_s_bb      = zerodp
    frq_B_s_nd_bd   = zerodp
    frq_B_bd        = zerodp
    frq_B_nd_bd     = zerodp
    
    ! defining valid periods - update to .TRUE. if no crisis
    tranquil	= .FALSE.
	goodtimes	= .FALSE.
	badtimes	= .FALSE.
    bbtimes     = .FALSE.
	safertimes	= .FALSE.
	riskytimes	= .FALSE.
    
    ! Initialize first period variables
    t           = 1
    j           = shoc_simul(t)
    k           = rgm_simul(t)
    m           = i_b_zero
    asset(t)    = b(m)
    bss(t)      = sav(k,m,j)
    inflss(t)   = pigrid(j)
    defss(t)    = def(k,m,j)
    IF (defss(t) < onedp) THEN
        yss(t) = ygrid(j)
    ELSE
        yss(t) = ygrid(j) - penalty(ygrid(j),k)
    END IF 
    pss(t)      = psav(k,m,j)
    realpss(t)  = real_psav(k,m,j)
    rlpssv2(t)  = real_psav_v2(k,m,j)
    countdef(t) = def(k,m,j)
    defprob(t)  = dprob(k,m,j)
    css(t)      = cons(k,m,j)
    qfss(t) 	= p_riskfree(j)
    spreadss(t) = onedp - (realpss(t)/qfss(t))**4
    spread__(t) = onedp - (realpss(t)/bita_l)**4
    rfree(t)    = onedp - (qfss(t))**4
    spreadv2(t) = onedp - (rlpssv2(t))**4
    defpss(t)   = pdef(k,m,j)
       
    inflss_res(t)   = zerodp
    logyss_res(t)   = zerodp
    
    vlend_simul(t)  = v_l(k,m,j)
    vvlend_simul(t) = vv_l(k,m,j)
    vborr_simul(t)  = v(k,m,j)
    
    DO t=2,T_simul
        ! realized output shock
        j           = shoc_simul(t)
        ! realized regime shock
        k           = rgm_simul(t)
        ! compute the realized inflation
        inflss(t)   = pigrid(j)
        
        IF (defss(t-1) < onedp) THEN ! Did NOT default last period
                
            ! compute inflation adjusted debt
            b_infl      = bss(t-1)/(onedp+inflss(t))
            asset(t)    = b_infl
            
            ! index -- inflation adjusted debt
            m           = MAXVAL(MINLOC(ABS(b_infl-b)))
            
            ! default decision
            defpr       = INTERPOLATE(b,def(k,:,j),ngpb,b_infl)
            
            
            IF (defpr .EQ. zerodp) THEN ! no default this period
                defss(t)    = defpr
                yss(t)      = ygrid(j)
                bss(t)      = INTERPOLATE(b,savnd(k,:,j),ngpb,b_infl)
                pss(t)      = INTERPOLATE(b,psavnd(k,:,j),ngpb,b_infl)
                css(t)      = yss(t) + (rc+delta)*asset(t) - pss(t)*(bss(t)-(1-delta)*asset(t))
                defprob(t)  = INTERPOLATE(b,dprobnd(k,:,j),ngpb,b_infl)
                
                qfss(t) 	= p_riskfree(j)
                realpss(t)  = INTERPOLATE(b,real_psavnd(k,:,j),ngpb,b_infl)
                rlpssv2(t)  = INTERPOLATE(b,real_psavnd_v2(k,:,j),ngpb,b_infl)
                spreadss(t) = onedp - (realpss(t)/qfss(t))**4
                spread__(t) = onedp - (realpss(t)/bita_l)**4
                rfree(t)    = onedp - (qfss(t))**4
                spreadv2(t) = onedp - (rlpssv2(t))**4


                ! welfare values
                vlend_simul(t)  = INTERPOLATE(b,vnd_l(k,:,j),ngpb,b_infl)
                vvlend_simul(t) = INTERPOLATE(b,vvnd_l(k,:,j),ngpb,b_infl)
                vborr_simul(t)  = INTERPOLATE(b,vnd(k,:,j),ngpb,b_infl)
                
            ELSEIF (defpr .EQ. onedp) THEN ! DEFAULT this period
                defss(t)    = defpr
                yss(t)      = ygrid(j) - penalty(ygrid(j),k)
                bss(t)      = asset(t)
                pss(t)      = eps
                css(t)      = yss(t)
                defprob(t)  = INTERPOLATE(b,dprobid(k,:,j),ngpb,b_infl)    
                
                qfss(t) 	= p_riskfree(j)
                realpss(t)  = eps
                rlpssv2(t)  = eps
                spreadss(t) = infty     ! infty
                spread__(t) = infty     ! infty
                rfree(t)    = onedp - (qfss(t))**4     ! infty
                spreadv2(t) = infty     ! infty

                ! welfare values
                vlend_simul(t)  = INTERPOLATE(b,vdef_l(k,:,j),ngpb,b_infl)
                vvlend_simul(t) = INTERPOLATE(b,vvdef_l(k,:,j),ngpb,b_infl)
                vborr_simul(t)  = INTERPOLATE(b,vdef(k,:,j),ngpb,b_infl)
                
            ELSE ! Between DEFAULT and NOT default (mixed strategy)
                CALL RANDOM_SEED
                CALL RANDOM_NUMBER(rand)
                
                IF (rand < defpr) THEN ! Treat as DEFAULT
                    defss(t)    = onedp
                    yss(t)      = ygrid(j) - penalty(ygrid(j),k)
                    bss(t)      = asset(t)
                    pss(t)      = eps
                    css(t)      = yss(t)
                    IF (def(k,m,j).EQ.onedp) THEN
                        defprob(t)  = dprobid(k,m,j)
                    ELSE
                        defprob(t)  = dprobid(k,m-1,j)
                    END IF

                    qfss(t) 	= p_riskfree(j)
                    realpss(t)  = eps
                    rlpssv2(t)  = eps
                    spreadss(t) = infty     ! infty
                    spread__(t) = infty     ! infty
                    rfree(t)    = onedp - (qfss(t))**4     ! infty
                    spreadv2(t) = infty     ! infty
                    
                    ! welfare values
                    vlend_simul(t)  = INTERPOLATE(b,vdef_l(k,:,j),ngpb,b_infl)
                    vvlend_simul(t) = INTERPOLATE(b,vvdef_l(k,:,j),ngpb,b_infl)
                    vborr_simul(t)  = INTERPOLATE(b,vdef(k,:,j),ngpb,b_infl)
                
                ELSE ! Treat as NO default
                    defss(t)    = zerodp
                    yss(t)      = ygrid(j)
                    bss(t)      = INTERPOLATE(b,savnd(k,:,j),ngpb,b_infl)
                    pss(t)      = INTERPOLATE(b,psavnd(k,:,j),ngpb,b_infl)
                    css(t)      = yss(t) + (rc+delta)*asset(t) - pss(t)*(bss(t)-(1-delta)*asset(t))
                    
                    IF (def(k,m,j) .EQ. onedp) THEN
                        defprob(t)  = dprobnd(k,m+1,j)
                    ELSE
                        defprob(t)  = dprobnd(k,m,j)
                    END IF
                    
                    qfss(t) 	= p_riskfree(j)
                    realpss(t)  = INTERPOLATE(b,real_psavnd(k,:,j),ngpb,b_infl)
                    rlpssv2(t)  = INTERPOLATE(b,real_psavnd_v2(k,:,j),ngpb,b_infl)
                    spreadss(t) = onedp - (realpss(t)/qfss(t))**4
                    spread__(t) = onedp - (realpss(t)/bita_l)**4
                    rfree(t)    = onedp - (qfss(t))**4
                    spreadv2(t) = onedp - (rlpssv2(t))**4
                    
                    ! welfare values
                    vlend_simul(t)  = INTERPOLATE(b,vnd_l(k,:,j),ngpb,b_infl)
                    vvlend_simul(t) = INTERPOLATE(b,vvnd_l(k,:,j),ngpb,b_infl)
                    vborr_simul(t)  = INTERPOLATE(b,vnd(k,:,j),ngpb,b_infl)
                
                END IF
            END IF
            
            defpss(t)   = INTERPOLATE(b,pdef(k,:,j),ngpb,b_infl)
            countdef(t) = defss(t)

        ELSE ! in DEFAULT last period
            
            ! compute inflation adjusted debt
            b_infl      = lambda*bss(t-1)/(onedp+inflss(t))
            asset(t)    = b_infl
            m           = MAXVAL(MINLOC(ABS(b_infl-b)))
            
            !CALL RANDOM_SEED
            CALL RANDOM_NUMBER(rand)
            excl_simul(t) = rand
                
            IF ( rand < theta_mat(rgm_simul(t),rgm_simul(t-1)) ) THEN ! Regain access to financial markets
                
                defpr = INTERPOLATE(b,def(k,:,j),ngpb,b_infl)
                
                IF (defpr .EQ. zerodp) THEN ! no default this period
                    defss(t)    = defpr
                    yss(t)      = ygrid(j)
                    bss(t)      = INTERPOLATE(b,savnd(k,:,j),ngpb,b_infl)
                    pss(t)      = INTERPOLATE(b,psavnd(k,:,j),ngpb,b_infl)
                    css(t)      = yss(t) + (rc+delta)*asset(t) - pss(t)*(bss(t)-(1-delta)*asset(t))
                    defprob(t)  = INTERPOLATE(b,dprobnd(k,:,j),ngpb,b_infl)
                    countdef(t) = zerodp
                    
                    qfss(t) 	= p_riskfree(j)
                    realpss(t)  = INTERPOLATE(b,real_psavnd(k,:,j),ngpb,b_infl)
                    rlpssv2(t)  = INTERPOLATE(b,real_psavnd_v2(k,:,j),ngpb,b_infl)
                    spreadss(t) = onedp - (realpss(t)/qfss(t))**4
                    spread__(t) = onedp - (realpss(t)/bita_l)**4
                    rfree(t)    = onedp - (qfss(t))**4
                    spreadv2(t) = onedp - (rlpssv2(t))**4
                    
                    ! welfare values
                    vlend_simul(t)  = INTERPOLATE(b,vnd_l(k,:,j),ngpb,b_infl)
                    vvlend_simul(t) = INTERPOLATE(b,vvnd_l(k,:,j),ngpb,b_infl)
                    vborr_simul(t)  = INTERPOLATE(b,vnd(k,:,j),ngpb,b_infl)             
                    
                ELSEIF (defpr .EQ. onedp) THEN ! DEFAULT this period
                    defss(t)    = defpr
                    yss(t)      = ygrid(j) - penalty(ygrid(j),k)
                    bss(t)      = asset(t)
                    pss(t)      = eps
                    css(t)      = yss(t)
                    defprob(t)  = INTERPOLATE(b,dprobid(k,:,j),ngpb,b_infl)
                    countdef(t) = defss(t)
                    
                    
                    qfss(t) 	= p_riskfree(j)
                    realpss(t)  = eps
                    rlpssv2(t)  = eps
                    spreadss(t) = infty     ! infty
                    spread__(t) = infty     ! infty
                    rfree(t)    = onedp - (qfss(t))**4     ! infty
                    spreadv2(t) = infty     ! infty
                    
                    ! welfare values
                    vlend_simul(t)  = INTERPOLATE(b,vdef_l(k,:,j),ngpb,b_infl)
                    vvlend_simul(t) = INTERPOLATE(b,vvdef_l(k,:,j),ngpb,b_infl)
                    vborr_simul(t)  = INTERPOLATE(b,vdef(k,:,j),ngpb,b_infl)

                    
                ELSE ! Between DEFAULT and NOT default (mixed strategy)
                    
                    !CALL RANDOM_SEED
                    CALL RANDOM_NUMBER(rand)
                    
                    IF (rand < defpr) THEN ! Treat as DEFAULT
                        defss(t)    = onedp
                        yss(t)      = ygrid(j) - penalty(ygrid(j),k)
                        bss(t)      = asset(t)
                        pss(t)      = eps
                        css(t)      = yss(t)
                        IF (def(k,m,j) .EQ. onedp) THEN ! If closest grid point is DEFAULT
                            defprob(t)  = dprobid(k,m,j)
                        ELSE ! Else choose the point with higher debt
                            defprob(t)  = dprobid(k,m-1,j)
                        END IF
                        countdef(t) = defss(t)

                        qfss(t) 	= p_riskfree(j)
                        realpss(t)  = eps
                        rlpssv2(t)  = eps
                        spreadss(t) = infty     ! infty
                        spread__(t) = infty     ! infty
                        rfree(t)    = onedp - (qfss(t))**4     ! infty
                        spreadv2(t) = infty     ! infty
                        
                        ! welfare values
                        vlend_simul(t)  = INTERPOLATE(b,vdef_l(k,:,j),ngpb,b_infl)
                        vvlend_simul(t) = INTERPOLATE(b,vvdef_l(k,:,j),ngpb,b_infl)
                        vborr_simul(t)  = INTERPOLATE(b,vdef(k,:,j),ngpb,b_infl)

                    ELSE ! Treat as NO default
                        defss(t)    = zerodp
                        yss(t)      = ygrid(j)
                        bss(t)      = INTERPOLATE(b,savnd(k,:,j),ngpb,b_infl)
                        pss(t)      = INTERPOLATE(b,psavnd(k,:,j),ngpb,b_infl)
                        css(t)      = yss(t) + (rc+delta)*asset(t) - pss(t)*(bss(t)-(1-delta)*asset(t))
                        IF (def(k,m,j) .EQ. onedp) THEN ! If closest grid point is DEFAULT
                            defprob(t)  = dprobnd(k,m+1,j) ! Then choose the point with lower debt
                        ELSE
                            defprob(t)  = dprobnd(k,m,j)
                        END IF
                        countdef(t) = zerodp

                        qfss(t) 	= p_riskfree(j)
                        realpss(t)  = INTERPOLATE(b,real_psavnd(k,:,j),ngpb,b_infl)
                        rlpssv2(t)  = INTERPOLATE(b,real_psavnd_v2(k,:,j),ngpb,b_infl)
                        spreadss(t) = onedp - (realpss(t)/qfss(t))**4
                        spread__(t) = onedp - (realpss(t)/bita_l)**4
                        rfree(t)    = onedp - (qfss(t))**4
                        spreadv2(t) = onedp - (rlpssv2(t))**4
                        
                        ! welfare values
                        vlend_simul(t)  = INTERPOLATE(b,vnd_l(k,:,j),ngpb,b_infl)
                        vvlend_simul(t) = INTERPOLATE(b,vvnd_l(k,:,j),ngpb,b_infl)
                        vborr_simul(t)  = INTERPOLATE(b,vnd(k,:,j),ngpb,b_infl)

                
                    END IF
                END IF
            
            ELSE ! Remain in autarky
                defss(t)    = onedp
                countdef(t) = zerodp
                yss(t)      = ygrid(j) - penalty(ygrid(j),k)
                bss(t)      = asset(t)
                pss(t)      = eps
                css(t)      = yss(t)
                defprob(t)  = INTERPOLATE(b,dprobid(k,:,j),ngpb,b_infl)


                qfss(t) 	= p_riskfree(j)
                realpss(t)  = eps
                rlpssv2(t)  = eps
                spreadss(t) = infty     ! infty
                spread__(t) = infty     ! infty
                rfree(t)    = onedp - (qfss(t))**4     ! infty
                spreadv2(t) = infty     ! infty
                
                ! welfare values
                vlend_simul(t)  = INTERPOLATE(b,vdef_l(k,:,j),ngpb,b_infl)
                vvlend_simul(t) = INTERPOLATE(b,vvdef_l(k,:,j),ngpb,b_infl)
                vborr_simul(t)  = INTERPOLATE(b,vdef(k,:,j),ngpb,b_infl)
                
            END IF
            
            defpss(t)   = INTERPOLATE(b,pdef(k,:,j),ngpb,b_infl)
            
        END IF
        
        ! residuals
        inflss_res(t)   = inflss(t)     - SUM(ptran(shoc_simul(t-1),:)*pigrid)
        logyss_res(t)   = LOG(yss(t))   - SUM(ptran(shoc_simul(t-1),:)*LOG(ygrid))

        ! update if valid no-crisis period
        !tranquil(t)     = (t > T_simul_erg) .AND. (defss(t) < onedp) .AND. (spreadss(t) < rcrisis)
        tranquil(t)     = (t > T_simul_erg) .AND. (defss(t) < onedp) .AND. (spreadss(t) < rcrisis)
        
		! update good and bad times
        goodtimes(t)	= tranquil(t) .AND. (yss(t) .GE. yrec)
		badtimes(t)		= tranquil(t) .AND. (yss(t) < yrec) 
        bbtimes(t)      = tranquil(t) .AND. (yss(t) < yrec) .AND. (defprob(t) > pdef_bad)
        
        ! update regimes
        safertimes(t)	= (rgm_simul(t) .EQ. 1)
		riskytimes(t)   = (rgm_simul(t) .EQ. 2) 
        
        ! empirical frequency counts
        IF ( t .GT. T_simul_erg) THEN
            frq_B_s(k,m,j)      = frq_B_s(k,m,j) + onedp
            frq_B(m)            = frq_B(m) + onedp
        END IF

        IF ( (t .GT. T_simul_erg) .AND. tranquil(t) ) THEN
            frq_B_s_nd(k,m,j)   = frq_B_s_nd(k,m,j) + onedp
            frq_B_nd(m)         = frq_B_nd(m) + onedp
        END IF
        
        IF ( (t .GT. T_simul_erg) .AND. goodtimes(t) ) THEN
            frq_B_s_gd(k,m,j)   = frq_B_s_gd(k,m,j) + onedp
            frq_B_gd(m)         = frq_B_gd(m) + onedp
        END IF

        IF ( (t .GT. T_simul_erg) .AND. goodtimes(t) ) THEN
            frq_B_s_nd_gd(k,m,j)= frq_B_s_nd_gd(k,m,j) + onedp
            frq_B_nd_gd(m)      = frq_B_nd_gd(m) + onedp
        END IF
        
        IF ( (t .GT. T_simul_erg) .AND. badtimes(t) ) THEN
            frq_B_s_bd(k,m,j)   = frq_B_s_bd(k,m,j) + onedp
            frq_B_bd(m)         = frq_B_bd(m) + onedp
        END IF
        
        IF ( (t .GT. T_simul_erg) .AND. bbtimes(t) ) THEN
            frq_B_s_bb(k,m,j)   = frq_B_s_bb(k,m,j) + onedp
        END IF

        IF ( (t .GT. T_simul_erg) .AND. badtimes(t) ) THEN
            frq_B_s_nd_bd(k,m,j)= frq_B_s_nd_bd(k,m,j) + onedp
            frq_B_nd_bd(m)      = frq_B_nd_bd(m) + onedp
        END IF
        
    END DO  

    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!! SIMULATION AVERAGES !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    IF (ALL( .NOT. (tranquil)) .OR. (ind_simul == 1)) THEN
        
        OPEN (unit=9, file= TRIM(outdir) // "defss.out", position="rewind")
        WRITE (9, *) defss
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "spreadss.out", position="rewind")
        WRITE (9, *) spreadss
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "realpss.out", position="rewind")
        WRITE (9, *) realpss
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "qfss.out", position="rewind")
        WRITE (9, *) qfss
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "riskytimes.out", position="rewind")
        WRITE (9, *) riskytimes
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "bbtimes.out", position="rewind")
        WRITE (9, *) bbtimes
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "countdef.out", position="rewind")
        WRITE (9, *) countdef
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "excl_simul.out", position="rewind")
        WRITE (9, *) excl_simul
        CLOSE (unit=9)
        
        OPEN (unit=9, file= TRIM(outdir) // "rand_simul.out", position="rewind")
        WRITE (9, *) rand_simul
        CLOSE (unit=9)
        
        
        OPEN (unit=9, file= TRIM(outdir) // "rnd__simul.out", position="rewind")
        WRITE (9, *) rnd__simul
        CLOSE (unit=9)
        
        
    END IF 
    
    !! TRANQUIL TIMES !!
    ! computing the implied ergodic distribution
    tmp             = SUM(frq_B_s)
    frq_B_s         = frq_B_s/tmp
    erg_sav_shock   = erg_sav_shock + frq_B_s
    
    ! computing the implied ergodic distribution
    tmp             = SUM(frq_B)
    frq_B           = frq_B/tmp
    erg_sav         = erg_sav + frq_B

    ! computing the implied ergodic distribution - no default
    tmp             = SUM(frq_B_nd)
    frq_B_nd        = frq_B_nd/tmp
    erg_sav_nd      = erg_sav_nd + frq_B_nd

    ! computing the implied ergodic distribution - no default
    tmp             = SUM(frq_B_s_nd)
    frq_B_s_nd      = frq_B_s_nd/tmp
    erg_sav_shock_nd= erg_sav_shock_nd + frq_B_s_nd
    
    !! GOOD TIMES !!
    ! computing the implied ergodic distribution
    tmp                 = SUM(frq_B_s_gd)
    frq_B_s_gd          = frq_B_s_gd/tmp
    erg_sav_shock_gd    = erg_sav_shock_gd + frq_B_s_gd
    
    ! computing the implied ergodic distribution
    tmp                 = SUM(frq_B_gd)
    frq_B_gd            = frq_B_gd/tmp
    erg_sav_gd          = erg_sav_gd + frq_B_gd

    ! computing the implied ergodic distribution - no default
    tmp                 = SUM(frq_B_nd_gd)
    frq_B_nd_gd         = frq_B_nd_gd/tmp
    erg_sav_nd_gd       = erg_sav_nd_gd + frq_B_nd_gd

    ! computing the implied ergodic distribution - no default
    tmp                 = SUM(frq_B_s_nd_gd)
    frq_B_s_nd_gd       = frq_B_s_nd_gd/tmp
    erg_sav_shock_nd_gd = erg_sav_shock_nd_gd + frq_B_s_nd_gd
    
    !! BAD TIMES !!
    ! computing the implied ergodic distribution
    tmp                 = SUM(frq_B_s_bd)
    frq_B_s_bd          = frq_B_s_bd/tmp
    erg_sav_shock_bd    = erg_sav_shock_bd + frq_B_s_bd
    
    ! computing the implied ergodic distribution
    tmp                 = SUM(frq_B_s_bb)
    frq_B_s_bb          = frq_B_s_bb/tmp
    erg_sav_shock_bb    = erg_sav_shock_bb + frq_B_s_bb
    
    ! computing the implied ergodic distribution
    tmp                 = SUM(frq_B_bd)
    frq_B_bd            = frq_B_bd/tmp
    erg_sav_bd          = erg_sav_bd + frq_B_bd

    ! computing the implied ergodic distribution - no default
    tmp                 = SUM(frq_B_nd_bd)
    frq_B_nd_bd         = frq_B_nd_bd/tmp
    erg_sav_nd_bd       = erg_sav_nd_bd + frq_B_nd_bd

    ! computing the implied ergodic distribution - no default
    tmp                 = SUM(frq_B_s_nd_bd)
    frq_B_s_nd_bd       = frq_B_s_nd_bd/tmp
    erg_sav_shock_nd_bd = erg_sav_shock_nd_bd + frq_B_s_nd_bd
    
    ! Empirical default probability
    !dprob_avg(ind_simul) =  ( onedp + AVG(countdef,T_simul) )**4.0_dp - onedp
    !dprob_avg(ind_simul) =  onedp - ( onedp - AVG(countdef,T_simul) )**4.0_dp
    dprob_avg(ind_simul)        =  AVG(countdef,T_simul)*4.0_dp
    dprob_avg_h(ind_simul)      =  AVG(countdef,T_simul, safertimes)*4.0_dp
    dprob_avg_l(ind_simul)      =  AVG(countdef,T_simul, riskytimes)*4.0_dp
    
    ! Debt level
    debt_in_avg(ind_simul)      = -SUM(asset,tranquil)/SUM(yss,tranquil)/4.0_dp
    debt_in_avg_h(ind_simul)    = -SUM(asset,tranquil .AND. safertimes)/SUM(yss,tranquil .AND. safertimes)/4.0_dp
    debt_in_avg_l(ind_simul)    = -SUM(asset,tranquil .AND. riskytimes)/SUM(yss,tranquil .AND. riskytimes)/4.0_dp
    
    debt_out_avg(ind_simul)     = -SUM(bss,tranquil)/SUM(yss,tranquil)/4.0_dp
    debt_out_avg_h(ind_simul)   = -SUM(bss,tranquil .AND. safertimes)/SUM(yss,tranquil .AND. safertimes)/4.0_dp
    debt_out_avg_l(ind_simul)   = -SUM(bss,tranquil .AND. riskytimes)/SUM(yss,tranquil .AND. riskytimes)/4.0_dp
    
    ! Spreads - avg
    spread1_avg(ind_simul)      = AVG(spreadss, T_simul, tranquil)
    spread1_avg_h(ind_simul)    = AVG(spreadss, T_simul, tranquil .AND. safertimes)
    spread1_avg_l(ind_simul)    = AVG(spreadss, T_simul, tranquil .AND. riskytimes)
       
    spread__avg(ind_simul)      = AVG(spread__, T_simul, tranquil)
    
    rfree_avg(ind_simul)        = AVG(rfree, T_simul, tranquil)
    rfree_avg_h(ind_simul)      = AVG(rfree, T_simul, tranquil .AND. safertimes)
    rfree_avg_l(ind_simul)      = AVG(rfree, T_simul, tranquil .AND. riskytimes)
    
    spread2_avg(ind_simul)      = AVG(spreadv2, T_simul, tranquil)
    
    ! Spreads - good times - avg
    spreadg_avg(ind_simul)      = AVG(spreadss, T_simul, goodtimes)
    spreadg_avg_h(ind_simul)    = AVG(spreadss, T_simul, goodtimes .AND. safertimes)
    spreadg_avg_l(ind_simul)    = AVG(spreadss, T_simul, goodtimes .AND. riskytimes)
    
    rfreeg_avg(ind_simul)       = AVG(rfree, T_simul, goodtimes)
    rfreeg_avg_h(ind_simul)     = AVG(rfree, T_simul, goodtimes .AND. safertimes)
    rfreeg_avg_l(ind_simul)     = AVG(rfree, T_simul, goodtimes .AND. riskytimes)
 
    sprdv2g_avg(ind_simul)      = AVG(spreadv2, T_simul, goodtimes)
    
    ! Spreads - bad times - avg
    spreadb_avg(ind_simul)      = AVG(spreadss, T_simul, badtimes)
    spreadb_avg_h(ind_simul)    = AVG(spreadss, T_simul, badtimes .AND. safertimes)
    spreadb_avg_l(ind_simul)    = AVG(spreadss, T_simul, badtimes .AND. riskytimes)
    
    rfreeb_avg(ind_simul)       = AVG(rfree, T_simul, badtimes)
    rfreeb_avg_h(ind_simul)     = AVG(rfree, T_simul, badtimes .AND. safertimes)
    rfreeb_avg_l(ind_simul)     = AVG(rfree, T_simul, badtimes .AND. riskytimes)
    
    sprdv2b_avg(ind_simul)      = AVG(spreadv2, T_simul, badtimes)
    
    ! Default probabilities
    dprobg_avg(ind_simul)       = onedp - ( onedp - AVGBLK(countdef, T_simul, cds_mat, goodtimes) )**(4.0_dp/REAL(cds_mat,DP))
    dprobg_avg_h(ind_simul)     = onedp - ( onedp - AVGBLK(countdef, T_simul, cds_mat, goodtimes .AND. safertimes) )**(4.0_dp/REAL(cds_mat,DP))
    dprobg_avg_l(ind_simul)     = onedp - ( onedp - AVGBLK(countdef, T_simul, cds_mat, goodtimes .AND. riskytimes) )**(4.0_dp/REAL(cds_mat,DP))
    
    dprobb_avg(ind_simul)       = onedp - ( onedp - AVGBLK(countdef, T_simul, cds_mat, badtimes)  )**(4.0_dp/REAL(cds_mat,DP))
    dprobb_avg_h(ind_simul)     = onedp - ( onedp - AVGBLK(countdef, T_simul, cds_mat, badtimes .AND. safertimes)  )**(4.0_dp/REAL(cds_mat,DP))
    dprobb_avg_l(ind_simul)     = onedp - ( onedp - AVGBLK(countdef, T_simul, cds_mat, badtimes .AND. riskytimes)  )**(4.0_dp/REAL(cds_mat,DP))
    
    !dprobg_avg(ind_simul)   = onedp - ( onedp - AVG(defprob, T_simul, goodtimes) )**4.0_dp
    !dprobb_avg(ind_simul)   = onedp - ( onedp - AVG(defprob, T_simul, badtimes) )**4.0_dp
    !dprobg_avg(ind_simul)   = onedp - ( onedp - AVG(defprob, T_simul, goodtimes) )**4.0_dp
    !dprobb_avg(ind_simul)   = onedp - ( onedp - AVG(defprob, T_simul, badtimes) )**4.0_dp
    !dprobg_avg(ind_simul)   = AVG(defprob, T_simul, goodtimes)*4.0_dp
    !dprobb_avg(ind_simul)   = AVG(defprob, T_simul, badtimes)*4.0_dp
    
    ! Spread variance
    spread1_sdv(ind_simul)  = SQRT(COVARIANCE(     spreadss,                &
                                                    spreadss,               &
                                                    T_simul,                &
                                                    tranquil                &
                                                ))  
    
    spread__sdv(ind_simul)  = SQRT(COVARIANCE(     spread__,                &
                                                    spread__,               &
                                                    T_simul,                &
                                                    tranquil                &
                                                )) 
    
    spread2_sdv(ind_simul)  = SQRT(COVARIANCE(     spreadv2,                &
                                                    spreadv2,               &
                                                    T_simul,                &
                                                    tranquil                &
                                                )) 
        
    ! Real bond price - avg
    realprice_avg(ind_simul)        = AVG(realpss, T_simul, tranquil)
    realprice_avg_h(ind_simul)      = AVG(realpss, T_simul, tranquil .AND. safertimes)
    realprice_avg_l(ind_simul)      = AVG(realpss, T_simul, tranquil .AND. riskytimes)
        
    realprice_avg_gd(ind_simul)     = AVG(realpss, T_simul, goodtimes)
    realprice_avg_gd_h(ind_simul)   = AVG(realpss, T_simul, goodtimes .AND. safertimes)
    realprice_avg_gd_l(ind_simul)   = AVG(realpss, T_simul, goodtimes .AND. riskytimes)
    
    realprice_avg_bd(ind_simul)     = AVG(realpss, T_simul, badtimes)
    realprice_avg_bd_h(ind_simul)   = AVG(realpss, T_simul, badtimes .AND. safertimes)
    realprice_avg_bd_l(ind_simul)   = AVG(realpss, T_simul, badtimes .AND. riskytimes)
    
    ! Real bond price - variance
    realprice_sdv(ind_simul)    = SQRT(COVARIANCE(    realpss,          &
                                                    realpss,            &
                                                    T_simul,            &
                                                    tranquil            &
                                                ))  
    
    realprice_sdv_gd(ind_simul) = SQRT(COVARIANCE(    realpss,          &
                                                    realpss,            &
                                                    T_simul,            &
                                                    goodtimes           &
                                                ))  
   
    realprice_sdv_bd(ind_simul) = SQRT(COVARIANCE(    realpss,          &
                                                    realpss,            &
                                                    T_simul,            &
                                                    badtimes            &
                                                ))          
    
    ! Value functions - lender
    vlend_avg(ind_simul)        = AVG(vlend_simul, T_simul, tranquil)
    vlend_avg_h(ind_simul)      = AVG(vlend_simul, T_simul, tranquil .AND. safertimes)
    vlend_avg_l(ind_simul)      = AVG(vlend_simul, T_simul, tranquil .AND. riskytimes)
        
    vlendg_avg(ind_simul)       = AVG(vlend_simul, T_simul, goodtimes)
    vlendg_avg_h(ind_simul)     = AVG(vlend_simul, T_simul, goodtimes .AND. safertimes)
    vlendg_avg_l(ind_simul)     = AVG(vlend_simul, T_simul, goodtimes .AND. riskytimes)
    
    vlendb_avg(ind_simul)       = AVG(vlend_simul, T_simul, badtimes)
    vlendb_avg_h(ind_simul)     = AVG(vlend_simul, T_simul, badtimes .AND. safertimes)
    vlendb_avg_l(ind_simul)     = AVG(vlend_simul, T_simul, badtimes .AND. riskytimes)
    
    vlendbb_avg(ind_simul)       = AVG(vlend_simul, T_simul, bbtimes)
    vlendbb_avg_h(ind_simul)     = AVG(vlend_simul, T_simul, bbtimes .AND. safertimes)
    vlendbb_avg_l(ind_simul)     = AVG(vlend_simul, T_simul, bbtimes .AND. riskytimes)
	
    ! Value functions - lender
    vvlend_avg(ind_simul)       = AVG(vvlend_simul, T_simul, tranquil)
    vvlend_avg_h(ind_simul)     = AVG(vvlend_simul, T_simul, tranquil .AND. safertimes)
    vvlend_avg_l(ind_simul)     = AVG(vvlend_simul, T_simul, tranquil .AND. riskytimes)
        
    vvlendg_avg(ind_simul)      = AVG(vvlend_simul, T_simul, goodtimes)
    vvlendg_avg_h(ind_simul)    = AVG(vvlend_simul, T_simul, goodtimes .AND. safertimes)
    vvlendg_avg_l(ind_simul)    = AVG(vvlend_simul, T_simul, goodtimes .AND. riskytimes)
    
    vvlendb_avg(ind_simul)      = AVG(vvlend_simul, T_simul, badtimes)
    vvlendb_avg_h(ind_simul)    = AVG(vvlend_simul, T_simul, badtimes .AND. safertimes)
    vvlendb_avg_l(ind_simul)    = AVG(vvlend_simul, T_simul, badtimes .AND. riskytimes)	
    
    vvlendbb_avg(ind_simul)     = AVG(vvlend_simul, T_simul, bbtimes)
    vvlendbb_avg_h(ind_simul)   = AVG(vvlend_simul, T_simul, bbtimes .AND. safertimes)
    vvlendbb_avg_l(ind_simul)   = AVG(vvlend_simul, T_simul, bbtimes .AND. riskytimes)
	
    ! Value functions - borrower
    vborr_avg(ind_simul)        = AVG(vborr_simul, T_simul, tranquil)
    vborr_avg_h(ind_simul)      = AVG(vborr_simul, T_simul, tranquil .AND. safertimes)
    vborr_avg_l(ind_simul)      = AVG(vborr_simul, T_simul, tranquil .AND. riskytimes)
        
    vborrg_avg(ind_simul)       = AVG(vborr_simul, T_simul, goodtimes)
    vborrg_avg_h(ind_simul)     = AVG(vborr_simul, T_simul, goodtimes .AND. safertimes)
    vborrg_avg_l(ind_simul)     = AVG(vborr_simul, T_simul, goodtimes .AND. riskytimes)
    
    vborrb_avg(ind_simul)       = AVG(vborr_simul, T_simul, badtimes)
    vborrb_avg_h(ind_simul)     = AVG(vborr_simul, T_simul, badtimes .AND. safertimes)
    vborrb_avg_l(ind_simul)     = AVG(vborr_simul, T_simul, badtimes .AND. riskytimes)
    
    vborrb_avg(ind_simul)       = AVG(vborr_simul, T_simul, badtimes)
    vborrb_avg_h(ind_simul)     = AVG(vborr_simul, T_simul, badtimes .AND. safertimes)
    vborrb_avg_l(ind_simul)     = AVG(vborr_simul, T_simul, badtimes .AND. riskytimes)	
    
    vborrbb_avg(ind_simul)      = AVG(vborr_simul, T_simul, bbtimes)
    vborrbb_avg_h(ind_simul)    = AVG(vborr_simul, T_simul, bbtimes .AND. safertimes)
    vborrbb_avg_l(ind_simul)    = AVG(vborr_simul, T_simul, bbtimes .AND. riskytimes)        
        
    !! DECOMPOSITION OF SPREADS 
    !!
    !! decomposing realpss (q) = E(xyz)
    !! 
    !! terms are (see POST_PROCESS)
    !! >> decompose_mean_real_psav
    !! >> decompose_prod_mean
    !! >> decompose_mean_nodef
    !! >> decompose_mean_sdfl
    !! >> decompose_mean_defl
    !! >> decompose_mean_nodef_cov_sdfl_defl
    !! >> decompose_mean_sdfl_cov_nodef_defl
    !! >> decompose_mean_defl_cov_sdfl_nodef
    !! >> decompose_residual
    
    ! >> decompose_mean_real_psav
    decompose_mean_real_psav(ind_simul)              = SUM(frq_B_s_nd*real_psav)
    decompose_mean_real_psav_nodef(ind_simul)        = SUM(frq_B_s_nd*real_psav_nodef)
    decompose_mean_real_psav_def(ind_simul)          = SUM(frq_B_s_nd*real_psav_def)
    
    ! >> decompose_prod_mean
    decompose_prod_mean(ind_simul)                  = SUM(frq_B_s_nd*realp_prod_mean)
    
    ! >> decompose_mean_nodef
    decompose_mean_nodef(ind_simul)                 = SUM(frq_B_s_nd*realp_mean_nodef)
    
    ! >> decompose_mean_sdfl
    decompose_mean_sdfl(ind_simul)                  = SUM(frq_B_s_nd*realp_mean_sdfl)
    
    ! >> decompose_mean_defl
    decompose_mean_defl(ind_simul)                  = SUM(frq_B_s_nd*realp_mean_defl)
    
    ! >> decompose_mean_nodef_cov_sdfl_defl
    decompose_mean_nodef_cov_sdfl_defl(ind_simul)   = SUM(frq_B_s_nd*realp_mean_nodef_cov_sdfl_defl)
    
    ! >> decompose_mean_sdfl_cov_nodef_defl    
    decompose_mean_sdfl_cov_nodef_defl(ind_simul)   = SUM(frq_B_s_nd*realp_mean_sdfl_cov_nodef_defl)
    
    ! >> decompose_mean_defl_cov_sdfl_nodef
    decompose_mean_defl_cov_sdfl_nodef(ind_simul)   = SUM(frq_B_s_nd*realp_mean_defl_cov_sdfl_nodef)

        
    !! DECOMPOSITION OF SPREADS  --- GOOD TIMES
    !!
    !! decomposing realpss (q) = E(xyz)
    !! 
    !! terms are (see POST_PROCESS)
    !! >> dcp_mean_real_psav_gd
    !! >> dcp_prod_mean_gd
    !! >> dcp_mean_nodef_gd
    !! >> dcp_mean_sdfl_gd
    !! >> dcp_mean_defl_gd
    !! >> dcp_mean_nodef_cov_sdfl_defl_gd
    !! >> dcp_mean_sdfl_cov_nodef_defl_gd
    !! >> dcp_mean_defl_cov_sdfl_nodef_gd
    !! >> dcp_residual_gd
    
    ! >> dcp_mean_real_psav
    dcp_mean_real_psav_gd(ind_simul)              = SUM(frq_B_s_nd_gd*real_psav)
    dcp_mean_real_psav_nodef_gd(ind_simul)        = SUM(frq_B_s_nd_gd*real_psav_nodef)
    dcp_mean_real_psav_def_gd(ind_simul)          = SUM(frq_B_s_nd_gd*real_psav_def)
    
    ! >> dcp_prod_mean
    dcp_prod_mean_gd(ind_simul)                  = SUM(frq_B_s_nd_gd*realp_prod_mean)
    
    ! >> dcp_mean_nodef
    dcp_mean_nodef_gd(ind_simul)                 = SUM(frq_B_s_nd_gd*realp_mean_nodef)
    
    ! >> dcp_mean_sdfl
    dcp_mean_sdfl_gd(ind_simul)                  = SUM(frq_B_s_nd_gd*realp_mean_sdfl)
    
    ! >> dcp_mean_defl
    dcp_mean_defl_gd(ind_simul)                  = SUM(frq_B_s_nd_gd*realp_mean_defl)
    
    ! >> dcp_mean_nodef_cov_sdfl_defl
    dcp_mean_nodef_cov_sdfl_defl_gd(ind_simul)   = SUM(frq_B_s_nd_gd*realp_mean_nodef_cov_sdfl_defl)
    
    ! >> dcp_mean_sdfl_cov_nodef_defl    
    dcp_mean_sdfl_cov_nodef_defl_gd(ind_simul)   = SUM(frq_B_s_nd_gd*realp_mean_sdfl_cov_nodef_defl)
    
    ! >> dcp_mean_defl_cov_sdfl_nodef
    dcp_mean_defl_cov_sdfl_nodef_gd(ind_simul)   = SUM(frq_B_s_nd_gd*realp_mean_defl_cov_sdfl_nodef)

        
    !! DECOMPOSITION OF SPREADS  --- BAD TIMES
    !!
    !! decomposing realpss (q) = E(xyz)
    !! 
    !! terms are (see POST_PROCESS)
    !! >> dcp_mean_real_psav_bd
    !! >> dcp_prod_mean_bd
    !! >> dcp_mean_nodef_bd
    !! >> dcp_mean_sdfl_bd
    !! >> dcp_mean_defl_bd
    !! >> dcp_mean_nodef_cov_sdfl_defl_bd
    !! >> dcp_mean_sdfl_cov_nodef_defl_bd
    !! >> dcp_mean_defl_cov_sdfl_nodef_bd
    !! >> dcp_residual_bd
    
    ! >> dcp_mean_real_psav
    dcp_mean_real_psav_bd(ind_simul)              = SUM(frq_B_s_nd_bd*real_psav)
    dcp_mean_real_psav_nodef_bd(ind_simul)        = SUM(frq_B_s_nd_bd*real_psav_nodef)
    dcp_mean_real_psav_def_bd(ind_simul)          = SUM(frq_B_s_nd_bd*real_psav_def)
    
    ! >> dcp_prod_mean
    dcp_prod_mean_bd(ind_simul)                  = SUM(frq_B_s_nd_bd*realp_prod_mean)
    
    ! >> dcp_mean_nodef
    dcp_mean_nodef_bd(ind_simul)                 = SUM(frq_B_s_nd_bd*realp_mean_nodef)
    
    ! >> dcp_mean_sdfl
    dcp_mean_sdfl_bd(ind_simul)                  = SUM(frq_B_s_nd_bd*realp_mean_sdfl)
    
    ! >> dcp_mean_defl
    dcp_mean_defl_bd(ind_simul)                  = SUM(frq_B_s_nd_bd*realp_mean_defl)
    
    ! >> dcp_mean_nodef_cov_sdfl_defl
    dcp_mean_nodef_cov_sdfl_defl_bd(ind_simul)   = SUM(frq_B_s_nd_bd*realp_mean_nodef_cov_sdfl_defl)
    
    ! >> dcp_mean_sdfl_cov_nodef_defl    
    dcp_mean_sdfl_cov_nodef_defl_bd(ind_simul)   = SUM(frq_B_s_nd_bd*realp_mean_sdfl_cov_nodef_defl)
    
    ! >> dcp_mean_defl_cov_sdfl_nodef
    dcp_mean_defl_cov_sdfl_nodef_bd(ind_simul)   = SUM(frq_B_s_nd_bd*realp_mean_defl_cov_sdfl_nodef)
 
    
    !! ALTERNATIVE DECOMPOSITION OF SPREADS 
    !!
    !! decomposing realpss (q) = E(x(y+z))
    !! 
    !! terms are (see POST_PROCESS)
    !! >> decompose_prod_mean_nodef_alt
    !! >> decompose_prod_mean_indef_alt
    !! >> decompose_cov_defl_sdfl_nodef_alt
    !! >> decompose_cov_defl_sdfl_indef_alt

    ! >> decompose_prod_mean - no default
    decompose_prod_mean_nodef_alt(ind_simul)        = SUM(frq_B_s_nd*realp_prod_mean_nodef_alt)
       
    ! >> decompose_cov_defl_sdfl_nodef_alt
    decompose_cov_defl_sdfl_nodef_alt(ind_simul)    = SUM(frq_B_s_nd*realp_cov_defl_sdfl_nodef_alt)
    
    ! >> decompose_prod_mean - in default
    decompose_prod_mean_indef_alt(ind_simul)        = SUM(frq_B_s_nd*realp_prod_mean_indef_alt)
    
    ! >> decompose_cov_defl_sdfl_indef_alt
    decompose_cov_defl_sdfl_indef_alt(ind_simul)    = SUM(frq_B_s_nd*realp_cov_defl_sdfl_indef_alt)
        

    
    !! ALTERNATIVE DECOMPOSITION OF SPREADS  -- GOOD TIMES
    !!
    !! decomposing realpss (q) = E(x(y+z))
    !! 
    !! terms are (see POST_PROCESS)
    !! >> dcp_prod_mean_nodef_alt_gd
    !! >> dcp_prod_mean_indef_alt_gd
    !! >> dcp_cov_defl_sdfl_nodef_alt_gd
    !! >> dcp_cov_defl_sdfl_indef_alt_gd

    ! >> dcp_prod_mean - no default
    dcp_prod_mean_nodef_alt_gd(ind_simul)        = SUM(frq_B_s_nd_gd*realp_prod_mean_nodef_alt)
       
    ! >> dcp_cov_defl_sdfl_nodef_alt
    dcp_cov_defl_sdfl_nodef_alt_gd(ind_simul)    = SUM(frq_B_s_nd_gd*realp_cov_defl_sdfl_nodef_alt)
    
    ! >> dcp_prod_mean - in default
    dcp_prod_mean_indef_alt_gd(ind_simul)        = SUM(frq_B_s_nd_gd*realp_prod_mean_indef_alt)
    
    ! >> dcp_cov_defl_sdfl_indef_alt
    dcp_cov_defl_sdfl_indef_alt_gd(ind_simul)    = SUM(frq_B_s_nd_gd*realp_cov_defl_sdfl_indef_alt)
      

    
    !! ALTERNATIVE DECOMPOSITION OF SPREADS  -- BAD TIMES
    !!
    !! decomposing realpss (q) = E(x(y+z))
    !! 
    !! terms are (see POST_PROCESS)
    !! >> dcp_prod_mean_nodef_alt_bd
    !! >> dcp_prod_mean_indef_alt_bd
    !! >> dcp_cov_defl_sdfl_nodef_alt_bd
    !! >> dcp_cov_defl_sdfl_indef_alt_bd

    ! >> dcp_prod_mean - no default
    dcp_prod_mean_nodef_alt_bd(ind_simul)        = SUM(frq_B_s_nd_bd*realp_prod_mean_nodef_alt)
       
    ! >> dcp_cov_defl_sdfl_nodef_alt
    dcp_cov_defl_sdfl_nodef_alt_bd(ind_simul)    = SUM(frq_B_s_nd_bd*realp_cov_defl_sdfl_nodef_alt)
    
    ! >> dcp_prod_mean - in default
    dcp_prod_mean_indef_alt_bd(ind_simul)        = SUM(frq_B_s_nd_bd*realp_prod_mean_indef_alt)
    
    ! >> dcp_cov_defl_sdfl_indef_alt
    dcp_cov_defl_sdfl_indef_alt_bd(ind_simul)    = SUM(frq_B_s_nd_bd*realp_cov_defl_sdfl_indef_alt)
    
END SUBROUTINE RUNSIMUL_SIMPLE

END MODULE AUXILIARY
